-- top-level wrapper around ALTGX megafunctions
-- This version allows to cofigure three variations of altgx configuration:
-- single channel; dual-channel via tx_coreclk, dual-channel via explicit bonding

library IEEE;
use     IEEE.Std_Logic_1164.all;

entity altgx_tlk is
generic(
  SIMULATION      : integer :=  0;
  SIMPLE_TLK_WRAP : integer :=  1;  -- enables "Simple" version of TLK2501 TX emulation (no disparity recalculation)
  DATAGEN_MODE    : integer :=  0;  -- 0=regular operation; 1=inject SEND_RESET sequences; 2=inject counter data
  BONDING_MODE    : integer :=  2   -- 0=single-channel; 1=dual channel via tx_coreclk; 2=dual-channel via bonding;
									-- 3=dual-channel via bonding, plus generates 50 MHz ICLK for LSC core
);
port (
	POWER_UP_RST_N    : in  std_logic;
	-- transmitter (to both DAQ and FTK)
	TXD            : in std_logic_vector(15 downto 0);
	TX_EN          : in std_logic;
	TX_ER          : in std_logic;
	TX_CLK         : out std_logic;  -- generated internally, passed to LSC/LDC core
	ICLK           : out std_logic;  -- optionally generates and outputs 50 MHz ICLK
	-- receiver (from DAQ):
	RXD0           : out std_logic_vector(15 downto 0); -- Received parallel data
	RX_CLK0        : out std_logic;  -- Recovered clock.
	RX_LOCKED0     : out std_logic;  -- Receiver locked and synced to incoming data
	RX_ER0         : out std_logic;
	RX_DV0         : out std_logic;
	-- receiver (from FTK):
	RXD1           : out std_logic_vector(15 downto 0); -- Received parallel data
	RX_CLK1        : out std_logic;  -- Recovered clock.
	RX_LOCKED1     : out std_logic;  -- Receiver locked and synced to incoming data
	RX_ER1         : out std_logic;
	RX_DV1         : out std_logic;
	-- Cyclone transceiver ports (connect to pins)
	SLOWCLK       : in std_logic;   -- 50 MHz, altgx reconfiguation
	CALIBCLK      : in std_logic;   -- 100 MHz altgx calibration
	REFCLK0       : in std_logic;   -- 100 MHz, LVDS - altgx clock for DAQ channel MPLL
	REFCLK1       : in std_logic;   -- 100 MHz, LVDS - altgx clock for FTK channel MPLL
	                                -- (should be the same as REFCLK0 to bond the channels)
	GXB_TX0       : out std_logic;
	GXB_RX0       : in std_logic;
	GXB_TX1       : out std_logic;
	GXB_RX1       : in std_logic;
	debug_bits    : out std_logic_vector(3 downto 0);
	LOS0_stable   : in std_logic;
	LOS1_stable	  : in std_logic;
	FTK_SEND_ACK  : in std_logic    -- send special acknowledgement on FTK channel
								    -- in response to FTK_XOFF_ENA programming
   );
end altgx_tlk;

architecture structure of altgx_tlk is
	-- clocks and resets
	signal reset_sig : STD_LOGIC;
	signal tx_clk_sig : STD_LOGIC;
	signal tx_ready_sig : STD_LOGIC;
	
	-- signals shared by both transceivers
	signal pll_locked_sig : STD_LOGIC_VECTOR(0 DOWNTO 0);
	signal pll_areset_sig : STD_LOGIC_VECTOR(0 DOWNTO 0);
	signal tx_digitalreset_sig : STD_LOGIC_VECTOR(0 DOWNTO 0);
	signal rx_analogreset_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal rx_digitalreset_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal rx_datain_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal tx_dataout_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	
	-- data to be transmitted
	signal TXD0    : STD_LOGIC_VECTOR(15 downto 0);
	signal TX_EN0  : STD_LOGIC;
	signal TX_ER0  : STD_LOGIC;
	signal TXD1    : STD_LOGIC_VECTOR(15 downto 0);
	signal TX_EN1  : STD_LOGIC;
	signal TX_ER1  : STD_LOGIC;
	
	-- data received
	signal RXD0_sig : STD_LOGIC_VECTOR(15 downto 0);
	signal RX_ER0_sig : STD_LOGIC;
	signal RX_DV0_sig : STD_LOGIC;
	signal RXD1_sig : STD_LOGIC_VECTOR(15 downto 0);
	signal RX_ER1_sig : STD_LOGIC;
	signal RX_DV1_sig : STD_LOGIC;
	
	-- DAQ signals
	signal tx_datain0_sig : STD_LOGIC_VECTOR(15 DOWNTO 0);
	signal tx_ctrlenable0_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal tx_forcedisp0_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal tx_dispval0_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal rx_clk0_sig : STD_LOGIC;
	signal rx_ready0_sig : STD_LOGIC;
	signal rx_dataout0_sig : STD_LOGIC_VECTOR(15 DOWNTO 0);
	signal rx_ctrldetect0_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal rx_errdetect0_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal rx_byteorderalignstatus0_sig : STD_LOGIC_VECTOR(0 DOWNTO 0);
	signal rx_freqlocked0_sig : STD_LOGIC_VECTOR(0 DOWNTO 0);
	signal rx_analogreset0_sig : STD_LOGIC_VECTOR(0 DOWNTO 0);
	signal rx_digitalreset0_sig : STD_LOGIC_VECTOR(0 DOWNTO 0);
	signal rx_enapatternalign0_sig	: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal rx_syncstatus0_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal rx_patterndetect0_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	
	-- DAQ byte ordering
	signal reset_byteorder0_sig : STD_LOGIC;
	signal rx_ctrldetect0_unord_sig	: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal rx_dataout0_unord_sig		: STD_LOGIC_VECTOR (15 DOWNTO 0);
	signal rx_errdetect0_unord_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal rx_syncstatus0_cmb_sig : STD_LOGIC;

	-- FTK signals
	signal tx_datain1_sig : STD_LOGIC_VECTOR(15 DOWNTO 0);
	signal tx_ctrlenable1_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal tx_forcedisp1_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal tx_dispval1_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal rx_clk1_sig : STD_LOGIC;
	signal rx_ready1_sig : STD_LOGIC;
	signal rx_dataout1_sig : STD_LOGIC_VECTOR(15 DOWNTO 0);
	signal rx_ctrldetect1_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal rx_errdetect1_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal rx_byteorderalignstatus1_sig : STD_LOGIC_VECTOR(0 DOWNTO 0);
	signal rx_freqlocked1_sig : STD_LOGIC_VECTOR(0 DOWNTO 0);
	signal rx_analogreset1_sig : STD_LOGIC_VECTOR(0 DOWNTO 0);
	signal rx_digitalreset1_sig : STD_LOGIC_VECTOR(0 DOWNTO 0);
	signal rx_enapatternalign1_sig	: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal rx_syncstatus1_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	signal rx_patterndetect1_sig : STD_LOGIC_VECTOR(1 DOWNTO 0);
	
	-- FTK byte ordering
	signal reset_byteorder1_sig : STD_LOGIC;
	signal rx_ctrldetect1_unord_sig	: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal rx_dataout1_unord_sig		: STD_LOGIC_VECTOR (15 DOWNTO 0);
	signal rx_errdetect1_unord_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal rx_syncstatus1_cmb_sig : STD_LOGIC;
	
	-- Signals for altgx_reconfig
	signal cal_blk_clk_sig : STD_LOGIC;
	signal reconfig_clk_sig : STD_LOGIC;
	signal busy_sig		: STD_LOGIC ;
	signal reconfig_fromgxb0_sig	: STD_LOGIC_VECTOR (4 DOWNTO 0);
	signal reconfig_fromgxb1_sig	: STD_LOGIC_VECTOR (4 DOWNTO 0);
	signal reconfig_togxb_sig		: STD_LOGIC_VECTOR (3 DOWNTO 0);
	
	-- debug signals (will be synthesized away if not used)
	signal debug_tx : std_logic_vector(1 downto 0);
	signal debug_rx0 : std_logic_vector(1 downto 0);
	signal debug_ord0 : std_logic_vector(1 downto 0);
	signal debug_rx1 : std_logic_vector(1 downto 0);
	signal debug_ord1 : std_logic_vector(1 downto 0);
	
-- altgx_reconfig component (required for receiver channels)
component altgx_reco_single
	PORT
	(
		reconfig_clk		: IN STD_LOGIC ;
		reconfig_fromgxb		: IN STD_LOGIC_VECTOR (4 DOWNTO 0);
		busy		: OUT STD_LOGIC ;
		error		: OUT STD_LOGIC ;
		reconfig_togxb		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0)
	);
end component;

component altgx_reco_mslave
	PORT
	(
		reconfig_clk		: IN STD_LOGIC ;
		reconfig_fromgxb		: IN STD_LOGIC_VECTOR (9 DOWNTO 0);
		busy		: OUT STD_LOGIC ;
		error		: OUT STD_LOGIC ;
		reconfig_togxb		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0)
	);
end component;

component altgx_reco_dual
	PORT
	(
		reconfig_clk		: IN STD_LOGIC ;
		reconfig_fromgxb		: IN STD_LOGIC_VECTOR (4 DOWNTO 0);
		busy		: OUT STD_LOGIC ;
		error		: OUT STD_LOGIC ;
		reconfig_togxb		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0)
	);
end component;

-- BONDING_MODE=0 single-channel transceiver (can be used for HOLA LDC)
component altgx_single
	PORT
	(
		cal_blk_clk		: IN STD_LOGIC ;
		pll_areset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		pll_inclk		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		reconfig_clk		: IN STD_LOGIC ;
		reconfig_togxb		: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_analogreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_datain		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_ctrlenable		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_datain		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		tx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dispval		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_forcedisp		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		pll_locked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		reconfig_fromgxb		: OUT STD_LOGIC_VECTOR (4 DOWNTO 0);
		rx_clkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_ctrldetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_dataout		: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		rx_disperr		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_errdetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_freqlocked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_patterndetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_runningdisp		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_syncstatus		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_clkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dataout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0)
	);
end component;

-- BONDING_MODE=1 master transceiver block that instantiates the PLL
component altgx_single_master
	PORT
	(
		cal_blk_clk		: IN STD_LOGIC ;
		pll_areset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		pll_inclk		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		reconfig_clk		: IN STD_LOGIC ;
		reconfig_togxb		: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_analogreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_datain		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_ctrlenable		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_datain		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		tx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dispval		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_forcedisp		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		pll_locked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		reconfig_fromgxb		: OUT STD_LOGIC_VECTOR (4 DOWNTO 0);
		rx_clkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_ctrldetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_dataout		: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		rx_disperr		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_errdetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_freqlocked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_patterndetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_runningdisp		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_syncstatus		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_clkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dataout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0)
	);
end component;

-- BONDING_MODE=1 slave transceiver block that borrows the PLL from the master
component altgx_single_slave
	PORT
	(
		cal_blk_clk		: IN STD_LOGIC ;
		pll_areset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		pll_inclk		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		reconfig_clk		: IN STD_LOGIC ;
		reconfig_togxb		: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_analogreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_datain		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_coreclk		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_ctrlenable		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_datain		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		tx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dispval		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_forcedisp		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		pll_locked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		reconfig_fromgxb		: OUT STD_LOGIC_VECTOR (4 DOWNTO 0);
		rx_clkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_ctrldetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_dataout		: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		rx_disperr		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_errdetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_freqlocked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_patterndetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_runningdisp		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_syncstatus		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_clkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dataout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0)
	);
end component;

-- BONDING_MODE=2 dual-channel transceiver using two bonded channels
component altgx_dual
	PORT
	(
		cal_blk_clk		: IN STD_LOGIC ;
		pll_areset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		pll_inclk		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		reconfig_clk		: IN STD_LOGIC ;
		reconfig_togxb		: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_analogreset		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_datain		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_digitalreset		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_ctrlenable		: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		tx_datain		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		tx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dispval		: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		tx_forcedisp		: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		coreclkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		pll_locked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		reconfig_fromgxb		: OUT STD_LOGIC_VECTOR (4 DOWNTO 0);
		rx_clkout		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_ctrldetect		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_dataout		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
		rx_disperr		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_errdetect		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_freqlocked		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_patterndetect		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_runningdisp		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_syncstatus		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
		tx_clkout		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_dataout		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0)
	);
end component;

-- BONDING_MODE=3 dual-channel transceiver using two bonded channels
-- This version adds an extra clock output to altgx MPLL (50 MHz ICLK_2)
-- Note that this has to be manually added by hand to altgx_dual_iclk.vhd (grep for "AKHOLA")
component altgx_dual_iclk
	PORT
	(
		cal_blk_clk		: IN STD_LOGIC ;
		pll_areset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		pll_inclk		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		reconfig_clk		: IN STD_LOGIC ;
		reconfig_togxb		: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_analogreset		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_datain		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_digitalreset		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_ctrlenable		: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		tx_datain		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		tx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dispval		: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		tx_forcedisp		: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		coreclkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		iclkout         : OUT STD_LOGIC;
		pll_locked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		reconfig_fromgxb		: OUT STD_LOGIC_VECTOR (4 DOWNTO 0);
		rx_clkout		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_ctrldetect		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_dataout		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
		rx_disperr		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_errdetect		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_freqlocked		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_patterndetect		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_runningdisp		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_syncstatus		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
		tx_clkout		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_dataout		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0)
	);
end component;

-- reset FSM components
component altgx_reset_tx
	port (
		clk             : in  std_logic;
		POWER_UP_RST_N  : in  std_logic;
		pll_locked      : in  std_logic_vector (0 downto 0);
		pll_areset      : out std_logic_vector (0 downto 0);
		tx_digitalreset : out std_logic_vector (0 downto 0);
		tx_ready        : out std_logic; -- for rx reset machine
		debug_st        : out std_logic_vector (1 downto 0) -- will be synthesized away if not used
	);
end component;
component altgx_reset_rx is
	generic (
		TLTD : integer := 401
	);
	port (
		clk             : in  std_logic;
		POWER_UP_RST_N  : in  std_logic;
		tx_ready        : in  std_logic; -- rx is only initialized after tx is up
		busy            : in  std_logic;
		rx_analogreset  : out std_logic_vector (0 downto 0);
		rx_freqlocked   : in  std_logic_vector (0 downto 0);
		rx_digitalreset : out std_logic_vector (0 downto 0);
		rx_ready        : out std_logic; -- for byte ordering block
		debug_st        : out std_logic_vector (1 downto 0) -- will be synthesized away if not used
	);
end component;

-- TLK-emulating wrapper for transmitter channel
component tlk_wrap_tx is
	PORT (
		-- reset
		RESET : IN STD_LOGIC;
		-- from LSC/LDC core
		TX_CLK : IN STD_LOGIC;
		TX_EN  : IN STD_LOGIC;
		TX_ER  : IN STD_LOGIC;
		TD     : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		-- to ALTGX
		tx_datain     : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		tx_ctrlenable : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
		tx_dispval    : OUT STD_LOGIC_VECTOR(1 DOWNTO 0)
	);
end component;

-- "Simple" version of TLK-emulating wrapper for transmitter channel
-- (this one doesn't re-compute running disparity to select the correct IDLE)
component tlk_wrap_tx_simple is
	PORT (
		-- reset
		RESET : IN STD_LOGIC;
		-- from LSC/LDC core
		TX_CLK : IN STD_LOGIC;
		TX_EN  : IN STD_LOGIC;
		TX_ER  : IN STD_LOGIC;
		TD     : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		-- to ALTGX
		tx_datain     : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		tx_ctrlenable : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
		tx_dispval    : OUT STD_LOGIC_VECTOR(1 DOWNTO 0)
	);
end component;

-- Byte ordering block (based on /I1/ or /I2/) for receiver channel
component tlk_byte_order is
	PORT (
		clk             : in  std_logic;  -- recovered clock
		reset           : in  std_logic;  -- power-up reset (active-high)
		rx_syncstatus   : in  std_logic;  -- is byte boundary locked?
		-- data from the serdes link:
		datain          : in  std_logic_vector(15 downto 0);
		ctrlin          : in  std_logic_vector(1 downto 0);
		errin           : in  std_logic_vector(1 downto 0);
		-- pipelined and byte-ordered output:
		dataout         : out std_logic_vector(15 downto 0);
		ctrlout         : out std_logic_vector(1 downto 0);
		errout          : out std_logic_vector(1 downto 0);
		status          : out std_logic_vector(0 downto 0);
		debug_st        : out std_logic_vector (1 downto 0) -- will be synthesized away if not used
	);
end component;

-- TLK-emulating wrapper for receiver channel
component tlk_wrap_rx is
	PORT (
		-- reset
		RESET : IN STD_LOGIC;
		-- from ALTGX
		rx_clkout : IN STD_LOGIC;
		rx_dataout : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		rx_ctrldetect : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
		rx_errdetect  : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
		rx_byteorderalignstatus : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
		-- to LSC/LDC core
		RXD : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		RX_ER : OUT STD_LOGIC;
		RX_DV : OUT STD_LOGIC
		);
end component;

-- simple simulator for SEND_RESET LSC state (16 IDLEs + 1 CARRIER + 1 RRES data)
component lsc_rres_simulator is
	port (
		POWER_UP_RST_N  : in  std_logic;
		TX_CLK          : in  std_logic;  -- TX_CLK from altgx
		TXD             : out std_logic_vector(15 downto 0);
		TX_EN           : out std_logic;
		TX_ER           : out std_logic
	);
end component;

-- simple simulator for SEND_RESET LSC state (16 IDLEs + 1 CARRIER + 1 RRES data)
component lsc_count_simulator is
generic (
	NDATA : integer := 65535  -- default size of counter
);
port (
	POWER_UP_RST_N  : in  std_logic;
	-- outputs to send
	TX_CLK          : in  std_logic;  -- TX_CLK from altgx
	TXD             : out std_logic_vector(15 downto 0);
	TX_EN           : out std_logic;
	TX_ER           : out std_logic;
	-- inputs to receive
	RX_CLK          : in  std_logic;  -- RX_CLK from altgx
	RXD             : in  std_logic_vector(15 downto 0);
	RX_DV           : in  std_logic;
	RX_ER           : in  std_logic;
	valid          : out std_logic
);
end component;

	-- Special character code values (rightmost bit is LSB and is sent first)
	CONSTANT ONES8 : STD_LOGIC_VECTOR := "11111111";
	CONSTANT ZERO8 : STD_LOGIC_VECTOR := "00000000";
	CONSTANT K23p7 : STD_LOGIC_VECTOR := "11110111";
	CONSTANT K28p5 : STD_LOGIC_VECTOR := "10111100";
	CONSTANT K30p7 : STD_LOGIC_VECTOR := "11111110";
	CONSTANT D5p6  : STD_LOGIC_VECTOR := "11000101";
	CONSTANT D16p2 : STD_LOGIC_VECTOR := "01010000";
	-- TLK-2501 keywords (rightmost byte is LSB and is sent first)
	CONSTANT IDLE1 : STD_LOGIC_VECTOR   := D5p6 & K28p5;    --K25.8, D5.6
	CONSTANT IDLE2 : STD_LOGIC_VECTOR   := D16p2 & K28p5;   --K28.5, D16.2
	CONSTANT CARRIER : STD_LOGIC_VECTOR := K23p7 & K23p7;   --K23.7, K23.7
	CONSTANT GXERR : STD_LOGIC_VECTOR   := K30p7 & K30p7;   --K30.7, K30.7
	CONSTANT ONES16 : STD_LOGIC_VECTOR  := ONES8 & ONES8;   -- dummy all-ones pattern

begin
	-- parallel assignments of general input ports
	reset_sig <= not POWER_UP_RST_N;
	reconfig_clk_sig <= SLOWCLK;
	cal_blk_clk_sig  <= CALIBCLK;

	-- parallel assignments of general output ports
	RX_LOCKED0 <= rx_byteorderalignstatus0_sig(0);
	-- RX_LOCKED1 is assigned at a later stage, because this signal may be re-assigned for debugging purposes
	TX_CLK        <= tx_clk_sig;
	RX_CLK0       <= rx_clk0_sig;
	RX_CLK1       <= rx_clk1_sig;
	
	-- other parallel assignments
	rx_syncstatus0_cmb_sig <= rx_syncstatus0_sig(0) or rx_syncstatus0_sig(1);
	rx_syncstatus1_cmb_sig <= rx_syncstatus1_sig(0) or rx_syncstatus1_sig(1);

-------------------------------------------------------------------------
-------------------------------------------------------------------------
-- Altera transceiver & transceiver reconfiguration IP cores
-------------------------------------------------------------------------
-------------------------------------------------------------------------

	-- BONDING_MODE=0 single-channel transceiver (can be used for HOLA LDC)
	BOND0 : if BONDING_MODE = 0 generate
		ICLK <= '0';   -- 50 MHZ ICLK generation is only supported in BONDING_MODE=3
		
		-- altgx_reconfig instance
		reco_inst : altgx_reco_single PORT MAP (
			reconfig_clk	 => reconfig_clk_sig,
			reconfig_fromgxb => reconfig_fromgxb0_sig,
			busy	 => busy_sig,
			error	 => open,
			reconfig_togxb	 => reconfig_togxb_sig
		);
		
		-- altgx instance (master transmitter and DAQ receiver)
		gx0_inst : altgx_single PORT MAP (
			cal_blk_clk		 => cal_blk_clk_sig,
			pll_areset		 => pll_areset_sig,
			pll_inclk(0)	 => REFCLK0,
			reconfig_clk	 => reconfig_clk_sig,
			reconfig_togxb	 => reconfig_togxb_sig,
			rx_analogreset	 => rx_analogreset0_sig,
			rx_datain(0)	 => GXB_RX0,
			rx_digitalreset	 => rx_digitalreset0_sig,
			tx_ctrlenable => tx_ctrlenable0_sig,
			tx_datain => tx_datain0_sig,
			tx_digitalreset	 => tx_digitalreset_sig,
			tx_dispval => tx_dispval0_sig,
			tx_forcedisp	 => tx_forcedisp0_sig,  -- '00' for simple simulation, '11' otherwise
			pll_locked		 => pll_locked_sig,
			reconfig_fromgxb => reconfig_fromgxb0_sig,
			rx_clkout(0)	 => rx_clk0_sig,
			rx_ctrldetect	 => rx_ctrldetect0_unord_sig,
			rx_dataout		 => rx_dataout0_unord_sig,
			rx_disperr		 => open,
			rx_errdetect	 => rx_errdetect0_unord_sig,
			rx_freqlocked	 => rx_freqlocked0_sig,
			rx_patterndetect => open,
			rx_runningdisp	 => open,
			rx_syncstatus	 => rx_syncstatus0_sig,
			tx_clkout(0)	 => tx_clk_sig,
			tx_dataout(0)	 => GXB_TX0
		);	
	
	end generate;

	-- BONDING_MODE=1 master transceiver block that instantiates the PLL
	BOND1 : if BONDING_MODE = 1 generate
		ICLK <= '0';   -- 50 MHZ ICLK generation is only supported in BONDING_MODE=3
	
		-- altgx_reconfig instance
		reco_inst : altgx_reco_mslave PORT MAP (
			reconfig_clk	 => reconfig_clk_sig,
			reconfig_fromgxb(4 downto 0) => reconfig_fromgxb0_sig,
			reconfig_fromgxb(9 downto 5) => reconfig_fromgxb1_sig,
			busy	 => busy_sig,
			error	 => open,
			reconfig_togxb	 => reconfig_togxb_sig
		);
		
		-- altgx instance (master transmitter and DAQ receiver)
		gx0_inst : altgx_single_master PORT MAP (
			cal_blk_clk		 => cal_blk_clk_sig,
			pll_areset		 => pll_areset_sig,
			pll_inclk(0)	 => REFCLK0,
			reconfig_clk	 => reconfig_clk_sig,
			reconfig_togxb	 => reconfig_togxb_sig,
			rx_analogreset	 => rx_analogreset0_sig,
			rx_datain(0)	 => GXB_RX0,
			rx_digitalreset	 => rx_digitalreset0_sig,
			tx_ctrlenable => tx_ctrlenable0_sig,
			tx_datain => tx_datain0_sig,
			tx_digitalreset	 => tx_digitalreset_sig,
			tx_dispval => tx_dispval0_sig,
			tx_forcedisp	 => tx_forcedisp0_sig,  -- '00' for simple simulation, '11' otherwise
			pll_locked		 => pll_locked_sig,
			reconfig_fromgxb => reconfig_fromgxb0_sig,
			rx_clkout(0)	 => rx_clk0_sig,
			rx_ctrldetect	 => rx_ctrldetect0_unord_sig,
			rx_dataout		 => rx_dataout0_unord_sig,
			rx_disperr		 => open,
			rx_errdetect	 => rx_errdetect0_unord_sig,
			rx_freqlocked	 => rx_freqlocked0_sig,
			rx_patterndetect => open,
			rx_runningdisp	 => open,
			rx_syncstatus	 => rx_syncstatus0_sig,
			tx_clkout(0)	 => tx_clk_sig,
			tx_dataout(0)	 => GXB_TX0
		);	
			
		-- altgx instance (slave transmitter and FTK receiver)
		gx1_inst : altgx_single_slave PORT MAP (
			cal_blk_clk		 => cal_blk_clk_sig,
			pll_areset		 => pll_areset_sig,
			pll_inclk(0)	 => REFCLK1,
			reconfig_clk	 => reconfig_clk_sig,
			reconfig_togxb	 => reconfig_togxb_sig,
			rx_analogreset	 => rx_analogreset1_sig,
			rx_datain(0)	 => GXB_RX1,
			rx_digitalreset	 => rx_digitalreset1_sig,
			tx_coreclk(0)     => tx_clk_sig,
			tx_ctrlenable => tx_ctrlenable1_sig,
			tx_datain => tx_datain1_sig,
			tx_digitalreset	 => tx_digitalreset_sig,
			tx_dispval => tx_dispval1_sig,
			tx_forcedisp	 => tx_forcedisp1_sig,  -- '00' for simple simulation, '11' otherwise
			pll_locked		 => open,
			reconfig_fromgxb => reconfig_fromgxb1_sig,
			rx_clkout(0)	 => rx_clk1_sig,
			rx_ctrldetect	 => rx_ctrldetect1_unord_sig,
			rx_dataout		 => rx_dataout1_unord_sig,
			rx_disperr		 => open,
			rx_errdetect	 => rx_errdetect1_unord_sig,
			rx_freqlocked	 => rx_freqlocked1_sig,
			rx_patterndetect => open,
			rx_runningdisp	 => open,
			rx_syncstatus	 => rx_syncstatus1_sig,
			tx_clkout		 => open,
			tx_dataout(0)	 => GXB_TX1
		);
	
	end generate;

	-- BONDING_MODE=2 dual-channel transceiver using two bonded channels
	BOND2 : if BONDING_MODE = 2 generate
		ICLK <= '0';   -- 50 MHZ ICLK generation is only supported in BONDING_MODE=3
		
		-- altgx_reconfig instance
		reco_inst : altgx_reco_dual PORT MAP (
			reconfig_clk	 => reconfig_clk_sig,
			reconfig_fromgxb	 => reconfig_fromgxb0_sig,
			busy		 => busy_sig,
			error	 => open,
			reconfig_togxb	 => reconfig_togxb_sig
		);
		
		-- resets
		rx_analogreset_sig(0) <= rx_analogreset0_sig(0);
		rx_analogreset_sig(1) <= rx_analogreset1_sig(0);
		rx_digitalreset_sig(0) <= rx_digitalreset0_sig(0);
		rx_digitalreset_sig(1) <= rx_digitalreset1_sig(0);
		-- serial receiver inputs
		rx_datain_sig(0) <= GXB_RX0;
		rx_datain_sig(1) <= GXB_RX1;
		-- serial transmitter outputs
		GXB_TX0 <= tx_dataout_sig(0);
		GXB_TX1 <= tx_dataout_sig(1);
	
		-- altgx instance (bonded transmitters and two receivers)
		gx0_inst : altgx_dual PORT MAP (
			cal_blk_clk	 => cal_blk_clk_sig,
			pll_areset	 => pll_areset_sig,
			pll_inclk(0)	 => REFCLK0,
			reconfig_clk	 => reconfig_clk_sig,
			reconfig_togxb	 => reconfig_togxb_sig,
			rx_analogreset	 => rx_analogreset_sig,
			rx_datain	 => rx_datain_sig,
			rx_digitalreset	 => rx_digitalreset_sig,
			tx_ctrlenable(1 DOWNTO 0)	 => tx_ctrlenable0_sig,
			tx_ctrlenable(3 DOWNTO 2)	 => tx_ctrlenable1_sig,
			tx_datain(15 DOWNTO 0)	 => tx_datain0_sig,
			tx_datain(31 DOWNTO 16)	 => tx_datain1_sig,
			tx_digitalreset	 => tx_digitalreset_sig,
			tx_dispval(1 DOWNTO 0)	 => tx_dispval0_sig,
			tx_dispval(3 DOWNTO 2)	 => tx_dispval1_sig,
			tx_forcedisp(1 DOWNTO 0)	 => tx_forcedisp0_sig,
			tx_forcedisp(3 DOWNTO 2)	 => tx_forcedisp1_sig,
			coreclkout(0)	 => tx_clk_sig,
			pll_locked	 => pll_locked_sig,
			reconfig_fromgxb	 => reconfig_fromgxb0_sig,
			rx_clkout(0)	 => rx_clk0_sig,
			rx_clkout(1)	 => rx_clk1_sig,
			rx_ctrldetect(1 DOWNTO 0)	 => rx_ctrldetect0_unord_sig,
			rx_ctrldetect(3 DOWNTO 2)	 => rx_ctrldetect1_unord_sig,
			rx_dataout(15 DOWNTO 0)	 => rx_dataout0_unord_sig,
			rx_dataout(31 DOWNTO 16)	 => rx_dataout1_unord_sig,
			rx_disperr   => open,
			rx_errdetect(1 DOWNTO 0)	 => rx_errdetect0_unord_sig,
			rx_errdetect(3 DOWNTO 2)	 => rx_errdetect1_unord_sig,
			rx_freqlocked(0 DOWNTO 0)	 => rx_freqlocked0_sig,
			rx_freqlocked(1 DOWNTO 1)	 => rx_freqlocked1_sig,
			rx_patterndetect	 => open,
			rx_runningdisp	 => open,
			rx_syncstatus(1 DOWNTO 0)	 => rx_syncstatus0_sig,
			rx_syncstatus(3 DOWNTO 2)	 => rx_syncstatus1_sig,
			tx_clkout	    => open,  -- use coreclkout instead (bonded channels)
			tx_dataout	 => tx_dataout_sig
		);
		
	end generate;
	
	-- BONDING_MODE=3 dual-channel transceiver using two bonded channels
	-- This version adds an extra clock output to altgx MPLL (50 MHz ICLK_2)
	-- Note that this has to be manually added by hand to altgx_dual_iclk.vhd (grep for "AKHOLA")
	BOND3 : if BONDING_MODE = 3 generate
		
		-- altgx_reconfig instance
		reco_inst : altgx_reco_dual PORT MAP (
			reconfig_clk	 => reconfig_clk_sig,
			reconfig_fromgxb	 => reconfig_fromgxb0_sig,
			busy		 => busy_sig,
			error	 => open,
			reconfig_togxb	 => reconfig_togxb_sig
		);
		
		-- resets
		rx_analogreset_sig(0) <= rx_analogreset0_sig(0);
		rx_analogreset_sig(1) <= rx_analogreset1_sig(0);
		rx_digitalreset_sig(0) <= rx_digitalreset0_sig(0);
		rx_digitalreset_sig(1) <= rx_digitalreset1_sig(0);
		-- serial receiver inputs
		rx_datain_sig(0) <= GXB_RX0;
		rx_datain_sig(1) <= GXB_RX1;
		-- serial transmitter outputs
		GXB_TX0 <= tx_dataout_sig(0);
		GXB_TX1 <= tx_dataout_sig(1);
	
		-- altgx instance (bonded transmitters and two receivers)
		gx0_inst : altgx_dual_iclk PORT MAP (
			cal_blk_clk	 => cal_blk_clk_sig,
			pll_areset	 => pll_areset_sig,
			pll_inclk(0)	 => REFCLK0,
			reconfig_clk	 => reconfig_clk_sig,
			reconfig_togxb	 => reconfig_togxb_sig,
			rx_analogreset	 => rx_analogreset_sig,
			rx_datain	 => rx_datain_sig,
			rx_digitalreset	 => rx_digitalreset_sig,
			tx_ctrlenable(1 DOWNTO 0)	 => tx_ctrlenable0_sig,
			tx_ctrlenable(3 DOWNTO 2)	 => tx_ctrlenable1_sig,
			tx_datain(15 DOWNTO 0)	 => tx_datain0_sig,
			tx_datain(31 DOWNTO 16)	 => tx_datain1_sig,
			tx_digitalreset	 => tx_digitalreset_sig,
			tx_dispval(1 DOWNTO 0)	 => tx_dispval0_sig,
			tx_dispval(3 DOWNTO 2)	 => tx_dispval1_sig,
			tx_forcedisp(1 DOWNTO 0)	 => tx_forcedisp0_sig,
			tx_forcedisp(3 DOWNTO 2)	 => tx_forcedisp1_sig,
			coreclkout(0)	 => tx_clk_sig,
			iclkout          => ICLK,
			pll_locked	 => pll_locked_sig,
			reconfig_fromgxb	 => reconfig_fromgxb0_sig,
			rx_clkout(0)	 => rx_clk0_sig,
			rx_clkout(1)	 => rx_clk1_sig,
			rx_ctrldetect(1 DOWNTO 0)	 => rx_ctrldetect0_unord_sig,
			rx_ctrldetect(3 DOWNTO 2)	 => rx_ctrldetect1_unord_sig,
			rx_dataout(15 DOWNTO 0)	 => rx_dataout0_unord_sig,
			rx_dataout(31 DOWNTO 16)	 => rx_dataout1_unord_sig,
			rx_disperr   => open,
			rx_errdetect(1 DOWNTO 0)	 => rx_errdetect0_unord_sig,
			rx_errdetect(3 DOWNTO 2)	 => rx_errdetect1_unord_sig,
			rx_freqlocked(0 DOWNTO 0)	 => rx_freqlocked0_sig,
			rx_freqlocked(1 DOWNTO 1)	 => rx_freqlocked1_sig,
			rx_patterndetect	 => open,
			rx_runningdisp	 => open,
			rx_syncstatus(1 DOWNTO 0)	 => rx_syncstatus0_sig,
			rx_syncstatus(3 DOWNTO 2)	 => rx_syncstatus1_sig,
			tx_clkout	    => open,  -- use coreclkout instead (bonded channels)
			tx_dataout	 => tx_dataout_sig
		);
		
	end generate;
-------------------------------------------------------------------------
-------------------------------------------------------------------------
-- Custom transceiver reset FSM implementing Altera recommendations
-------------------------------------------------------------------------
-------------------------------------------------------------------------

	-- transmitter reset fsm instance
	tx_rst : altgx_reset_tx PORT MAP (
		clk              => CALIBCLK, --SLOWCLK
		POWER_UP_RST_N   => POWER_UP_RST_N,
		pll_locked       => pll_locked_sig,
		pll_areset       => pll_areset_sig,
		tx_digitalreset  => tx_digitalreset_sig,
		tx_ready         => tx_ready_sig,
		debug_st         => debug_tx
	);

	-- receiver reset fsm instances; LTD wait time is reduced for simulation
	RSTSIM : if SIMULATION = 1 generate
		rx0_rst : altgx_reset_rx 
			GENERIC MAP (
				TLTD => 4
			)
			PORT MAP (
				clk              => CALIBCLK, --SLOWCLK
				POWER_UP_RST_N   => POWER_UP_RST_N,
				tx_ready         => tx_ready_sig,
				busy             => busy_sig,
				rx_analogreset   => rx_analogreset0_sig,
				rx_freqlocked    => rx_freqlocked0_sig,
				rx_digitalreset  => rx_digitalreset0_sig,
				rx_ready         => rx_ready0_sig,
				debug_st         => debug_rx0
			);
			
		-- debug bits
		debug_bits(3 downto 2) <= debug_rx0;
		debug_bits(1 downto 0) <= debug_tx;
		
		rx1_rst : altgx_reset_rx 
			GENERIC MAP (
				TLTD => 4
			)
			PORT MAP (
				clk              => CALIBCLK, --SLOWCLK
				POWER_UP_RST_N   => POWER_UP_RST_N,
				tx_ready         => tx_ready_sig,
				busy             => busy_sig,
				rx_analogreset   => rx_analogreset1_sig,
				rx_freqlocked    => rx_freqlocked1_sig,
				rx_digitalreset  => rx_digitalreset1_sig,
				rx_ready         => rx_ready1_sig,
				debug_st         => open
			);
	end generate;
	RSTNOSIM : if SIMULATION = 0 generate
		rx0_rst : altgx_reset_rx 
			GENERIC MAP (
				TLTD => 510
			)
			PORT MAP (
				clk              => CALIBCLK, --SLOWCLK
				POWER_UP_RST_N   => POWER_UP_RST_N,
				tx_ready         => tx_ready_sig,
				busy             => busy_sig,
				rx_analogreset   => rx_analogreset0_sig,
				rx_freqlocked    => rx_freqlocked0_sig,
				rx_digitalreset  => rx_digitalreset0_sig,
				rx_ready         => rx_ready0_sig,
				debug_st         => open
			);

		-- debug bits (assign any internal signal to debug_bits, and then map them to LRL)
		debug_bits <= "0000";

		rx1_rst : altgx_reset_rx 
			GENERIC MAP (
				TLTD => 510
			)
			PORT MAP (
				clk              => CALIBCLK, --SLOWCLK,
				POWER_UP_RST_N   => POWER_UP_RST_N,
				tx_ready         => tx_ready_sig,
				busy             => busy_sig,
				rx_analogreset   => rx_analogreset1_sig,
				rx_freqlocked    => rx_freqlocked1_sig,
				rx_digitalreset  => rx_digitalreset1_sig,
				rx_ready         => rx_ready1_sig,
				debug_st         => open
			);
	end generate;
	
-------------------------------------------------------------------------
-------------------------------------------------------------------------
-- DAQ transceiver wrappers
-------------------------------------------------------------------------
-------------------------------------------------------------------------
	-- wrap ALTGX DAQ transmitter channel to emulate TLK2501
	-- DAQ always gets the data directly from the HOLA core:
	TX_EN0 <= TX_EN;
	TX_ER0 <= TX_ER;
	TXD0   <= TXD;
	
	TX0SIMPLE : if SIMPLE_TLK_WRAP = 1 generate
		tx0_wrap : tlk_wrap_tx_simple
		port map (
			-- inputs (from LSC core)
			RESET         => reset_sig,
			TX_CLK        => tx_clk_sig,
			TX_EN         => TX_EN0,
			TX_ER         => TX_ER0,
			TD            => TXD0,
			-- outputs (to altgx)
			tx_datain     => tx_datain0_sig,
			tx_ctrlenable => tx_ctrlenable0_sig,
			tx_dispval    => tx_dispval0_sig
		);
		tx_forcedisp0_sig <= "00";
	end generate;
	TX0COMPLEX : if SIMPLE_TLK_WRAP = 0 generate
		tx0_wrap : tlk_wrap_tx
		port map (
			-- inputs (from LSC core)
			RESET         => reset_sig,
			TX_CLK        => tx_clk_sig,
			TX_EN         => TX_EN0,
			TX_ER         => TX_ER0,
			TD            => TXD0,
			-- outputs (to altgx)
			tx_datain     => tx_datain0_sig,
			tx_ctrlenable => tx_ctrlenable0_sig,
			tx_dispval    => tx_dispval0_sig
		);
		tx_forcedisp0_sig <= "11";
	end generate;

	-- synchronize combinational reset signals for byte ordering block
	rx0_reset : process(CALIBCLK,POWER_UP_RST_N) --SLOWCLK
	begin
	if POWER_UP_RST_N = '0' then  -- asynchronous reset (active high)
		reset_byteorder0_sig <= '1';
	elsif CALIBCLK'event and CALIBCLK = '1' then  -- rising clock edge
		reset_byteorder0_sig <= (not rx_ready0_sig) or (LOS0_stable);
	end if;
	end process rx0_reset;
	
	-- Byte ordering block (based on /I1/ or /I2/) for receiver channel (DAQ)
	rx0_ord : tlk_byte_order
	port map (
		clk             => rx_clk0_sig,
		reset           => reset_byteorder0_sig,
		rx_syncstatus   => rx_syncstatus0_cmb_sig,
		-- data from the serdes link:
		datain          => rx_dataout0_unord_sig,
		ctrlin          => rx_ctrldetect0_unord_sig,
		errin           => rx_errdetect0_unord_sig,
		-- pipelined and byte-ordered output:
		dataout         => rx_dataout0_sig,
		ctrlout         => rx_ctrldetect0_sig,
		errout          => rx_errdetect0_sig,
		status          => rx_byteorderalignstatus0_sig,
		debug_st        => open --debug_ord0
	);
	
	-- wrap ALTGX receiver channel (DAQ) to emulate TLK2501
	rx0_wrap : tlk_wrap_rx
	port map (
		RESET           => reset_byteorder0_sig,
		-- inputs (from altgx)
		rx_clkout       => rx_clk0_sig,
		rx_dataout      => rx_dataout0_sig,
		rx_ctrldetect   => rx_ctrldetect0_sig,
		rx_errdetect    => rx_errdetect0_sig,
		rx_byteorderalignstatus => rx_byteorderalignstatus0_sig,
		-- outputs (to LSC core)
		RXD             => RXD0_sig,
		RX_ER           => RX_ER0_sig,
		RX_DV           => RX_DV0_sig
	);
	RXD0 <= RXD0_sig;
	RX_ER0 <= RX_ER0_sig;
	RX_DV0 <= RX_DV0_sig;
	
-------------------------------------------------------------------------
-------------------------------------------------------------------------
-- FTK transceiver wrappers
-------------------------------------------------------------------------
-------------------------------------------------------------------------

	-- wrap ALTGX DAQ transmitter channel to emulate TLK2501
	-- If FTK_SEND_ACK='1', we send the error code K30.7 K30.7 to FTK
	-- to signify reception of the FTK_XOFF_ENA pattern.
	TX0D : if DATAGEN_MODE = 0 generate
		RX_LOCKED1 <= rx_byteorderalignstatus1_sig(0);
		TX_EN1 <= TX_EN WHEN FTK_SEND_ACK='0' ELSE '1';
		TX_ER1 <= TX_ER WHEN FTK_SEND_ACK='0' ELSE '1';
		TXD1   <= TXD;
	end generate;

	-- special mode: generates and directly sends SEND_RESET sequence
	-- use this to debug transceiver locking issues - whether they are caused by 
	-- bad timing in the FPGA LSC-core fabric
	TX1D : if DATAGEN_MODE = 1 generate
		RX_LOCKED1 <= rx_byteorderalignstatus1_sig(0);
		rres_sim : lsc_rres_simulator PORT MAP (
			POWER_UP_RST_N => POWER_UP_RST_N,
			TX_CLK         => tx_clk_sig,
			TXD            => TXD1,
			TX_EN          => TX_EN1,
			TX_ER          => TX_ER1
		);	
	end generate;

	-- special mode: generates counter patterns and sends them to the transmit channel1
	-- the receive channel1 receives the same patterns and validates them
	-- (output to RX_LOCKED1 port)
	TX2D : if DATAGEN_MODE = 2 generate
		count_sim : lsc_count_simulator PORT MAP (
			POWER_UP_RST_N => POWER_UP_RST_N,
			TX_CLK         => tx_clk_sig,
			TXD            => TXD1,
			TX_EN          => TX_EN1,
			TX_ER          => TX_ER1,
			RX_CLK         => rx_clk1_sig,
			RXD            => RXD1_sig,
			RX_DV          => RX_DV1_sig,
			RX_ER          => RX_ER1_sig,
			valid          => RX_LOCKED1  -- LED used for debugging!
		);	
	end generate;
	
	TX1SIMPLE : if SIMPLE_TLK_WRAP = 1 generate
		tx1_wrap : tlk_wrap_tx_simple
		port map (
			-- inputs (from LSC core)
			RESET         => reset_sig,
			TX_CLK        => tx_clk_sig,
			TX_EN         => TX_EN1,
			TX_ER         => TX_ER1,
			TD            => TXD1,
			-- outputs (to altgx)
			tx_datain     => tx_datain1_sig,
			tx_ctrlenable => tx_ctrlenable1_sig,
			tx_dispval    => tx_dispval1_sig
		);
		tx_forcedisp1_sig <= "00";
	end generate;
	TX1COMPLEX : if SIMPLE_TLK_WRAP = 0 generate
		tx1_wrap : tlk_wrap_tx
		port map (
			-- inputs (from LSC core)
			RESET         => reset_sig,
			TX_CLK        => tx_clk_sig,
			TX_EN         => TX_EN1,
			TX_ER         => TX_ER1,
			TD            => TXD1,
			-- outputs (to altgx)
			tx_datain     => tx_datain1_sig,
			tx_ctrlenable => tx_ctrlenable1_sig,
			tx_dispval    => tx_dispval1_sig
		);
		tx_forcedisp1_sig <= "11";
	end generate;
	
	-- synchronize combinational reset signals for byte ordering block
	rx1_reset : process(CALIBCLK,POWER_UP_RST_N) -- SLOWCLK
	begin
	if POWER_UP_RST_N = '0' then  -- asynchronous reset (active high)
		reset_byteorder1_sig <= '1';
	elsif CALIBCLK'event and CALIBCLK = '1' then  -- rising clock edge
		reset_byteorder1_sig <= (not rx_ready1_sig) or (LOS1_stable);
	end if;
	end process rx1_reset;
	
	-- Byte ordering block (based on /I1/ or /I2/) for receiver channel (FTK)
	rx1_ord : tlk_byte_order
	port map (
		clk             => rx_clk1_sig,
		reset           => reset_byteorder1_sig,
		rx_syncstatus   => rx_syncstatus1_cmb_sig,
		-- data from the serdes link:
		datain          => rx_dataout1_unord_sig,
		ctrlin          => rx_ctrldetect1_unord_sig,
		errin           => rx_errdetect1_unord_sig,
		-- pipelined and byte-ordered output:
		dataout         => rx_dataout1_sig,
		ctrlout         => rx_ctrldetect1_sig,
		errout          => rx_errdetect1_sig,
		status          => rx_byteorderalignstatus1_sig,
		debug_st        => open
	);
	
	-- wrap ALTGX receiver channel (FTK) to emulate TLK2501
	rx1_wrap : tlk_wrap_rx
	port map (
		RESET           => reset_byteorder1_sig,
		-- inputs (from altgx)
		rx_clkout       => rx_clk1_sig,
		rx_dataout      => rx_dataout1_sig,
		rx_ctrldetect   => rx_ctrldetect1_sig,
		rx_errdetect    => rx_errdetect1_sig,
		rx_byteorderalignstatus => rx_byteorderalignstatus1_sig,
		-- outputs (to LSC core)
		RXD             => RXD1_sig,
		RX_ER           => RX_ER1_sig,
		RX_DV           => RX_DV1_sig
	);
	RXD1 <= RXD1_sig;
	RX_ER1 <= RX_ER1_sig;
	RX_DV1 <= RX_DV1_sig;

end structure;
