-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--
--  File name : my_text.vhd
--  Title     : Text interpretation package for S-LINK FEMB 
--  Purpose   : Text interpretation and conversion routines for FEMB
--  Author(s) : Pjotr Kapusta, 
--              Erik van der Bij - CERN Geneva
--
--  Notes     : Package also used in ROMB
--
-------------------------------------------------------------------------------
-- Version | Author |  Mod.Date  |          Changes made
-------------------------------------------------------------------------------
--   1.00  |   PK   |    1996    | Original coding
--   2.01  |   EB   | 11-05-2000 | v32_to_i modified to not use to_integer
--   2.02  |   EB   | 15-05-2000 | added note package used in ROMB
--         |        |            |
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_textio.all;
use std.textio.all;

--
-- *********************************************************************

package MY_TEXT is

  type COMMAND is array (natural range <>) of line;



  procedure get_token(in_line  : inout line;
                      out_line : out   line;
                      nmb      : inout integer);

--

  procedure lstrcmpn(in_line : inout line;
                     sref    : in    string;
                     n       : in    positive;
                     ok      : inout boolean); 

  procedure write_line(file f  :       text;
                       in_line : inout line);

--

  procedure get_command(file f :       text;
                        comm   : inout COMMAND;
                        nmb    : out   integer);

--
  procedure print_string(file f :    text;
                         s      : in string);
--
  procedure print_string_time(file f :    text;
                              s      : in string;
                              t      : in time);
--
  procedure print_s_vector_time(file f :    text;
                                s      : in string;
                                v      : in std_logic_vector;
                                t      : in time); 
--
  function strcmpn(s1, s2 : in string;
                   n      : in integer) return boolean; 

--
  function up_down(s    : string) return boolean;
--
  function strlen(s     : string) return integer;
--
  function slogic(S     : string) return std_logic;
--
  function v32_to_i(dv  : std_logic_vector) return integer;
--
  function i_to_v32(val : integer) return std_logic_vector;
--
  procedure hex_read(ll  : inout line;
                     val : out   integer);

end MY_TEXT;
-- *********************************************************************

package body MY_TEXT is

-- Writes line "line" into output file "f"
  procedure write_line(file f  :       text;
                       in_line : inout line) is

    variable ll : line := new string'(in_line.all);
  begin

    WRITELINE(f, ll);
  end write_line;

-- *********************************************************************

-- Places the first word found in "in_line" into "out_line"
-- Also reports out_line's length in "nmb" variable
  procedure get_token(in_line  : inout line;
                      out_line : out   line;
                      nmb      : inout integer) is

    variable ll  : line;
    variable cc  : character;
    variable res : boolean;
  begin

    nmb := 0;
    loop

--  if ENDLINE(in_line) then exit;
      if IN_LINE'length = 0 then exit;
      end if;
-- empty line has been found

      READ(in_line, cc, res);

      if cc /= ' ' then
        loop
          WRITE(ll, cc);
          nmb := nmb+1;
          --           if ENDLINE(in_line) then exit;
          if (in_line'length = 0) then exit;
          end if;
          READ(in_line, cc, res);
          if cc = ' ' then exit;
          end if;
        end loop;
        exit;
      end if;
    end loop;

    out_line := ll;
  end get_token;

-- *********************************************************************

-- left string comparison: match n characters of sref in in_line.
  procedure lstrcmpn(in_line : inout line;
                     sref    : in    string;
                     n       : in    positive;
                     ok      : inout boolean)  is


    variable i  : integer := 0;
    variable ll : line    := new string'(in_line.all);
    variable cc : character;
  begin

    ok := true;
    for i in 1 to n loop
--  if ENDLINE(ll) then
      if ll'length = 0 then

        ok := false;
        exit;
      end if;
      READ(ll, cc);
      if cc /= sref(i) then
        ok := false;
        exit;
      end if;
    end loop;

  end lstrcmpn;
-- *********************************************************************
  function strlen(s : string) return integer is
    variable l  : integer := 0;
    variable ll : line    := new string'(s);
    variable cc : character;
  begin
    loop
-- if ENDLINE(ll) then
      if (ll'length = 0) then
        return l;
      else
        l := l+1;
      end if;
      READ(ll, cc);
    end loop;
  end strlen;
-- *********************************************************************
-- compares n characters in s1 and s2 from the left
  function strcmpn(s1, s2 : in string;
                   n      : in integer) return boolean is

    variable ok : boolean := true;
    variable i  : integer;
  begin
    for i in 1 to n loop
      if s1(i) /= s2(i) then
        ok := false;
        exit;
      end if;
    end loop;
    return ok;
  end;
-- *********************************************************************
-- returns if a string s is "up" or "down", else error
  function up_down(s : string) return boolean is
    variable ok : boolean := true;
    variable i  : integer;
  begin
-- changed
--if strcmpn(s,"up",2) and (strlen(s) = 2) then
    if strcmpn(s, "up", 2) then
      return true;
    else
-- changed
--    if strcmpn(s,"down",4) and (strlen(s) =4) then
      if strcmpn(s, "down", 4) then
        return false;
      else
        assert false report
          "? wrong up/down parameter"
          severity failure;
		return false; -- to remove compiler warning
      end if;
    end if;
  end up_down;
-- *********************************************************************
-- casts string literal "0" and "1" into std logic values
  function slogic(S : string) return std_logic is
  begin
    if strlen(s) /= 1 then
      assert false report
        "? wrong parameter format (STD_LOGIC)"
        severity failure;
    end if;
    case s(1) is
      when '0' =>
        return '0';
      when '1' =>
        return '1';
      when others =>
        assert false report
          "? wrong parameter format (STD_LOGIC)"
          severity failure;
		return 'X'; -- to remove compiler warning
    end case;
  end slogic;

-- *********************************************************************
-- reads a line from file f and outputs each word into comm array of words
-- PS - a "comm" could be a nonary command, a unary command + argument, etc
  procedure get_command(file f     :       text;
                        comm       : inout COMMAND;
                        nmb        : out   integer) is


    variable m_line, t_line : line;
    variable length         : natural;
    variable i              : natural := 1;
  begin
    
    while not ENDFILE(f) loop
      readline(f, m_line);
      loop
        get_token(m_line, comm(i), length);
        if length = 0 then exit;
        end if;
        if (i = 1) and (comm(1)(1) = '#') then exit;
        end if;
        i := i+1;
      end loop;
      if i /= 1 then exit;  -- else this was a comment or empty line; try again
      end if;
    end loop;
    nmb := i-1;

  end get_command;


-- *********************************************************************
-- accessor to write_line, which takes object of type "line"
  procedure print_string(file f : text;
                         s      : string) is

    variable ll : line := new string'(s);
  begin
    write_line(f, ll);
  end print_string;
-- *********************************************************************
  procedure print_string_time(file f : text;
                              s      : string;
                              t      : time) is
    variable ll : line;
    variable lt : line;
  begin
    WRITE(lt, t, left, 10);
    ll := new string'(lt.all & " " & s);
    write_line(f, ll);
  end print_string_time;
-- *********************************************************************
  procedure print_s_vector_time(file f : text;
                                s      : string;
                                v      : std_logic_vector;
                                t      : time) is
    variable ll  : line;
    variable ll1 : line;
  begin
    HWRITE(ll, v);
    WRITE(ll1, v);
    print_string_time(f, s & ll1.all & " " & ll.all, t);
  end print_s_vector_time;

-- *********************************************************************

  function v32_to_i(dv : std_logic_vector) return integer is
-- twos-complement conversion to an integer
    variable d32 : std_logic_vector(1 to 32);
    variable val : integer;
    variable s32 : signed (1 to 32);

  begin

    assert dv'length <= 32 report
                        "? v32_to_i - argument too long"
                        severity failure;

    s32 := signed(dv);
    val := conv_integer(s32);

--d32(1 to 32) := (others => '0'); 
--
--d32((32-dv'RIGHT+dv'LEFT) to 32):=dv;
--
--if d32(1)='1' then
--        for i in d32'range loop
--              if d32(i) = '1' then
--                           d32(i) := '0';
--              else
--                           d32(i) := '1';
--              end if;
--        end loop;
--        val := -(to_integer(d32(2 to 32))+1);
--else
--        val := to_integer(d32(2 to 32));
--end if;

    return val;
  end v32_to_i;
-- *********************************************************************
  function i_to_v32(val : integer) return std_logic_vector is

    variable d32 : std_logic_vector(1 to 32);
  begin
    d32 := Conv_Std_Logic_Vector(val, 32);
    return d32;
  end i_to_v32;

-- *********************************************************************
-- line-to-integer conversion assuming hex number is saved in line
  procedure hex_read(ll  : inout line;
                     val : out   integer) is
    variable dv : std_logic_vector(1 to 32);
  begin
    HREAD(ll, dv);
    val := v32_to_i(dv);
  end hex_read;
-- *********************************************************************

  
end MY_TEXT;




-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--
--  File name : femb_defi.vhd
--  Title     : Package with S-LINK FEMB Definitions and interpretation of
--              commands
--  Purpose   : 
--  Author(s) : Pjotr Kapusta, 
--              Erik van der Bij - CERN Geneva
--
--  Notes     : 
--
-------------------------------------------------------------------------------
-- Version | Author |  Mod.Date  |          Changes made
-------------------------------------------------------------------------------
--   1.00  |   PK   |    1996    | Original coding
--   2.01  |   EB   | 11-05-2000 | conv_std_logic_vector. Remove set 16/33/66
--   2.02  |   EB   | 12-05-2000 | "Set extra" check removed
--         |        |            |
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_textio.all;
use IEEE.STD_LOGIC_ARITH.all;
use std.textio.all;
use work.my_text.all;


package FEMB_DEFI is

  type CODE is (

    SET_T,                              -- set time parameter
    SET_I,                              -- set integer parameter
    INIT16,  -- sets timing params. for 16.5 MHz. ILLEGAL
    INIT33,  -- sets timing params. for 33 MHz.   ILLEGAL
    INIT66,  -- sets timing params. for 66 MHz.   ILLEGAL

    FINISH,                             -- end of simulation
    RESET,                              -- perform reset of S-link
    TEST,                               -- perform S-link test
    SEND_D,                             -- send word of data
    SEND_B,                             -- send block of data
    SEND_C,                             -- send control word

    ASSERT_R,                           -- assert URESET
    ASSERT_T,                           -- assert UTEST 
    ASSERT_C,                           -- assert UCTRL 
    ASSERT_W,                           -- assert UWEN  
    ASSERT_D,                           -- assert UD    

    WAIT_F,                             -- wait for specified time
    WAIT_C,                             -- wait for edge of the UCLK
    WAIT_L,                             -- wait for edge of the LFF#
    WAIT_D,                             -- wait for edge of the LDOWN#

    START,                              -- starts  clock
    STOP                                -- stops   clock
    );

  type ACTION is (
    ATOMIC,
    SET,
    PROC
    ); 

  type OPERATION is record
    name : string (1 to 10);
    val  : positive;
    qual : string (1 to 6);
    qval : natural;
    code : CODE;
    act  : ACTION;
    pos  : natural;
  end record;

-------------------------------------------------------------------------------
-->           offsets in parameter arrays                                   <--
-------------------------------------------------------------------------------

  constant p_null : natural := 0;

  constant p_clk_h : natural := 1;      -- uclk high period 
  constant p_clk_l : natural := 2;      -- uclk low period
  constant p_ds    : natural := 3;      -- data setup
  constant p_dh    : natural := 4;      -- data hold
  constant p_es    : natural := 5;      -- enable setup
  constant p_eh    : natural := 6;      -- enable hold
  constant p_wf    : natural := 7;      -- max. delay for FF# low after UCLK goes high
  constant p_fin   : natural := 8;      -- last up edge

  constant p_ex    : natural := 1;      -- number of extra words after LFF# goes low  
  constant p_patt  : natural := 2;      -- type of data pattern
  constant p_width : natural := 3;      -- data width of the model
  constant p_pati  : natural := 4;      -- initial value of pattern
  constant p_dres  : natural := 5;      -- delay after URESET_n goes high  


  type OP_ARRAY is array (integer range <>) of OPERATION;
  type TIME_PARAM is array (integer range <>) of time;
  type INT_PARAM is array (integer range <>) of integer;


  constant OP_A : OP_ARRAY := (

-- general settings

    1  => ("set       ", 3, "clkh  ", 4, SET_T, SET, p_clk_h),
    2  => ("set       ", 3, "clkl  ", 4, SET_T, SET, p_clk_l),
    3  => ("set       ", 3, "ds    ", 2, SET_T, SET, p_ds),
    4  => ("set       ", 3, "dh    ", 2, SET_T, SET, p_dh),
    5  => ("set       ", 3, "es    ", 2, SET_T, SET, p_es),
    6  => ("set       ", 3, "eh    ", 2, SET_T, SET, p_eh),
    7  => ("set       ", 3, "wff   ", 2, SET_T, SET, p_wf),
    8  => ("set       ", 3, "ex    ", 2, SET_I, SET, p_ex),
    9  => ("set       ", 3, "patt  ", 4, SET_I, SET, p_patt),
    10 => ("set       ", 3, "pati  ", 4, SET_I, SET, p_pati),
    11 => ("set       ", 3, "width ", 4, SET_I, SET, p_width),
    12 => ("set       ", 3, "dres  ", 4, SET_I, SET, p_dres),
    13 => ("init      ", 3, "16    ", 2, INIT16, SET, p_null),  -- illegal
    14 => ("init      ", 3, "33    ", 2, INIT33, SET, p_null),  -- illegal
    15 => ("init      ", 3, "66    ", 2, INIT66, SET, p_null),  -- illegal

-- procedures
    16 => ("xxxxxxxxxx", 3, "xxxxxx", 0, FINISH, PROC, p_null),
    17 => ("finish    ", 3, "      ", 0, FINISH, PROC, p_null),
    18 => ("reset     ", 3, "      ", 0, RESET, PROC, p_null),
    19 => ("test      ", 3, "      ", 0, TEST, PROC, p_null),
    20 => ("send      ", 3, "data  ", 3, SEND_D, PROC, p_null),
    21 => ("send      ", 3, "bdata ", 3, SEND_B, PROC, p_null),
    22 => ("send      ", 3, "ctrl  ", 3, SEND_C, PROC, p_null),

-- atomics
    23 => ("assert    ", 3, "ureset", 3, ASSERT_R, ATOMIC, p_null),
    24 => ("assert    ", 3, "utest ", 3, ASSERT_T, ATOMIC, p_null),
    25 => ("assert    ", 3, "uctrl ", 3, ASSERT_C, ATOMIC, p_null),
    26 => ("assert    ", 3, "uwen  ", 3, ASSERT_W, ATOMIC, p_null),
    27 => ("assert    ", 3, "ud    ", 3, ASSERT_D, ATOMIC, p_null),
    28 => ("wait      ", 3, "for   ", 3, WAIT_F, ATOMIC, p_null),
    29 => ("wait      ", 3, "clk   ", 3, WAIT_C, ATOMIC, p_null),
    30 => ("wait      ", 3, "lff   ", 3, WAIT_L, ATOMIC, p_null),
    31 => ("wait      ", 3, "ldown ", 3, WAIT_D, ATOMIC, p_null),
    32 => ("start     ", 3, "clk   ", 3, START, ATOMIC, p_null),
    33 => ("stop      ", 3, "clk   ", 3, STOP, ATOMIC, p_null)
    ); 



-------------------------------------------------------------------------------
-->              output constants                                           <--
-------------------------------------------------------------------------------

  constant SOUT_UCLK   : string := "--> UCLK    ";
  constant SOUT_EXTRA  : string := "--> EXTRA   ";
  constant SOUT_UD     : string := "----> UD(1:32) ";
  constant SOUT_UCTRL  : string := "----> UCTRL   ";
  constant SOUT_UWEN   : string := "------> UWEN#   ";
  constant SOUT_LFF    : string := "<-------- LFF#     ";
  constant SOUT_LRL    : string := "<-------- LRL#(1:4) ";
  constant SOUT_URESET : string := "----------> URESET# ";
  constant SOUT_UTEST  : string := "------------> UTEST#  ";
  constant SOUT_LDOWN  : string := "--------------> LDOWN#  ";
  constant SOUT_UDW    : string := "----------------> UDW     ";




  procedure exe_action(op               : in    OPERATION;  -- command
                       variable comm    : inout COMMAND;    -- tokens
                       signal tp        : inout TIME_PARAM;  -- array of time parm
                       signal ip        : inout INT_PARAM;  -- array of int parm
                       signal UD        : inout std_logic_vector(1 to 32);
                       signal LFF_n     : in    std_logic;
                       signal URESET_n  : inout std_logic;
                       signal UTEST_n   : inout std_logic;
                       signal UDW       : inout std_logic_vector(1 to 2);
                       signal UCTRL_n   : inout std_logic;
                       signal UWEN_n    : inout std_logic;
                       signal UCLK      : in    std_logic;
                       signal LRL_n     : in    std_logic_vector(1 to 4);
                       signal LDOWN_n   : in    std_logic;
                       signal CLK_EN    : out   std_logic;
                       signal last_up   :       time;
                       signal last_down :       time;
                       signal extra     :       integer
                       );
--
  procedure toggle (signal sig : inout std_logic);
--
  procedure pattern(
    patt : in    integer;
    dv   : inout std_logic_vector(1 to 32));
--
  procedure send_word(
    constant cnt     :     std_logic;
    variable dv      :     std_logic_vector;
    signal tp        : in  TIME_PARAM;
    signal ip        : in  INT_PARAM;
    signal UCLK      : in  std_logic;
    signal LFF_n     : in  std_logic;
    signal UD        : out std_logic_vector;
    signal UCTRL_n   : out std_logic;
    signal UWEN_n    : out std_logic;
    signal last_up   : in  time;
    signal last_down : in  time;
    signal extra     : in  integer;
    width            : in  integer
    );                     
end FEMB_DEFI;

-- ***************************************************************

package body FEMB_DEFI is

  procedure toggle (signal sig : inout std_logic) is
  begin
    if sig = '0' then
      sig <= '1';
    else
      sig <= '0';
    end if;
  end toggle;

--

  procedure exe_action(op               : in    OPERATION;
                       variable comm    : inout COMMAND;
                       signal tp        : inout TIME_PARAM;
                       signal ip        : inout INT_PARAM;
                       signal UD        : inout std_logic_vector(1 to 32);
                       signal LFF_n     : in    std_logic;
                       signal URESET_n  : inout std_logic;
                       signal UTEST_n   : inout std_logic;
                       signal UDW       : inout std_logic_vector(1 to 2);
                       signal UCTRL_n   : inout std_logic;
                       signal UWEN_n    : inout std_logic;
                       signal UCLK      : in    std_logic;
                       signal LRL_n     : in    std_logic_vector(1 to 4);
                       signal LDOWN_n   : in    std_logic;
                       signal CLK_EN    : out   std_logic;
                       signal last_up   :       time;
                       signal last_down :       time;
                       signal extra     :       integer
                       ) is
    variable tm, tm1, tm2, period : time;
    variable dt, i                : integer;
    variable dv                   : std_logic_vector(1 to 32);
    variable qu                   : natural;
    variable ll                   : line;
    variable ok                   : boolean;
    variable prev                 : integer := 0;
  begin

    if op.qval /= 0 then
      qu := 2;
    else
      qu := 1;
    end if;

    case op.act is
-----------------------------------------------------------------------
-->                           atomics                               <--
-----------------------------------------------------------------------
      
      when ATOMIC =>
        case op.code is
          when WAIT_F =>
            ll := new string'(comm(qu+1).all &" "& comm(qu+2).all);
            READ(ll, tm);
            wait for tm;
          when WAIT_C =>
            if up_down(comm(qu+1).all) then
              wait until UCLK'event and UCLK = '1';
            else
              wait until UCLK'event and UCLK = '0';
            end if;
          when WAIT_L =>
            if up_down(comm(qu+1).all) then
              wait until LFF_n'event and LFF_n = '1';
            else
              wait until LFF_n'event and LFF_n = '0';
            end if;
          when WAIT_D =>
            if up_down(comm(qu+1).all) then
              wait until LDOWN_n'event and LDOWN_n = '1';
            else
              wait until LDOWN_n'event and LDOWN_n = '0';
            end if;
--ASSERT_R ,  -- assert URESET
--ASSERT_T ,  -- assert UTEST 
--ASSERT_C ,  -- assert UCTRL 
--ASSERT_W ,  -- assert UWEN  
--ASSERT_D ,  -- assert UD    
          when ASSERT_R =>
            URESET_n <= slogic(comm(qu+1).all);
          when ASSERT_T =>
            UTEST_n <= slogic(comm(qu+1).all);
          when ASSERT_C =>
            UCTRL_n <= slogic(comm(qu+1).all);
          when ASSERT_W =>
            UWEN_n <= slogic(comm(qu+1).all);
          when ASSERT_D =>
            HREAD(comm(qu+1), dv);
            UD(1 to 32) <= dv(1 to 32);
          when START =>
            CLK_EN <= '1';
            wait until UCLK'event and UCLK = '1';
            -- to start correctly "last_up" - "last_down"
          when STOP =>
            CLK_EN <= '0';
          when others =>
            assert false report
              "? - unrecognized ATOMIC operation"
              severity failure;
        end case;

-----------------------------------------------------------------------
-->                           SETTINGS                              <--
-----------------------------------------------------------------------
        
      when SET =>
        case op.code is
          when SET_T =>
            ll         := new string'(comm(qu+1).all &" "& comm(qu+2).all);
            READ(ll, tm);
            tp(op.pos) <= tm;
          when SET_I =>
            if op.pos = p_pati then
              hex_read(comm(qu+1), dt);
            else
              READ(comm(qu+1), dt);
            end if;
            ip(op.pos) <= dt;
            if op.pos = p_width then
              case dt is
                when 32 =>
                  UDW <= "00";
                when 16 =>
                  UDW <= "10";
                when 8 =>
                  UDW <= "01";
                when others =>
                  assert false report
                    "? illegal data width"
                    severity failure;
              end case;
            end if;

-- The INIT commands are not used anymore as the S-LINK specification
-- only allows clocks up to 40 MHz. V2.01 VHDL
-- when INIT16  =>
--                                  tp(p_clk_h) <= 30 ns;
--                                  tp(p_clk_l) <= 30 ns;
--                                  tp(p_ds)    <= 13 ns;
--                                  tp(p_dh)    <=  1 ns;
--                                  tp(p_es)    <= 13 ns;
--                                  tp(p_eh)    <=  1 ns;
--                                  tp(p_wf)    <= 16 ns;
-- when INIT33  =>
--                                  tp(p_clk_h) <= 15 ns;
--                                  tp(p_clk_l) <= 15 ns;
--                                  tp(p_ds)    <= 10 ns;
--                                  tp(p_dh)    <=  1 ns;
--                                  tp(p_es)    <= 10 ns;
--                                  tp(p_eh)    <=  1 ns;
--                                  tp(p_wf)    <= 12 ns;
-- when INIT66  =>
--                                  tp(p_clk_h) <= 7500 ps;
--                                  tp(p_clk_l) <= 7500 ps;
--                                  tp(p_ds)    <=  9 ns;
--                                  tp(p_dh)    <=  1 ns;
--                                  tp(p_es)    <=  9 ns;
--                                  tp(p_eh)    <=  1 ns;
--                                  tp(p_wf)    <= 10 ns;
            
          when others =>
            assert false report
              "? - unrecognized SET operation"
              severity failure;
        end case;

-----------------------------------------------------------------------
-->                          PROCEDURES                             <--
-----------------------------------------------------------------------
      when PROC =>
        case op.code is

          when FINISH =>
            CLK_EN <= '0';
            assert false report
              "finish statement found"
              severity failure;         -- to stop simulation
            wait;

          when RESET =>
            URESET_n <= '0';
            --wait until LDOWN_n'event and LDOWN_n='0'; 
            wait until LDOWN_n'event and LDOWN_n = '1';
            URESET_n <= '1';
            for i in 1 to ip(p_dres) loop
              wait until UCLK'event and UCLK = '1';
            end loop;
          when TEST =>

          when SEND_D =>
            HREAD(comm(qu+1), dv);
            if LDOWN_n = '0' then wait until LDOWN_n'event
                                    and LDOWN_n = '1';
                                  -- what else can I specify ??? No timing defined !
            end if;
            send_word
              ('1', dv, tp, ip, UCLK, LFF_n, UD, UCTRL_n, UWEN_n, last_up,
               last_down, extra, ip(p_width));

          when SEND_B =>
            READ(comm(qu+1), dt);
            dv(1 to 32) :=
              Conv_Std_Logic_Vector(ip(p_pati), 32)(31 downto 0);
            for i in 1 to dt loop
              if LDOWN_n = '0' then wait until LDOWN_n'event
                                      and LDOWN_n = '1';
                                    -- what else can I specify ??? No timing defined !
              end if;
              send_word
                ('1', dv, tp, ip, UCLK, LFF_n, UD, UCTRL_n, UWEN_n, last_up,
                 last_down, extra, ip(p_width));
              pattern(ip(p_patt), dv);
            end loop;

          when SEND_C =>
            HREAD(comm(qu+1), dv);
            if LDOWN_n = '0' then wait until LDOWN_n'event
                                    and LDOWN_n = '1';
                                  -- what else can I specify ??? No timing defined !
            end if;
            send_word
              ('0', dv, tp, ip, UCLK, LFF_n, UD, UCTRL_n, UWEN_n, last_up,
               last_down, extra, ip(p_width));

          when others =>
            assert false report
              "? - unrecognized PROCEDURE"
              severity failure;
        end case;
      when others =>
        assert false report
          "? - unrecognized action"
          severity failure;
    end case;

  end exe_action;
-------------------------------------------------------------------
-->                  pattern                                    <--
-------------------------------------------------------------------
  procedure pattern(
    patt : in    integer;
    dv   : inout std_logic_vector(1 to 32)) is
    variable dat : integer;
  begin

    case patt is
      when 0 =>
        dat := 16#00ABCDEF#;
      when others =>
        dat := v32_to_i(dv);
        dat := dat + patt;
    end case;

    dv := Conv_Std_Logic_Vector(dat, 32);
  end pattern;

-------------------------------------------------------------------
-->                  send_word                                  <--
-------------------------------------------------------------------
  procedure send_word(
    constant cnt     :     std_logic;
    variable dv      :     std_logic_vector;
    signal tp        : in  TIME_PARAM;
    signal ip        : in  INT_PARAM;
    signal UCLK      : in  std_logic;
    signal LFF_n     : in  std_logic;
    signal UD        : out std_logic_vector;
    signal UCTRL_n   : out std_logic;
    signal UWEN_n    : out std_logic;
    signal last_up   : in  time;
    signal last_down : in  time;
    signal extra     : in  integer;
    width            : in  integer
    ) is

    variable period, tm : time;

  begin
    period := tp(p_clk_h) + tp(p_clk_l);

    if (period - tp(p_wf) > tp(p_ds))
      and (period-tp(p_wf) > tp(p_es)) then

      if (NOW - last_up) < tp(p_wf) then
        wait for tp(p_wf) - (NOW-last_up);
      end if;
    end if;

    if (LFF_n = '0') and (extra >= ip(p_ex)) then
      wait until LFF_n = '1';
    end if;

    tm := NOW - last_up;
    if tp(p_ds) >= tp(p_es) then
      if (period-tm) > tp(p_ds) then
        tm                 := (last_up + period - tp(p_ds))- NOW;
        wait for tm;
        UD(33-width to 32) <= dv(33-width to 32);
        UCTRL_n            <= cnt;
        wait for (tp(p_ds)-tp(p_es));
        UWEN_n             <= '0';
      else
        wait until UCLK'event and UCLK = '1';
        wait for (period- tp(p_ds));
        UD(33-width to 32) <= dv(33-width to 32);
        UCTRL_n            <= cnt;
        wait for (tp(p_ds)-tp(p_es));
        UWEN_n             <= '0';
      end if;
    else
      if(period-tm) > tp(p_es) then
        tm                 := (last_up + period -tp(p_es)) -NOW;
        wait for tm;
        UWEN_n             <= '0';
        wait for (tp(p_es)-tp(p_ds));
        UD(33-width to 32) <= dv(33-width to 32);
        UCTRL_n            <= cnt;
      else
        wait until UCLK'event and UCLK = '1';
        wait for (period- tp(p_es));
        UWEN_n             <= '0';
        wait for (tp(p_es)-tp(p_ds));
        UD(33-width to 32) <= dv(33-width to 32);
        UCTRL_n            <= cnt;
      end if;
    end if;


    wait until UCLK'event and UCLK = '1';
    if tp(p_dh) >= tp(p_eh) then
      wait for tp(p_eh);
      UWEN_n  <= '1';
      wait for (tp(p_dh)-tp(p_eh));
      UD      <= "UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU";
      UCTRL_n <= 'U';
    else
      wait for tp(p_dh);
      UD      <= "UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU";
      UCTRL_n <= 'U';
      wait for (tp(p_eh)-tp(p_dh));
      UWEN_n  <= '1';
    end if;

  end send_word;

end FEMB_DEFI;


-- ***************************************************************

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--
--  File name : femb.vhd
--  Title     : Entity and Architecture of an S-LINK FEMB 
--  Purpose   : Complete simulation model of an S-LINK FEMB
--  Author(s) : Pjotr Kapusta, 
--              Erik van der Bij - CERN Geneva
--
--  Notes     : reads  file femb_in.txt
--              writes file femb_out.txt
--
-------------------------------------------------------------------------------
-- Version | Author |  Mod.Date  |          Changes made
-------------------------------------------------------------------------------
--   1.00  |   PK   |    1996    | Original coding
--   2.01  |   EB   | 11-05-2000 | Correct settings of Tsu/h
--   2.02  |   EB   | 12-05-2000 | Severity settings, default of p_ex (extra) 
--   2.03  |   EB   | 15-05-2000 | Removed message on  clk down, A->ARCH 
--   2.04  |  JMH   | 06-02-2003 | generic added for time_param
--         |        |            |
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_TEXTIO.all;
use std.textio.all;
--
--
--
entity FEMB is
  generic(
    UCLK_HIGH      : time := 12500 ps;  -- UCLK high time
    UCLK_LOW       : time := 12500 ps;  -- UCLK low time
    DATA_SETUP     : time := 10 ns;     -- Data setup time
    DATA_HOLD      : time := 1 ns;      -- Data hold time
    ENABLE_SETUP   : time := 10 ns;     -- Enable setup time
    ENABLE_HOLD    : time := 1 ns;      -- Enable hold time
    MAX_CLK_TO_LFF : time := 12 ns;     -- Maximum clock to Full flag
	FEMB_IN_FILENAME : string := "femb_in.txt";
	FEMB_OUT_FILENAME : string := "femb_out.txt"
    );
  port (
    UD       : buffer std_logic_vector(1 to 32);
    LFF_n    : in     std_logic;
    URESET_n : buffer std_logic                := '1';
    UTEST_n  : buffer std_logic                := '1';
    UDW      : buffer std_logic_vector(1 to 2) := "00";
    UCTRL_n  : buffer std_logic;
    UWEN_n   : buffer std_logic                := '1';
    UCLK     : buffer std_logic;
    LRL      : in     std_logic_vector(1 to 4);
    LDOWN_n  : in     std_logic
    );

end FEMB;

library hola_sim_lib;
use hola_sim_lib.my_text.all;
use hola_sim_lib.femb_defi.all;

---------------------------------------------------------------------------
architecture ARCH of FEMB is
--
  constant FEMB_IN_FNAME : string := FEMB_IN_FILENAME;
  constant FEMB_OUT_FNAME : string := FEMB_OUT_FILENAME;
  file INFILE  : text open read_mode is FEMB_IN_FNAME;
  file OUTFILE : text open write_mode is FEMB_OUT_FNAME;

---------------------- internal signals ------------------------------------
  signal CLK_EN : std_logic := '0';

  signal last_up   : time    := 0 ns;
  signal last_down : time    := 0 ns;
  signal extra     : integer := 0;
  signal ureset_up : time    := 0 ns;
---------------------- parameters ------------------------------------------
  signal t_p : TIME_PARAM(p_clk_h to p_fin) := (
-- defaults are for 40 MHz
    p_clk_h => UCLK_HIGH,  -- 12500 ps, -- UCLK high time
    p_clk_l => UCLK_LOW,  -- 12500 ps, -- UCLK low time
    p_ds    => DATA_SETUP,  --    10 ns, -- Data setup time
    p_dh    => DATA_HOLD,  --     1 ns, -- Data hold time
    p_es    => ENABLE_SETUP,  --    10 ns, -- Enable setup time
    p_eh    => ENABLE_HOLD,  --     1 ns, -- Enable hold time
    p_wf    => MAX_CLK_TO_LFF,  --    12 ns, -- Maximum clock to Full flag
    p_fin   => 0 ns                     -- ? unused - last entry in array
    );  

  signal i_p : INT_PARAM(p_ex to p_dres) := (
-- defaults
    p_ex    => 2,                       -- two extra words after LFF# down 
    p_patt  => 0,                       -- pattern nmb 0
    p_width => 32,                      -- 32 bits
    p_pati  => 0,                       -- pattern number 0
    p_dres  => 4    -- four clock cycles of delay after URESET goes high
    ); 

begin

------------------------------------------------------------------------
-->                      FILE INTERPRETER                            <--
------------------------------------------------------------------------

  FI : process
    variable i, length, tnmb, cnmb : integer;
    variable comm                  : COMMAND(1 to 10);
    variable ok                    : boolean;


  begin

    while not endfile(INFILE) loop

      get_command(INFILE, comm, tnmb);

      if tnmb /= 0 then

        SEARCH : for i in OP_A'range loop

          lstrcmpn(comm(1), OP_A(i).name, OP_A(i).val, ok);
          if ok = true then
            if (OP_A(i).qval /= 0) and tnmb >= 2 then
              lstrcmpn(comm(2), OP_A(i).qual, OP_A(i).qval, ok);
              if ok then
                cnmb := i;
                exit SEARCH;
              end if;
            else
              cnmb := i;
              exit SEARCH;
            end if;
          end if;
        end loop SEARCH;
        if ok then
          print_string_time(OUTFILE,
                            "###### found command: "&OP_A(cnmb).name&" "&OP_A(cnmb).qual, NOW);
          exe_action(OP_A(cnmb),
                     comm,
                     t_p,
                     i_p,
                     UD,
                     LFF_n,
                     URESET_n,
                     UTEST_n,
                     UDW,
                     UCTRL_n,
                     UWEN_n,
                     UCLK,
                     LRL,
                     LDOWN_n,
                     CLK_EN,
                     last_up,
                     last_down,
                     extra
                     );
        else
          assert false report "? unrecognized: " & comm(1).all
            severity failure;
        end if;
      end if;
    end loop;
    assert false report "end of file reached"
      severity note;
    wait;
  end process FI;

--------------------------------------------------------------------------
-->                  UCLK_GEN                                          <--  
--------------------------------------------------------------------------
  UCLK_GEN : process

    variable clk_h, clk_l : time;

  begin

    wait until CLK_EN = '1';
    clk_h := t_p(p_clk_h);
    clk_l := t_p(p_clk_l);

    PERIOD : loop

      UCLK    <= '1';
      print_string_time(OUTFILE, SOUT_UCLK & "up", NOW);
      last_up <= NOW;
      if UWEN_n = '0' then
        if LFF_n = '0' then
          extra <= (extra + 1);
--                   print_string_time(OUTFILE,SOUT_EXTRA & "++",NOW);
        else
          extra <= 0;
--                   print_string_time(OUTFILE,SOUT_EXTRA & "zero",NOW);
        end if;
      end if;
      wait for clk_h;
      UCLK      <= '0';
      -- print_string_time(OUTFILE,SOUT_UCLK & "down",NOW);
      last_down <= NOW;
      wait for clk_l;
      if CLK_EN = '0' then
        exit;
      end if;
    end loop PERIOD;

  end process UCLK_GEN;


--------------------------------------------------------------------------
-->                    LFF_monitor                                     <--  
--------------------------------------------------------------------------
  LFF_monitor : process(LFF_n)

  begin
    if LFF_n = '0' then
      print_string_time(OUTFILE, SOUT_LFF & "down", NOW);
    else
      print_string_time(OUTFILE, SOUT_LFF & "up", NOW);
    end if;
------------------------ check LFF# delay after UCLK up ------- CHECK ----
    if NOW - last_up > t_p(p_wf) then
      assert false report
        "? twff violation"
        severity error;
      print_string_time(OUTFILE, "? LFF twff violation", NOW);
      print_string_time(OUTFILE, "  last UCLK up", last_up);
    end if;
--------------------------------------------------------------------------
  end process LFF_monitor;
--------------------------------------------------------------------------
-->                    LRL_monitor                                     <--  
--------------------------------------------------------------------------
  LRL_monitor : process (LRL)
  begin
    print_s_vector_time(OUTFILE, SOUT_LRL & "set to: ", LRL, NOW);
  end process LRL_monitor;

-------------------------------------------------------------------------
-->                     RESET monitor                                 <--
-------------------------------------------------------------------------
  RESET_monitor : process
    variable e_cnt : integer;
  begin
    wait until URESET_n'event and URESET_n = '0';
    print_string_time(OUTFILE, SOUT_URESET & "down", NOW);
    e_cnt := 0;
    RES : loop
      wait until UCLK'event and UCLK = '1';
      e_cnt := e_cnt + 1;  -- count clocks after reset is asserted
      if (e_cnt > 4) and (LDOWN_n = '1') then  -- LDOWN_n should be set within 4 clks of reset
        print_string(OUTFILE, "?RESET - trl number of cycles exceeded");
        assert false report
          "?RESET - trl number of cycles exceeded"
          severity error;
      else
        if LDOWN_n = '0' then
          exit RES;
        end if;
      end if;
    end loop RES;

    wait until URESET_n'event and URESET_n = '1';
    print_string_time(OUTFILE, SOUT_URESET & "up", NOW);
    ureset_up <= NOW;                   -- store time of URESET# up

  end process RESET_monitor;
-------------------------------------------------------------------------
-->                     LDOWN monitor                                 <--
-------------------------------------------------------------------------
  LDOWN_monitor : process(LDOWN_n)
  begin
    case LDOWN_n is
      when '0' =>
        print_string_time(OUTFILE, SOUT_LDOWN & "down", NOW);
      when '1' =>
        print_string_time(OUTFILE, SOUT_LDOWN & "up", NOW);
      when others =>
        print_string_time(OUTFILE, SOUT_LDOWN & "unknown", NOW);
    end case;
  end process LDOWN_monitor;
-------------------------------------------------------------------------
-->                      UDW  monitor                                 <--
-------------------------------------------------------------------------
  UDW_monitor : process(UDW)
  begin
    print_string_time(OUTFILE, SOUT_UDW & "set", NOW);
  end process UDW_monitor;
--------------------------------------------------------------------------
-->                       UWEN_monitor                                 <--  
--------------------------------------------------------------------------
  UWEN_monitor : process(UWEN_n)

  begin
    if UWEN_n = '0' then
      print_string_time(OUTFILE, SOUT_UWEN & "down", NOW);
    else
      print_string_time(OUTFILE, SOUT_UWEN & "up", NOW);
    end if;
  end process UWEN_monitor;
--------------------------------------------------------------------------
-->                       UD_monitor                                 <--  
--------------------------------------------------------------------------
  UD_monitor : process(UD)
  begin
    print_s_vector_time(OUTFILE, SOUT_UD & "set to: ", UD, NOW);
  end process UD_monitor;

--------------------------------------------------------------------------
-->                       UCTRL_monitor                                 <--
--------------------------------------------------------------------------
  UCTRL_monitor : process(UCTRL_n)
  begin
    case UCTRL_n is
      when '0' =>
        print_string_time(OUTFILE, SOUT_UCTRL & "down", NOW);
      when '1' =>
        print_string_time(OUTFILE, SOUT_UCTRL & "up", NOW);
      when others =>
        print_string_time(OUTFILE, SOUT_UCTRL & "unknown", NOW);
    end case;

  end process UCTRL_monitor;

end ARCH;
--------------------------------------------------------------------------
-->                       E N D   O F   F I L E                        <--
--------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--
--  File name : fifo.vhd
--  Title     : Entity and Architecture of a FIFO
--  Purpose   : FIFO used inside S-LINK ROMB model
--  Author(s) : Pjotr Kapusta, 
--              Erik van der Bij - CERN Geneva
--
--  Notes     : 
--
-------------------------------------------------------------------------------
-- Version | Author |  Mod.Date  |          Changes made
-------------------------------------------------------------------------------
--   1.00  |   PK   |    1996    | Original coding
--   2.01  |   EB   | 15-05-2000 | Added header, changed conv_integer 
--         |        |            |
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.std_logic_arith.all;

entity FIFO is
  generic (
    FIFO_SIZE   : integer := 100;
    STATE_DELAY : time    := 3 ns
    );

  port (
    DATA_IN     : in  std_logic_vector(1 to 33);
    STROBE_IN   : in  std_logic;
    ENABLE_IN_n : in  std_logic;
    DATA_OUT    : out std_logic_vector(1 to 33) :=
    "UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU";
    STROBE_OUT  : in  std_logic;
    THRESH      : in  std_logic_vector(1 to 16);
    NOT_EMPTY_n : out std_logic := '1';
    WARNING_n   : out std_logic := '1';
    FULL_n      : out std_logic := '1';
    RESET_n     : in  std_logic
    );
end FIFO;

architecture ARCH of FIFO is
  type FIFO_TABLE is array (integer range <>) of std_logic_vector(1 to 33);

  signal statef : integer := 0;
  signal r_pt   : integer := 0;
  signal w_pt   : integer := 0;
  signal table  : FIFO_TABLE(0 to FIFO_SIZE-1);
begin
----------------------------------------------------------------------------
  FILLER : process(STROBE_IN, RESET_n)
    variable w_off : integer;
  begin
    if RESET_n'event then
      if RESET_n = '0' then
        w_pt <= 0;                      -- reset write pointer
      end if;
    else
      if RESET_n = '1' and STROBE_IN = '1' then
        if ENABLE_IN_n = '0' then
          if statef = FIFO_SIZE then
            assert false report
              "? attempt to write to the full FIFO"
              severity error;
          else
            w_off        := w_pt;
            table(w_off) <= DATA_IN;
            w_off        := (w_off +1) mod FIFO_SIZE;
            w_pt         <= w_off;
          end if;
        end if;
      end if;
    end if;
  end process FILLER;
----------------------------------------------------------------------------
  TAKER : process(STROBE_OUT, RESET_n)
    variable r_off : integer;
  begin
    if RESET_n'event then
      if RESET_n = '0' then
        r_pt <= 0;                      -- reset write pointer
      end if;
    else
      if NOW /= 0 fs then
        if statef = 0 then
          assert false report
            "? attempt to read empty FIFO"
            severity error;
        else
          r_off    := r_pt;
          DATA_OUT <= table(r_off);
          r_off    := (r_off+1) mod FIFO_SIZE;
          r_pt     <= r_off;
        end if;
      end if;
    end if;
  end process TAKER;
----------------------------------------------------------------------------
  DIFF : process(w_pt, r_pt)
  begin
    if w_pt >= r_pt then
      statef <= w_pt - r_pt;
    else
      statef <= w_pt+ FIFO_SIZE- r_pt;
    end if;
  end process DIFF;
----------------------------------------------------------------------------

  STATE : process(statef)
  begin
    if statef >= conv_integer(unsigned(THRESH(1 to 16))) then  -- EB
      WARNING_n <= '0' after STATE_DELAY;
    else
      WARNING_n <= '1' after STATE_DELAY;
    end if;

    if statef >= FIFO_SIZE then
      FULL_n <= '0' after STATE_DELAY;
    else
      FULL_n <= '1' after STATE_DELAY;
    end if;

    if statef > 0 then
      NOT_EMPTY_n <= '0' after STATE_DELAY;
    else
      NOT_EMPTY_n <= '1' after STATE_DELAY;
    end if;

  end process STATE;
----------------------------------------------------------------------------
end ARCH;
--------------------------------------------------------------------------
-->                       E N D   O F   F I L E                        <--
--------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--
--  File name : romb_defi.vhdl
--  Title     : Package with S-LINK FEMBROMB Definitions and interpretation of
--              commands
--  Purpose   : 
--  Author(s) : Pjotr Kapusta    - Institute of Nuclear Physics, Krakow, Poland
--              Erik van der Bij - CERN Geneva
--
--  Notes     : 
--
-------------------------------------------------------------------------------
-- Version | Author |  Mod.Date  |          Changes made
-------------------------------------------------------------------------------
--   1.00  |   PK   |    1996    | Original coding
--   2.01  |   EB   | 12-05-2000 | Remove init 16/33/66
--         |        |            |
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_textio.all;
use IEEE.STD_LOGIC_ARITH.all;
use std.textio.all;
use work.my_text.all;


package ROMB_DEFI is

  type CODE is (

    SET_T,                              -- set time parameter
    SET_I,                              -- set integer parameter
    INIT16,  -- sets timing params. for 16.5 MHz. -- illegal
    INIT33,  -- sets timing params. for 33 MHz.   -- illegal
    INIT66,  -- sets timing params. for 66 MHz.   -- illegal

    FINISH,                             -- end of simulation
    RESET,                              -- perform reset of S-link
    TAKE,    -- take number of words from the ROMB FIFO
    PUTU,                               -- put  URL code 

    WAIT_F,                             -- wait for specified time
    WAIT_U,                             -- wait for the edge of the UXOFF#
    WAIT_C,                             -- wait for the edge of the LCLK
    WAIT_D,                             -- wait for the edge of the LDOWN#

    ASSERT_U,                           -- assert UTDO#
    ASSERT_R                            -- assert URESET# 
    );

  type ACTION is (
    ATOMIC,
    SET,
    PROC
    ); 

  type OPERATION is record
    name : string (1 to 10);
    val  : positive;
    qual : string (1 to 6);
    qval : natural;
    code : CODE;
    act  : ACTION;
    pos  : natural;
  end record;

-------------------------------------------------------------------------------
-->           offsets in parameter arrays                                   <--
-------------------------------------------------------------------------------

  constant p_null : natural := 0;

  constant p_ds  : natural := 1;        -- data setup
  constant p_dh  : natural := 2;        -- data hold
  constant p_es  : natural := 3;        -- enable setup
  constant p_eh  : natural := 4;        -- enable hold
  constant p_per : natural := 5;        -- read period
  constant p_fin : natural := 6;        -- 

  constant p_width : natural := 1;      -- data width of the model
  constant p_dep   : natural := 2;      -- state of the fifo at which UXOFF# is
  -- asserted
  constant p_mode  : natural := 3;      -- mode of FIFO server 


  type OP_ARRAY is array (integer range <>) of OPERATION;
  type TIME_PARAM is array (integer range <>) of time;
  type INT_PARAM is array (integer range <>) of integer;

  constant OP_A : OP_ARRAY := (

-- general settings

    1  => ("set       ", 3, "ds    ", 2, SET_T, SET, p_ds),
    2  => ("set       ", 3, "dh    ", 2, SET_T, SET, p_dh),
    3  => ("set       ", 3, "es    ", 2, SET_T, SET, p_es),
    4  => ("set       ", 3, "eh    ", 2, SET_T, SET, p_eh),
    5  => ("set       ", 3, "width ", 4, SET_I, SET, p_width),
    6  => ("set       ", 3, "per   ", 3, SET_T, SET, p_per),
    7  => ("set       ", 3, "depth ", 3, SET_I, SET, p_dep),
    8  => ("set       ", 3, "mode  ", 3, SET_I, SET, p_mode),
    9  => ("init      ", 3, "16    ", 2, INIT16, SET, p_null),  -- illegal
    10 => ("init      ", 3, "33    ", 2, INIT33, SET, p_null),  -- illegal
    11 => ("init      ", 3, "66    ", 2, INIT66, SET, p_null),  -- illegal

-- procedures
    12 => ("xxxxxxxxxx", 3, "xxxxxx", 0, FINISH, PROC, p_null),
    13 => ("finish    ", 3, "      ", 0, FINISH, PROC, p_null),
    14 => ("take      ", 3, "data  ", 3, TAKE, PROC, p_null),
    15 => ("put       ", 3, "url   ", 3, PUTU, PROC, p_null),
    16 => ("reset     ", 3, "      ", 0, RESET, PROC, p_null),

-- atomics
    17 => ("assert    ", 3, "utdo  ", 3, ASSERT_U, ATOMIC, p_null),
    18 => ("assert    ", 3, "ureset", 3, ASSERT_R, ATOMIC, p_null),
    19 => ("wait      ", 3, "for   ", 3, WAIT_F, ATOMIC, p_null),
    20 => ("wait      ", 3, "clk   ", 3, WAIT_C, ATOMIC, p_null),
    21 => ("wait      ", 3, "ldown ", 3, WAIT_D, ATOMIC, p_null),
    22 => ("wait      ", 3, "uxoff ", 3, WAIT_U, ATOMIC, p_null));

-------------------------------------------------------------------------------
-->              output constants                                           <--
-------------------------------------------------------------------------------

  constant SOUT_LCLK   : string := "--> LCLK    ";
  constant SOUT_LD     : string := "----> LCTRL# LDERR# LD(1:32) ";
  constant SOUT_LCTRL  : string := "----> LCTRL#  ";
  constant SOUT_LWEN   : string := "------> LWEN#   ";
  constant SOUT_UXOFF  : string := "<-------- UXOFF#";
  constant SOUT_URL    : string := "<-------- URL#(1:4) ";
  constant SOUT_URESET : string := "----------> URESET# ";
  constant SOUT_LDOWN  : string := "--------------> LDOWN#  ";
  constant SOUT_LDERR  : string := "--------------> LDERR#  ";
  constant SOUT_UDW    : string := "----------------> UDW     ";


--
  procedure toggle (signal sig : inout std_logic);
--
  procedure exe_action(
    op                 : in    OPERATION;
    variable comm      : inout COMMAND;
    signal tp          : inout TIME_PARAM;
    signal ip          : inout INT_PARAM;
    signal NOT_EMPTY_n : in    std_logic;
    signal LCLK        : in    std_logic;
    signal LWEN_n      : in    std_logic;
    signal URL         : inout std_logic_vector(1 to 4);
    signal UDW         : out   std_logic_vector(1 to 2);
    signal URESET_n    : out   std_logic;
    signal UTDO_n      : out   std_logic;
    signal LDOWN_n     : in    std_logic;
    signal UXOFF_n     : in    std_logic;
    signal STROBE_OUT  : inout std_logic;
    signal FRESET_n    : out   std_logic;
    signal last_up     : in    time;
    signal last_down   : in    time
    );
--
  function serv_dist(mode : integer; period : time) return time;
--
end ROMB_DEFI;

-- ***************************************************************

package body ROMB_DEFI is

  procedure toggle (signal sig : inout std_logic) is
  begin
    if sig = '0' then
      sig <= '1';
    else
      sig <= '0';
    end if;
  end toggle;

--
  function serv_dist(mode : integer; period : time) return time is
  begin
    case mode is
      when 0 =>
        return period;
      when others =>
        assert false report
          "?ROMB server mode not implemented"
          severity failure;
		return period;  -- to remove compiler warning
    end case;
  end serv_dist;
--
  procedure exe_action(
    op                 : in    OPERATION;
    variable comm      : inout COMMAND;
    signal tp          : inout TIME_PARAM;
    signal ip          : inout INT_PARAM;
    signal NOT_EMPTY_n : in    std_logic;
    signal LCLK        : in    std_logic;
    signal LWEN_n      : in    std_logic;
    signal URL         : inout std_logic_vector(1 to 4);
    signal UDW         : out   std_logic_vector(1 to 2);
    signal URESET_n    : out   std_logic;
    signal UTDO_n      : out   std_logic;
    signal LDOWN_n     : in    std_logic;
    signal UXOFF_n     : in    std_logic;
    signal STROBE_OUT  : inout std_logic;
    signal FRESET_n    : out   std_logic;
    signal last_up     : in    time;
    signal last_down   : in    time
    ) is
    variable qu  : natural;
    variable dt  : integer;
    variable tm  : time;
    variable ll  : line;
    variable udv : std_logic_vector(1 to 4);
  begin

    if op.qval /= 0 then
      qu := 2;
    else
      qu := 1;
    end if;

    case op.act is

-----------------------------------------------------------------------------
-->                            atomics                                    <--
-----------------------------------------------------------------------------
      when ATOMIC =>
        case op.code is
          when WAIT_F =>
            ll := new string'(comm(qu+1).all &" "& comm(qu+2).all);
            READ(ll, tm);
            wait for tm;
          when WAIT_C =>
            if up_down(comm(qu+1).all) then
              wait until LCLK'event and LCLK = '1';
            else
              wait until LCLK'event and LCLK = '0';
            end if;
          when WAIT_U =>
            if up_down(comm(qu+1).all) then
              wait until UXOFF_n'event and UXOFF_n = '1';
            else
              wait until UXOFF_n'event and UXOFF_n = '0';
            end if;
          when WAIT_D =>
            if up_down(comm(qu+1).all) then
              wait until LDOWN_n'event and LDOWN_n = '1';
            else
              wait until LDOWN_n'event and LDOWN_n = '0';
            end if;
          when ASSERT_U =>
            UTDO_n <= slogic(comm(qu+1).all);
          when ASSERT_R =>
            URESET_n <= slogic(comm(qu+1).all);
          when others =>
            assert false report
              "? - unrecognized ATOMIC operation"
              severity failure;
        end case;
-----------------------------------------------------------------------
-->                           SETTINGS                              <--
-----------------------------------------------------------------------
      when SET =>
        case op.code is
          when SET_T =>
            ll         := new string'(comm(qu+1).all &" "& comm(qu+2).all);
            READ(ll, tm);
            tp(op.pos) <= tm;
          when SET_I =>
            READ(comm(qu+1), dt);
            ip(op.pos) <= dt;
            if op.pos = p_width then
              case dt is
                when 32 =>
                  UDW <= "00";
                when 16 =>
                  UDW <= "10";
                when 8 =>
                  UDW <= "01";
                when others =>
                  assert false report
                    "? illegal data width"
                    severity failure;
              end case;
            end if;

-- INIT actions are not implemented anymore as S-LINK spec has only
-- one set of parameters
-- when INIT16  =>
--                                  tp(p_ds)    <= 13 ns;
--                                  tp(p_dh)    <=  1 ns;
--                                  tp(p_es)    <= 13 ns;
--                                  tp(p_eh)    <=  1 ns;
--                                  tp(p_per)   <= 60 ns;
-- when INIT33  =>
--                                  tp(p_ds)    <= 10 ns;
--                                  tp(p_dh)    <=  1 ns;
--                                  tp(p_es)    <= 10 ns;
--                                  tp(p_eh)    <=  1 ns;
--                                  tp(p_per)   <= 30 ns;
-- when INIT66  =>
--                                  tp(p_ds)    <=  9 ns;
--                                  tp(p_dh)    <=  1 ns;
--                                  tp(p_es)    <=  9 ns;
--                                  tp(p_eh)    <=  1 ns;
--                                  tp(p_per)   <= 15 ns;
          when others =>
            assert false report
              "? - unrecognized SET operation"
              severity failure;
        end case;
-----------------------------------------------------------------------
-->                          PROCEDURES                             <--
-----------------------------------------------------------------------
      when PROC =>
        case op.code is

          when FINISH =>
            assert false report
              "finish statement found"
              severity failure;         -- can't be NOTE as simulation 
            wait;                       -- should be stopped

          when RESET =>
            URESET_n <= '0';
            FRESET_n <= '0';
            --wait until LDOWN_n'event and LDOWN_n='0';
            wait until LDOWN_n'event and LDOWN_n = '1';
            URESET_n <= '1';
            FRESET_n <= '1';
          when TAKE =>
            READ(comm(qu+1), dt);
            for i in 1 to dt loop
              if NOT_EMPTY_n /= '0' then
                wait until NOT_EMPTY_n'event and NOT_EMPTY_n = '0';
              end if;
              toggle(STROBE_OUT);
              wait for serv_dist(ip(p_mode), tp(p_per));
            end loop;
          when PUTU =>
            HREAD(comm(qu+1), udv);
            URL(1 to 4) <= udv(1 to 4);
          when others =>
            assert false report
              "? - unrecognized PROCEDURE"
              severity failure;
        end case;
      when others =>
        assert false report
          "? - unrecognized action"
          severity failure;
    end case;

  end exe_action;

--


end ROMB_DEFI;

--------------------------------------------------------------------------
-->                       E N D   O F   F I L E                        <--
--------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--
--  File name : romb.vhd
--  Title     : Entity and Architecture of an S-LINK ROMB 
--  Purpose   : Complete simulation model of an S-LINK ROMB
--  Author(s) : Piotr Kapusta    - Institute of Nuclear Physics, Krakow, Poland
--              Erik van der Bij - CERN Geneva
--
--  Notes     : reads  file romb_in.txt
--              writes file romb_out.txt
--
-------------------------------------------------------------------------------
-- Version | Author |  Mod.Date  |          Changes made
-------------------------------------------------------------------------------
--   1.00  |   PK   |    1996    | Original coding
--   2.01  |   EB   | 12-05-2000 | Conv_Std_Logic_Vector, settings of Tsu/h
--   2.02  |   EB   | 15-05-2000 | No message on clk down change. LDERR Tsu/h
--   2.03  |   EB   | 16-05-2000 | Removed printing bug of lderr# and lctrl#
--         |        |            |
--   2.04  |  JMH   | 06-02-2003 | generic added for setup & hold times
--         |        |            | abnormal write check altered so that
--         |        |            | testwords is accepted if UTDO_n is asserted
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

-- 8/5/00 EB Conv_Std_Logic_Vecto
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_TEXTIO.all;
use IEEE.STD_LOGIC_ARITH.all;
use std.textio.all;


entity ROMB is
  generic(
    DATA_SETUP   : time := 10 ns;       -- Data setup time
    DATA_HOLD    : time := 1 ns;        -- Data hold  time
    ENABLE_SETUP : time := 10 ns;       -- Enable setup time
    ENABLE_HOLD  : time := 1 ns;        -- Enable hold  time
    CLOCK_PERIOD : time := 25 ns;       -- Clock period
	ROMB_IN_FILENAME : string := "romb_in.txt";
	ROMB_OUT_FILENAME : string := "romb_out.txt"
    );
  port (
    LD       : in     std_logic_vector(1 to 32);
    UXOFF_n  : buffer std_logic;
    URESET_n : buffer std_logic                := '1';
    UTDO_n   : buffer std_logic                := '1';
    UDW      : buffer std_logic_vector(1 to 2) := "00";
    LCTRL_n  : in     std_logic;
    LWEN_n   : in     std_logic                := '1';
    LCLK     : in     std_logic;
    LDERR_n  : in     std_logic;
    URL      : buffer std_logic_vector(1 to 4);
    LDOWN_n  : in     std_logic
    );

end ROMB;

---------------------------------------------------------------------------

library hola_sim_lib;
use hola_sim_lib.romb_defi.all;
use hola_sim_lib.my_text.all;

architecture ARCH of ROMB is
  component FIFO
    generic (
      FIFO_SIZE : integer := 100
	);
    port (
      DATA_IN     : in  std_logic_vector(1 to 33);
      STROBE_IN   : in  std_logic;
      ENABLE_IN_n : in  std_logic;
      DATA_OUT    : out std_logic_vector(1 to 33);
      STROBE_OUT  : in  std_logic;
      THRESH      : in  std_logic_vector(1 to 16);
      NOT_EMPTY_n : out std_logic;
      WARNING_n   : out std_logic;
      FULL_n      : out std_logic;
      RESET_n     : in  std_logic
      );
  end component;

  for ROMB_FIFO : FIFO use entity hola_sim_lib.FIFO(ARCH);


---------------------------------------------------------------------------
  signal DATA        : std_logic_vector(1 to 33);
  signal DATA_OUT    : std_logic_vector(1 to 33);
  signal STROBE_OUT  : std_logic                 := '0';
  signal THRESH      : std_logic_vector(1 to 16) := "0000000000000100";
  signal NOT_EMPTY_n : std_logic;
  signal WARNING_n   : std_logic;
  signal FULL_n      : std_logic;
  signal FRESET_n    : std_logic                 := '1';

---------------------------------------------------------------------------
  constant ROMB_IN_FNAME : string := ROMB_IN_FILENAME;
  constant ROMB_OUT_FNAME : string := ROMB_OUT_FILENAME;
  file INFILE  : text open read_mode is ROMB_IN_FNAME;
  file OUTFILE : text open write_mode is ROMB_OUT_FNAME;

---------------------- internal signals ------------------------------------
  signal last_up   : time := 0 ns;      -- stored last change up of LCLK
  signal last_down : time := 0 ns;      -- stored last change down of LCLK
  signal lwen_ch   : time := 0 ns;      -- stored last change of LWEN# 
  signal ld_ch     : time := 0 ns;      -- stored last change of LD 
  signal lctrl_ch  : time := 0 ns;      -- stored last change of LCTRL#
  signal lderr_ch  : time := 0 ns;      -- stored last change of LDERR#

---------------------- parameters ------------------------------------------
  signal t_p : TIME_PARAM(p_ds to p_fin) := (
    p_ds  => DATA_SETUP,  --  10 ns,  -- Data setup time
    p_dh  => DATA_HOLD,  --  1 ns,  -- Data hold  time
    p_es  => ENABLE_SETUP,  -- 10 ns,  -- Enable setup time
    p_eh  => ENABLE_HOLD,  --  1 ns,  -- Enable hold  time
    p_per => CLOCK_PERIOD,  -- 25 ns,  -- Clock period
    p_fin => 0 ns                       -- ? unused
    );

  signal i_p : INT_PARAM(p_width to p_mode) := (
-- initial defaults
    p_width => 32,                      -- 32 bits 
    p_dep   => 4,   -- if the fifo has four words then  UXOFF# is asserted
    p_mode  => 0);                      -- mode nmb 0


begin
  DATA(1 to 32) <= LD(1 to 32);
  DATA(33)      <= LCTRL_n;
  UXOFF_n       <= WARNING_n;

  THRESH(1 to 16) <= Conv_Std_Logic_Vector(i_p(p_dep), 16);

  ROMB_FIFO : FIFO
--generic map ( FIFO_SIZE => 1000)
    port map (DATA,
              LCLK,
              LWEN_n,
              DATA_OUT,
              STROBE_OUT,
              THRESH,
              NOT_EMPTY_n,
              WARNING_n,
              FULL_n,
              FRESET_n); 


------------------------------------------------------------------------
-->                      FILE INTERPRETER                            <--
------------------------------------------------------------------------

  FI : process
    variable i, length, tnmb, cnmb : integer;
    variable comm                  : COMMAND(1 to 10);
    variable ok                    : boolean;

  begin
    while not endfile(INFILE) loop
      get_command(INFILE, comm, tnmb);

      if tnmb /= 0 then

        SEARCH : for i in OP_A'range loop

          lstrcmpn(comm(1), OP_A(i).name, OP_A(i).val, ok);
          if ok = true then
            if (OP_A(i).qval /= 0) and tnmb >= 2 then
              lstrcmpn(comm(2), OP_A(i).qual, OP_A(i).qval, ok);
              if ok then
                cnmb := i;
                exit SEARCH;
              end if;
            else
              cnmb := i;
              exit SEARCH;
            end if;
          end if;
        end loop SEARCH;

        if ok then
          print_string_time(OUTFILE,
                            "###### found command: "&OP_A(cnmb).name&" "&OP_A(cnmb).qual, NOW);
          exe_action(OP_A(cnmb),
                     comm,
                     t_p,
                     i_p,
                     NOT_EMPTY_n,
                     LCLK,
                     LWEN_n,
                     URL,
                     UDW,
                     URESET_n,
                     UTDO_n,
                     LDOWN_n,
                     UXOFF_n,
                     STROBE_OUT,
                     FRESET_n,
                     last_up,
                     last_down
                     );
        else
          assert false report "? unrecognized: " & comm(1).all
            severity failure;
        end if;
      end if;
    end loop;
    assert false report "end of file reached"
      severity note;
    wait;
  end process FI;


--------------------------------------------------------------------------
-->                    UXOFF_monitor                                     <--  
--------------------------------------------------------------------------
  UXOFF_monitor : process(UXOFF_n)

  begin
    if UXOFF_n = '0' then
      print_string_time(OUTFILE, SOUT_UXOFF & "down", NOW);
    else
      print_string_time(OUTFILE, SOUT_UXOFF & "up", NOW);
    end if;
  end process UXOFF_monitor;


--------------------------------------------------------------------------
-->                    URL_monitor                                     <--  
--------------------------------------------------------------------------
  URL_monitor : process (URL)
  begin
    print_s_vector_time(OUTFILE, SOUT_URL & "set to: ", URL, NOW);
  end process URL_monitor;


-------------------------------------------------------------------------
-->                     RESET monitor                                 <--
-------------------------------------------------------------------------
  RESET_monitor : process
    variable e_cnt : integer;
  begin
    wait until URESET_n'event and URESET_n = '0';
    print_string_time(OUTFILE, SOUT_URESET & "down", NOW);
    e_cnt := 0;
    RES : loop
      wait until LCLK'event and LCLK = '1';
      e_cnt := e_cnt + 1;
      if (e_cnt > 4) and (LDOWN_n = '1') then
        print_string(OUTFILE, "?RESET - trl number of cycles exceeded");
        assert false report
          "?RESET - trl number of cycles exceeded"
          severity error;
      else
        if LDOWN_n = '0' then
          exit RES;
        end if;
      end if;
    end loop RES;

    wait until URESET_n'event and URESET_n = '1';
    print_string_time(OUTFILE, SOUT_URESET & "up", NOW);
  end process RESET_monitor;


-------------------------------------------------------------------------
-->                     LDOWN monitor                                 <--
-------------------------------------------------------------------------
  LDOWN_monitor : process(LDOWN_n)
  begin
    case LDOWN_n is
      when '0' =>
        print_string_time(OUTFILE, SOUT_LDOWN & "down", NOW);
      when '1' =>
        print_string_time(OUTFILE, SOUT_LDOWN & "up", NOW);
      when others =>
        print_string_time(OUTFILE, SOUT_LDOWN & "unknown", NOW);
    end case;
  end process LDOWN_monitor;


-------------------------------------------------------------------------
-->                      UDW  monitor                                 <--
-------------------------------------------------------------------------
  UDW_monitor : process(UDW)
  begin
    print_string_time(OUTFILE, SOUT_UDW & "set", NOW);
  end process UDW_monitor;


--------------------------------------------------------------------------
-->                       LWEN_monitor                                 <--  
--------------------------------------------------------------------------
  LWEN_monitor : process(LWEN_n)
  begin
    if LWEN_n = '0' then
      print_string_time(OUTFILE, SOUT_LWEN & "down", NOW);
    else
      print_string_time(OUTFILE, SOUT_LWEN & "up", NOW);
    end if;
    lwen_ch <= NOW;                     -- store time of change


-------- check hold of the LWEN# signal ------------------ CHECK --------
    if ((NOW - last_up) < t_p(p_eh)) and NOW /= 0 fs then
      print_string_time(OUTFILE, "?LWEN hold tenh violation", NOW);
      print_string_time(OUTFILE, "last LCLK change at: ", last_up);
      assert false report
        "?LWEN# hold time  violation"
        severity error;
    end if;
  end process LWEN_monitor;


--------------------------------------------------------------------------
-->                       LD_monitor                                   <--
--                                                                      --
--   Checks hold time of LD, LCTRL_n and LDERR_n                        --
--------------------------------------------------------------------------
  LD_monitor : process(LD, LCTRL_n, LDERR_n)
-- variable ll1 : LINE;
-- variable ll2 : LINE;
    variable bolo  : string(1 to 4);
    variable bolo1 : string(1 to 4);
  begin

    if LCTRL_n = '0' then bolo  := "down";
    else bolo                   := "up  ";
    end if;
    if LDERR_n = '0' then bolo1 := "down";
    else bolo1                  := "up  ";
    end if;

    print_s_vector_time(OUTFILE, SOUT_LD & " set to: " & bolo & " " & bolo1 &
                        " ", LD, NOW);

    if LD'event then
      ld_ch <= NOW;
      --------- check hold of the LD signals -------------------- CHECK --------
      if ((NOW - last_up) < t_p(p_dh)) and NOW /= 0 fs then
        print_string_time(OUTFILE, "?LD hold tdh violation", NOW);
        print_string_time(OUTFILE, "last LCLK change at: ", last_up);
        assert false report
          "?LD hold time violation"
          severity error;
      end if;
    end if;

    if LCTRL_n'event then
      lctrl_ch <= NOW;
      --------- check hold of the LCTRL# signal ----------------- CHECK --------
      if ((NOW - last_up) < t_p(p_dh)) and NOW /= 0 fs then
        print_string_time(OUTFILE, "?LCTRL hold tdh violation", NOW);
        print_string_time(OUTFILE, "last LCLK change at: ", last_up);
        assert false report
          "?LCTRL hold time violation"
          severity error;
      end if;
    end if;

    if LDERR_n'event then
      lderr_ch <= NOW;
      --------- check hold of the LDERR# signal ----------------- CHECK --------
      if ((NOW - last_up) < t_p(p_dh)) and NOW /= 0 fs then
        print_string_time(OUTFILE, "?LDERR# hold tdh violation", NOW);
        print_string_time(OUTFILE, "last LCLK change at: ", last_up);
        assert false report
          "?LDERR# hold time violation"
          severity error;
      end if;
    end if;
  end process LD_monitor;


--------------------------------------------------------------------------
-->                       LCLK_monitor                                 <--  
--------------------------------------------------------------------------
  LCLK_monitor : process(LCLK)
  begin
    case LCLK is
      when '0' =>
        --  print_string_time(OUTFILE,SOUT_LCLK & "down",NOW);
        last_down <= NOW;
      when '1' =>
        print_string_time(OUTFILE, SOUT_LCLK & "up", NOW);
        last_up <= NOW;

--------- check setup of the LWEN# signal ----------------- CHECK --------
        if ((NOW - lwen_ch) < t_p(p_es)) and NOW /= 0 fs then
          print_string_time(OUTFILE, "?LWEN setup tens violation", NOW);
          print_string_time(OUTFILE, "last LWEN# change at: ", lwen_ch);
          assert false report
            "?LWEN setup time violation"
            severity error;
        end if;
--------------------------------------------------------------------------
--------- check setup of the LD signals ------------------- CHECK --------
        if ((NOW - ld_ch) < t_p(p_ds)) and NOW /= 0 fs then
          print_string_time(OUTFILE, "?LD setup tds violation", NOW);
          print_string_time(OUTFILE, "last LD change at: ", ld_ch);
          assert false report
            "?LD setup time violation"
            severity error;
        end if;
--------------------------------------------------------------------------
--------- check setup of the LCTRL signal ----------------- CHECK --------
        if ((NOW - lctrl_ch) < t_p(p_ds)) and NOW /= 0 fs then
          print_string_time(OUTFILE, "?LCTRL# setup tds violation", NOW);
          print_string_time(OUTFILE, "last LCTRL# change at: ", lctrl_ch);
          assert false report
            "?LCTRL# setup time violation"
            severity error;
        end if;
--------------------------------------------------------------------------
--------- check setup of the LDERR signal ----------------- CHECK --------
        if ((NOW - lderr_ch) < t_p(p_ds)) and NOW /= 0 fs then
          print_string_time(OUTFILE, "?LDERR# setup tds violation", NOW);
          print_string_time(OUTFILE, "last LDERR# change at: ", lderr_ch);
          assert false report
            "?LDERR# setup time violation"
            severity error;
        end if;
--------------------------------------------------------------------------
--- check if abnormal write when LDOWN_n = '0' ----------- CHECK --------
        if LWEN_n = '0' and LDOWN_n = '0' and UTDO_n = '1' then
          print_string_time(OUTFILE, "?LDOWN# active when writing ", NOW);
          assert false report
            "?LDOWN# active when writing"
            severity error;
        end if;
--------------------------------------------------------------------------
      when others =>
        print_string_time(OUTFILE, SOUT_LCLK & "unknown", NOW);
    end case;

  end process LCLK_monitor;
  
end ARCH;
--------------------------------------------------------------------------
-->                       E N D   O F   F I L E                        <--
--------------------------------------------------------------------------


























--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--                                                                            --
-- File name   :     TLK2501_t-check-FSM.vhd                                  --
--                                                                            --
-- Author      : Michael Fauland, CERN, EP-Division                           --
--               Michael.Fauland@cern.ch                                      --
--                                                                            --
--               CERN - European Organization for Nuclear Research            --
--               EP-division, G24000                                          --
--               Michael Fauland                                              --
--               CH-1211 Geneva 23                                            --
--               Switzerland                                                  --
--                                                                            --
--                                                                            --
--                                COPYRIGHT CERN, 2001                        --
--                                                                            --
--                                                                            --
-- Description : timing check FSM for  TLK2501_t-check from mf                --
--               Added transmission error selection.                          --
--                                                                            --
-- EDMS item id: --                                                           --
--                                                                            --
-- Notes       : --                                                           --
--                                                                            --
--------------------------------------------------------------------------------
--                           Revision History                                 --
--------------------------------------------------------------------------------
-- Version |    Date     |Author| Modifications                               --
--------------------------------------------------------------------------------
--         | 05-Dec-2001 |  MF  |                                             --
--   1.1   | 17-Apr-2002 | ARG  | Delay outputs to satisfy hold time cond.    --
--         |             |      | Removed comments                            --
--   1.2   | 21-May-2002 | ARG  | Input FIBER_DISC added                      --
--   1.3   | 30-May-2002 | ARG  | RX_CLK held low when device down            --
--==============================================================================
--/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
--==============================================================================

library IEEE;
use IEEE.std_logic_1164.all;

--==============================================================================
entity tlk2501_FSM is  --  ||  --
--==============================================================\/==============

  port
    (
      -- Tx - part 

      TX_EN : in std_logic;
      TX_ER : in std_logic;

      TD : in std_logic_vector (15 downto 0);

      GTX_CLK    : in std_logic;        -- used as CLK for FSM
      TESTEN     : in std_logic;
      ENABLE     : in std_logic;        -- used as RESET_n (low active) for FSM
      FIBER_DISC : in std_logic;  -- Fiber disconnected, device goes down       


      -- Rx - part 

      RX_ER  : out std_logic;           --    and also  'PRBS_PASS'
      RX_CLK : out std_logic;
      RX_DV  : out std_logic;           --    and also 'LOS'

      RD       : out std_logic_vector (15 downto 0);
      TX_ERROR : in  std_logic

      );
--==============================================================/\==============    
end tlk2501_FSM;  --  ||  --
--==============================================================================

--==============================================================================
architecture beh_tlk2501_FSM of tlk2501_FSM is  --  ||  --
--==============================================================\/==============
  
  type state_values is (acq0, acq1, acq2, sync,
                             check, cs0, cs1, cs2);
  signal pres_state, next_state : state_values;

  signal int_iv_3 : std_logic;
  signal incr     : std_logic;

  signal int_data_in  : std_logic_vector(2 downto 0);
  signal int_data_out : std_logic_vector(2 downto 0);

  signal rd_sig : std_logic_vector(15 downto 0);

  signal int_data_out2 : std_logic_vector(2 downto 0);

  signal rd_sig2 : std_logic_vector(15 downto 0);

begin

  ------------------------------------------------------------------------------
  -- Process(es)
  ------------------------------------------------------------------------------  

  --~~~ FSM register ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  STATE_REG : process (GTX_CLK, ENABLE, FIBER_DISC, next_state)
  begin
    if(ENABLE = '0' or FIBER_DISC = '1') then
      pres_state <= acq0;

    elsif (GTX_CLK = '1') and (GTX_CLK'event) then
      pres_state <= next_state;

    end if;
  end process STATE_REG;

  --~~~ FSM combinational block ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  FSM : process (pres_state, int_data_in)
  begin
    
    case pres_state is
      
      when acq0 =>
        case int_data_in is
          when "000"  => next_state <= acq1;
          when "001"  => next_state <= acq1;
          when "010"  => next_state <= sync;
          when "011"  => next_state <= acq0; -- error propagation - why acq0 and not sync?
          when others => null;
        end case;

      when acq1 =>
        case int_data_in is
          when "000"  => next_state <= acq2;
          when "001"  => next_state <= acq2;
          when "010"  => next_state <= sync;
          when "011"  => next_state <= acq0;
          when others => null;
        end case;
        
      when acq2 =>
        case int_data_in is
          when "000"  => next_state <= sync;
          when "001"  => next_state <= sync;
          when "010"  => next_state <= sync;
          when "011"  => next_state <= acq0;
          when others => null;
        end case;
        
      when sync =>
        case int_data_in is
          when "000"  => next_state <= sync;
          when "001"  => next_state <= sync;
          when "010"  => next_state <= sync;
          when "011"  => next_state <= check;
          when others => null;
        end case;
        
      when check =>
        case int_data_in(2) is
          when '1' => next_state <= acq0;
          when '0' =>
            case int_data_in(1 downto 0) is
              when "00"   => next_state <= cs0;
              when "01"   => next_state <= cs0;
              when "10"   => next_state <= cs0;
              when "11"   => next_state <= check;
              when others => null;
                             
            end case;
            
          when others => null;
        end case;
        
      when cs0 =>
        case int_data_in is
          when "000"  => next_state <= cs1;
          when "001"  => next_state <= cs1;
          when "010"  => next_state <= cs1;
          when "011"  => next_state <= check;
          when others => null;
        end case;
        
      when cs1 =>
        case int_data_in is
          when "000"  => next_state <= cs2;
          when "001"  => next_state <= cs2;
          when "010"  => next_state <= cs2;
          when "011"  => next_state <= check;
          when others => null;
        end case;
        
      when cs2 =>
        case int_data_in is
          when "000"  => next_state <= sync;
          when "001"  => next_state <= sync;
          when "010"  => next_state <= sync;
          when "011"  => next_state <= check;
          when others => null;
        end case;

    end case;
    
    
  end process FSM;


  --~~~ Moore output definition using pres_state only ~~~~~~~~~~~~~~~~~~~~~~~~

  OUTPUTS : process (TD, pres_state, int_data_in, tx_error)
  begin
    
    case pres_state is
      when acq0 =>
        int_data_out <= "011";
        rd_sig       <= "1111111111111111";

      when acq1 =>
        int_data_out <= "011";
        rd_sig       <= "1111111111111111";
        
      when acq2 =>
        int_data_out <= "011";
        rd_sig       <= "1111111111111111";

      when sync =>
        case int_data_in is
          when "000" =>
--                int_data_out  <= "011"; 
            int_data_out <= "000";      -- aurelio 14-mar-2002 
            rd_sig       <= "1111111111111111";
          when "001" =>
            int_data_out <= "001";
            rd_sig       <= "1111111111111111";
          when "010" =>
            int_data_out <= "010";
            if (TX_ERROR = '0') then
              rd_sig <= TD;             -- only in normal mode 
            else
              rd_sig(15 downto 9) <= TD(15 downto 9);  -- only in normal mode 
              rd_sig(2 downto 0)  <= TD(2 downto 0);   -- only in normal mode 
              rd_sig(8 downto 3)  <= not TD(8 downto 3);  -- error 

            end if;
          when "011" =>
            int_data_out <= "011";
            rd_sig       <= "1111111111111111";

          when others => null;
                         
        end case;
        
      when check =>
        int_data_out <= "111";
        rd_sig       <= "1111111111111111";

        
      when cs0 =>
        int_data_out <= "011";
        rd_sig       <= "1111111111111111";
        
      when cs1 =>
        int_data_out <= "011";
        rd_sig       <= "1111111111111111";
        
      when cs2 =>
        int_data_out <= "011";
        rd_sig       <= "1111111111111111";
        
    end case;
    
  end process OUTPUTS;



  --~~~ process to generate transition from check to acq0 ~~~~~~~~~~~~~~~~~~~~

  COUNT_ERR_PROP_3 : process (GTX_CLK, ENABLE, FIBER_DISC, incr, pres_state)

    variable temp : integer;
    
    
    
  begin
    
    if (ENABLE = '0' or FIBER_DISC = '1') then
      temp := 0;
      
    elsif (((GTX_CLK'event) and (GTX_CLK = '1')) and (incr = '1') and ((pres_state = check)or
                                                                         (pres_state = cs0) or (pres_state = cs1) or (pres_state = cs2))) then
      temp := temp + 1;
    else
      temp := 0;
      
    end if;

    if (temp >= 3) then
      int_iv_3 <= '1';
    else
      int_iv_3 <= '0';
    end if;
    
  end process COUNT_ERR_PROP_3;

  output_delay : process
  begin
    wait until GTX_CLK'event and GTX_CLK = '1';
    int_data_out2 <= int_data_out;
    rd_sig2       <= rd_sig;
  end process;

  clock_output : process(enable, gtx_clk)
  begin
    if(ENABLE = '0') then
      rx_clk <= '0';
    else
      RX_CLK <= GTX_CLK;
    end if;
  end process;
  ------------------------------------------------------------------------------
  -- Concurrent Assignments
  ------------------------------------------------------------------------------  

  int_data_in(1) <= TX_EN;
  int_data_in(0) <= TX_ER;


  int_data_in(2) <= int_iv_3;

--    RX_DV             <=   int_data_out(1);
--    RX_ER             <=   int_data_out(0);    

--    RD                <=   rd_sig;

  RX_DV <= int_data_out(1)'delayed(3 ns);
  RX_ER <= int_data_out(0)'delayed(3 ns);

  RD <= rd_sig'delayed(3 ns);

  incr <= int_data_out(2);

--    RX_CLK            <=   GTX_CLK;



--==============================================================/\==============
end beh_tlk2501_FSM;  --  ||  --
--==============================================================================




--==============================================================================
--==============================================================================
--                           E N D   O F   F I L E                            --
--==============================================================================
--==============================================================================
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--                                                                            --
-- File name   :     TLK2501_t-check.vhd                                      --
--                                                                            --
-- Author      : Michael Fauland, CERN, EP-Division                           --
--               Michael.Fauland@cern.ch                                      --
--                                                                            --
--               CERN - European Organization for Nuclear Research            --
--               EP-division, G24000                                          --
--               Michael Fauland                                              --
--               CH-1211 Geneva 23                                            --
--               Switzerland                                                  --
--                                                                            --
--                                                                            --
--                                COPYRIGHT CERN, 2001                        --
--                                                                            --
--                                                                            --
-- Description : timing check for  TI 'TLK2501' serializer/deserializer       --
--               only one channel, 'ideal' transmission  without delay        --
--               (for the 'way back' just swap direction by                   --
--                respectively port mapping )                                 --
--                                                                            --
-- EDMS item id: --                                                           --
--                                                                            --
-- Notes       : not all functions all implemented such as:                   --
--               - LOOPEN                                                     --
--               - PRBS                                                       --
--                                                                            --
--                                                                            --             
--                                                                            --
--                ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~           --
--   TX_EN     -> | TLK2501_t-check                               | -> RX_ER  --
--   TX_ER     -> |                                               | -> RX_CLK --
--                |  ---------                        ---------   | -> RX_DV  --
--   TD 15..0) => |  |TLK2501|                        |TLK2501|   |           --
--                |  |       |                        |       |   | => RD(15..0) 
--   GTX_CLK   -> |  |       | --transmission-line--> |       |   |           --
--   TESTEN    -> |  |       |                        |       |   |           --    
--   ENABLE    -> |  |       |                        |       |   |           --
--                ~~~|.......|~~~~~~~~~~~~~~~~~~~~~~~~|.......|~~~~           --
--                   |       |                        |       |               --
--                   |       |                        |       |               --
--                   |       | <--transmission-line-- |       |               --
--                   |       |                        |       |               --
--                   |       |                        |       |               --
--                   ---------                        ---------               --
--                                                                            --
--                                                                            --
--                                                                            --
-- Synplify    : not ment for beeing synthesized, simulation only             --
--                                                                            --
--------------------------------------------------------------------------------
--                           Revision History                                 --
--------------------------------------------------------------------------------
-- Version |    Date     |Author| Modifications                               --
--------------------------------------------------------------------------------
--   0.1   | 05-Dec-2001 |  MF  |                                             --
--   0.2   | 15-Abr-2002 | ARG  | Removed unused signals                      --
--         |             |      | Internal signals .._sig reused, otherwise   --
--         |             |      | checking processes cannot work              --
--   0.3   | 17-Apr-2002 | ARG  | TX_ERROR added, inputs registered           --
--   0.4   | 22-Apr-2002 | ARG  | Assertion severity changed to ERROR         --
--         |             | ARG  | Removed commented lines                     --
--         |             | ARG  | Assertion separated for RXD and RX_ER/RX_DV --
--   0.5   | 26-Apr-2002 | ARG  | Set up times corrected                      --
--   0.6   | 21-May-2002 | ARG  | Input FIBER_DISC added                      --
--==============================================================================
--/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
--==============================================================================

library IEEE;
use IEEE.std_logic_1164.all;

-- AKK additional includes
use std.textio.all;
use work.my_text.all;

--==============================================================================
--==============================================================\/==============
entity tlk2501 is
  generic
    (
      in_setup : time := 1.5 ns;
      in_hold  : time := 0.4 ns;

      out_setup : time := 3.0 ns;       -- @ 125 MHz --FIXME? for 100 MHz
      out_hold  : time := 3.0 ns        -- @ 125 MHz
      );

  port
    (

      -- Tx - part 

      LOOPEN : in std_logic;
      PRBSEN : in std_logic;

      TX_EN : in std_logic;
      TX_ER : in std_logic;

      TD : in std_logic_vector (15 downto 0);

      GTX_CLK    : in std_logic;
      TESTEN     : in std_logic;
      ENABLE     : in std_logic;
      FIBER_DISC : in std_logic := '0';



      -- Rx - part 

      RX_ER  : out std_logic;           --    and also  'PRBS_PASS'
      RX_CLK : out std_logic;
      RX_DV  : out std_logic;           --    and also 'LOS'

      RD       : out std_logic_vector (15 downto 0);
      TX_ERROR : in  std_logic := '0'
      );
--==============================================================/\==============    
end tlk2501;  --  ||  --
--==============================================================================

library hola_sim_lib;

--==============================================================================
architecture beh_tlk2501 of tlk2501 is  --  ||  --
--==============================================================\/==============

  ------------------------------------------------------------------------------
  -- Component(s) declaration
  ------------------------------------------------------------------------------  


  component tlk2501_FSM
    port
      (
        -- Tx - part 

        TX_EN : in std_logic;
        TX_ER : in std_logic;

        TD : in std_logic_vector (15 downto 0);

        GTX_CLK    : in std_logic;      -- used as CLK for FSM
        TESTEN     : in std_logic;
        ENABLE     : in std_logic;      -- used as RESET_n (low active) for FSM
        FIBER_DISC : in std_logic;      -- fiber is dosconnected, device down


        -- Rx - part 

        RX_ER  : out std_logic;         --    and also  'PRBS_PASS'
        RX_CLK : out std_logic;
        RX_DV  : out std_logic;         --    and also 'LOS'

        RD       : out std_logic_vector (15 downto 0);
        TX_ERROR : in  std_logic

        );

  end component;
  for all : tlk2501_FSM use entity hola_sim_lib.tlk2501_FSM(beh_tlk2501_FSM);
--end for;

  -- Rx - part 

  signal RX_ER_sig  : std_logic;        --    and also  'PRBS_PASS'
  signal RX_CLK_sig : std_logic;
  signal RX_DV_sig  : std_logic;        --    and also 'LOS'

  signal RD_sig : std_logic_vector (15 downto 0);

  signal TX_ER_reg : std_logic;
  signal TX_EN_reg : std_logic;
  signal TD_reg    : std_logic_vector(15 downto 0);


begin

  ------------------------------------------------------------------------------
  -- Process(es)
  ------------------------------------------------------------------------------  
  ------------------------------------------------------------------------------  
  -- Inputs register
  ------------------------------------------------------------------------------  
  inreg : process
  begin
    wait until GTX_CLK'event and GTX_CLK = '1';
    TX_EN_reg <= TX_EN;
    TX_ER_reg <= TX_ER;
    TD_reg    <= TD;
  end process;


  CHECK_IN_S_H : process (GTX_CLK, TX_ER, TX_EN) -- AKK: TD_le not in sensitivity list?
    
    variable GTX_CLK_le : time := 0 ns;  --  '..._le'  ->  last event
    variable TX_ER_le   : time := 0 ns;
    variable TX_EN_le   : time := 0 ns;
    variable TD_le      : time := 0 ns;
    
  begin

    -- hold check
    if (TX_ER'event or TX_EN'event) then  -- or  TD'event 
    
      assert (NOW = 0 ns) or ((NOW - GTX_CLK_le) >= in_hold) or (NOW - GTX_CLK_le)=0 ns -- AKK added 0 ns check FIXME
        report "Hold Violation on input (TX_ER or TX_EN )"
        severity error;

      TX_ER_le := NOW;
      TX_EN_le := NOW;
      TD_le    := NOW;
      
    end if;

    -- setup check
    if ((GTX_CLK'event) and (GTX_CLK = '1')) then
	
      assert (NOW = 0 ns) or ((NOW - TX_ER_le) >= in_setup)
        or ((NOW - TX_EN_le) >= in_setup)
        or ((NOW - TD_le) >= in_setup)
        report "setup violation on input (TX_ER or TX_EN )"
        severity error;
      
      GTX_CLK_le := NOW;
      
    end if;
  end process CHECK_IN_S_H;


  CHECK_IN_S_H_TD : process (GTX_CLK, TD)
    
    variable GTX_CLK_le : time := 0 ns;  --  '..._le'  ->  last event
    variable TD_le      : time := 0 ns;
    
  begin

    -- hold check
    if (TD'event) then
      assert (NOW = 0 ns) or ((NOW - GTX_CLK_le) >= in_hold) or (NOW - GTX_CLK_le) = 0 ns -- AKK added 0 ns check

        report "Hold Violation on input ( TD)"
        severity error;

      TD_le := NOW;
      
    end if;

    -- setup check
    if ((GTX_CLK'event) and (GTX_CLK = '1')) then
      assert (NOW = 0 ns) or ((NOW - TD_le) >= in_setup)
        
        report "setup violation on input (TD)"
        severity error;
      
      GTX_CLK_le := NOW;
      
    end if;
  end process CHECK_IN_S_H_TD;

  CHECK_OUT_S_H : process (RX_CLK_sig, RX_DV_sig, RX_ER_sig, RD_sig)
    
    variable RX_CLK_le : time := 0 ns;  --  '..._le'  ->  last event
    variable RX_DV_le  : time := 0 ns;
    variable RX_ER_le  : time := 0 ns;
    variable RD_le     : time := 0 ns;

    
  begin

    -- hold check
    if (RX_DV_sig'event or RX_ER_sig'event) then
--      assert (NOW = 0 ns)  or ((NOW - RX_CLK'last_event) >= out_hold)
--      assert ((NOW - RX_CLK'last_event) >= out_hold)
      assert ((NOW - RX_CLK_sig'last_event) >= out_hold)
        report "Hold Violation on output ( RX_DV or RX_ER)"
        severity error;

      RX_DV_le := NOW;
      RX_ER_le := NOW;

    end if;

    if (RD_sig'event) then
--      assert (NOW = 0 ns)  or ((NOW - RX_CLK'last_event) >= out_hold)
      assert (NOW = 0 ns) or ((NOW - RX_CLK_sig'last_event) >= out_hold)
        report "Hold Violation on output (RD) in"
        severity error;

      RD_le := NOW;

    end if;


    -- setup check
    if ((RX_CLK_sig'event) and (RX_CLK_sig = '1')) then
      assert ((NOW = 0 ns) or ((NOW - RX_DV_sig'last_event) >= out_setup)
              or ((NOW - RX_ER_sig'last_event) >= out_setup))


        report "setup violation on output ( RX_DV or RX_ER)"
        severity error;
      --     assert(FALSE)
      --     report "Last clock up"
      --     severity ERROR;
      
      RX_CLK_le := NOW;
      
    end if;

    if ((RX_CLK_sig'event) and (RX_CLK_sig = '1')) then
      assert ((NOW = 0 ns) or ((NOW - RD_sig'last_event) >= out_setup))
        
        report "setup violation on output (RD)"
        severity error;
      
    end if;

  end process CHECK_OUT_S_H;


  CHECK_LOOPEN : process (LOOPEN)
  begin
    if (LOOPEN = '1') then
      assert (false)
        report "LOOPEN =1, loopback ACTIVE => DOUTTXP/DOUTTXN = 'Z' "
        severity warning;

    elsif(LOOPEN = '0') then
      assert (false)
        report "LOOPEN =0, standard operational state "
        severity warning;
      
    end if;
  end process CHECK_LOOPEN;


  CHECK_PRBSEN : process (PRBSEN)
  begin
    if (PRBSEN = '1') then
      assert (false)
        report "PRBSEN =1, pseudo random bitstream can be monitored on the"
        severity warning;

    elsif(PRBSEN = '0') then
      assert (false)
        report "PRBSEN = 0, no PRBS - test "
        severity warning;
      
    end if;
  end process CHECK_PRBSEN;



  CHECK_TESTEN : process (TESTEN)
  begin
    if (TESTEN = '1') then
      assert (false)
        report "TESTEN =1, this terminal should be unconnected or tied low  "
        severity warning;

    elsif(TESTEN = '0') then
      assert (false)
        report "TESTEN = 0, this terminal should be unconnected or tied low  "
        severity warning;
      
    end if;
  end process CHECK_TESTEN;


  CHECK_ENABLE : process (ENABLE)
  begin
    if (ENABLE = '1') then
      assert (false)
        report "ENABLE =1, TLK active  "
        severity warning;

    elsif(ENABLE = '0') then
      assert (false)
        report "ENABLE = 0, TLK in power down mode  "
        severity warning;
      
    end if;
  end process CHECK_ENABLE;

  ------------------------------------------------------------------------------
  -- Concurrent Assignments
  ------------------------------------------------------------------------------  

  RX_CLK <= RX_CLK_sig;
  RX_ER  <= RX_ER_sig;
  RX_DV  <= RX_DV_sig;
  RD     <= RD_sig;

-- end commented
  ------------------------------------------------------------------------------
  -- Component assignments
  ------------------------------------------------------------------------------  
  FSM : tlk2501_FSM
    port map
    (
      -- Tx - part --------------------------
      --      port  => signal

      TX_EN => TX_EN_reg,
      TX_ER => TX_ER_reg,

      TD => TD_reg,

      GTX_CLK    => GTX_CLK,
      TESTEN     => TESTEN,
      ENABLE     => ENABLE,
      FIBER_DISC => FIBER_DISC,

      -- Rx - part  -------------------------

      RX_ER  => RX_ER_sig,
      RX_CLK => RX_CLK_sig,
      RX_DV  => RX_DV_sig,

      RD       => RD_sig,
      TX_ERROR => TX_ERROR


      );




--==============================================================/\==============
end beh_tlk2501;  --  ||  --
--==============================================================================




--==============================================================================
--==============================================================================
--                           E N D   O F   F I L E                            --
--==============================================================================
--==============================================================================
library ieee;
use ieee.std_logic_1164.all;

package hola_sim_components is

  component FEMB
    port (
      UD       : buffer std_logic_vector(1 to 32);
      LFF_n    : in     std_logic;
      URESET_n : buffer std_logic                := '1';
      UTEST_n  : buffer std_logic                := '1';
      UDW      : buffer std_logic_vector(1 to 2) := "00";
      UCTRL_n  : buffer std_logic;
      UWEN_n   : buffer std_logic                := '1';
      UCLK     : buffer std_logic;
      LRL      : in     std_logic_vector(1 to 4);
      LDOWN_n  : in     std_logic);
  end component;

  component ROMB
    port (
      LD       : in     std_logic_vector(1 to 32);
      UXOFF_n  : buffer std_logic;
      URESET_n : buffer std_logic                := '1';
      UTDO_n   : buffer std_logic                := '1';
      UDW      : buffer std_logic_vector(1 to 2) := "00";
      LCTRL_n  : in     std_logic;
      LWEN_n   : in     std_logic                := '1';
      LCLK     : in     std_logic;
      LDERR_n  : in     std_logic;
      URL      : buffer std_logic_vector(1 to 4);
      LDOWN_n  : in     std_logic);
  end component;

  component tlk2501
    port (
      LOOPEN     : in  std_logic;
      PRBSEN     : in  std_logic;
      TX_EN      : in  std_logic;
      TX_ER      : in  std_logic;
      TD         : in  std_logic_vector (15 downto 0);
      GTX_CLK    : in  std_logic;
      TESTEN     : in  std_logic;
      ENABLE     : in  std_logic;
      FIBER_DISC : in  std_logic := '0';
      RX_ER      : out std_logic;
      RX_CLK     : out std_logic;
      RX_DV      : out std_logic;
      RD         : out std_logic_vector (15 downto 0);
      TX_ERROR   : in  std_logic := '0');
  end component;

end hola_sim_components;


