library IEEE;
use     IEEE.Std_Logic_1164.all;

entity ldc_top is
generic(
	ALTERA_XILINX    : integer :=    1;   -- 1=Altera 0=Xilinx
	SIMULATION       : integer :=    0;   -- simulation mode
	LCLK_SELECT      : integer :=    1;   -- LCLK generator
										  -- selection
										  -- 0 -> no clk gen.
										  -- 1 -> 40MHz clk gen.
										  -- 2 -> 62.5MHz clk gen.
	XCLK_FREQ        : integer :=  100;   --	Reference clock oscillator frequency
										  -- 100 -> 2.0 Gb/s link speed (default)
										  -- 125 -> 2.5 Gb/s link speed
	ACTIVITY_LENGTH  : integer :=   15;   -- ACTLED duration
	FIFODEPTH        : integer :=   512;  -- only powers of 2
	LOG2DEPTH        : integer :=    9;   -- 2log of depth
	FULLMARGIN       : integer :=   256   -- words left when LFF_N
);

port (
	POWER_UP_RST_N : in  std_logic;

	-- ldc core signals from FEMB
	LD            : out std_logic_vector(31 downto 0);
	UXOFF_N       : in  std_logic;
	URESET_N      : in  std_logic;
	UTDO_N        : in  std_logic;
	LCTRL_N       : out std_logic;
	LWEN_N        : out std_logic;
	LCLK          : out std_logic;  -- 40 MHz
	LDERR_N       : out std_logic;
	URL           : in  std_logic_vector( 3 downto 0);
	LDOWN_N       : out std_logic;

	-- LED outputs
	TESTLED_N     : out std_logic;       -- Red
	LDERRLED_N    : out std_logic;       -- Red.
	LUPLED_N      : out std_logic;        -- Green, wired to control block (=NOT LDOWN)
	FLOWCTLLED_N  : out std_logic;        -- Red
	ACTIVITYLED_N : out std_logic;        -- Green
		
	-- Free-running clocks from oscillators
	XCLK          : in std_logic;   -- 100 MHz (to sample receiver status)
	SLOWCLK       : in std_logic;   -- 50 MHz (for reconfiguration & calibration)
	LCLK_IN       : in   std_logic;
	
	-- Cyclone transceiver ports
	REFCLK        : in std_logic;   -- 100 MHz, LVDS
	GXB_TX0       : out std_logic;
	GXB_RX0       : in std_logic
   );

end ldc_top;

architecture structure of ldc_top is
	signal powerup_n  : STD_LOGIC;
	signal powerup_n_d1  : STD_LOGIC;
	signal powerup_n_d2  : STD_LOGIC;
	signal CALIBCLK  : STD_LOGIC;
	-- Signals for ldc core
	signal dll_reset_gnd : STD_LOGIC;
	-- transmitter signals
	signal tx_clk_sig  : STD_LOGIC;
	signal tx_en_sig   : STD_LOGIC;
	signal tx_er_sig   : STD_LOGIC;
	signal txd_sig     : STD_LOGIC_VECTOR(15 DOWNTO 0);
	-- receiver signals
	signal rx_clk0_sig : STD_LOGIC;
	signal rx_dv0_sig  : STD_LOGIC;
	signal rx_er0_sig  : STD_LOGIC;
	signal rx_locked0_sig  : STD_LOGIC;
	signal rxd0_sig    : STD_LOGIC_VECTOR(15 DOWNTO 0);

	-- LOS filtering
	signal LOS_stable  : STD_LOGIC;
	signal LOS_d : std_logic_vector(4 downto 0);
	
	-- old TLK wrapper (deprecated)
	component altgx_tlk_single is
	generic(
	  SIMULATION  : integer :=  0
	);
	port (
		POWER_UP_RST_N    : in  std_logic;
		-- transmitter
		TXD            : in std_logic_vector(15 downto 0);
		TX_EN          : in std_logic;
		TX_ER          : in std_logic;
		TX_CLK         : out std_logic;  -- generated internally, passed to LSC/LDC core
		-- receiver (from DAQ):
		RXD0           : out std_logic_vector(15 downto 0); -- Received parallel data
		RX_CLK0        : out std_logic;         -- Recovered clock.
		RX_LOCKED0     : out std_logic;
		RX_ER0         : out std_logic;
		RX_DV0         : out std_logic;
		-- Cyclone transceiver ports (connect to pins)
		SLOWCLK       : in std_logic;   -- 50 MHz, reconfig & calib clock
		REFCLK        : in std_logic;   -- 100 MHz, LVDS
		GXB_TX0       : out std_logic;
		GXB_RX0       : in std_logic;
		-- LOS signals
		LOS_stable   : in STD_LOGIC
	   );
	end component;

	-- new TLK wrapper
	component altgx_tlk is
		generic (
			SIMULATION       : integer;
			SIMPLE_TLK_WRAP  : integer;  -- enables "Simple" version of TLK2501 TX emulation (no disparity recalculation)
			DATAGEN_MODE     : integer;  -- 0=regular operation; 1=inject SEND_RESET sequences; 2=inject counter data
			BONDING_MODE     : integer   -- 0=single-channel; 1=dual channel via tx_coreclk; 2=dual-channel via bonding
		);
		port (
		POWER_UP_RST_N    : in  std_logic;
		-- transmitter (to both DAQ and FTK)
		TXD            : in std_logic_vector(15 downto 0);
		TX_EN          : in std_logic;
		TX_ER          : in std_logic;
		TX_CLK         : out std_logic;  -- generated internally, passed to LSC/LDC core
		ICLK           : out std_logic;
		-- receiver (from DAQ):
		RXD0           : out std_logic_vector(15 downto 0); -- Received parallel data
		RX_CLK0        : out std_logic;         -- Recovered clock.
		RX_LOCKED0     : out std_logic;
		RX_ER0         : out std_logic;
		RX_DV0         : out std_logic;
		-- receiver (from FTK):
		RXD1           : out std_logic_vector(15 downto 0); -- Received parallel data
		RX_CLK1        : out std_logic;         -- Recovered clock.
		RX_LOCKED1     : out std_logic;
		RX_ER1         : out std_logic;
		RX_DV1         : out std_logic;
		-- Cyclone transceiver ports (connect to pins)
		SLOWCLK       : in std_logic;   -- 50 MHz, altgx reconfiguation
		CALIBCLK      : in std_logic;   -- 100 MHz altgx calibration
		REFCLK0       : in std_logic;   -- 100 MHz, LVDS - altgx clock for DAQ channel MPLL
		REFCLK1       : in std_logic;   -- 100 MHz, LVDS - altgx clock for FTK channel MPLL
									    -- (should be the same as REFCLK0 to bond the channels)
		GXB_TX0       : out std_logic;
		GXB_RX0       : in std_logic;
		GXB_TX1       : out std_logic;
		GXB_RX1       : in std_logic;
		debug_bits    : out std_logic_vector( 3 downto 0);
		-- LOS signals
		LOS0_stable   : in STD_LOGIC;
	    LOS1_stable   : in STD_LOGIC;
		FTK_SEND_ACK  : in STD_LOGIC
	);
	end component;
	
	-- holaldc_core component
	component holaldc_core is
	generic (
		ALTERA_XILINX    : integer :=    1;
		SIMULATION       : integer :=    1;
		LCLK_SELECT      : integer :=    1;
		XCLK_FREQ        : integer :=  100;
		ACTIVITY_LENGTH  : integer :=   15;
		FIFODEPTH        : integer :=  512;
		LOG2DEPTH        : integer :=    9;
		FULLMARGIN       : integer :=  256
	);
	port (
		POWER_UP_RST_N : in std_logic;
		LD               : out  std_logic_vector(31 downto 0);
		UXOFF_N          : in   std_logic;
		URESET_N         : in   std_logic;
		UTDO_N           : in   std_logic;
		LCTRL_N          : out  std_logic;
		LWEN_N           : out  std_logic;
		LCLK             : out  std_logic;
		LDERR_N          : out  std_logic;
		URL              : in   std_logic_vector( 3 downto 0);
		LDOWN_N          : out  std_logic;
		TESTLED_N        : out  std_logic;
		LDERRLED_N       : out  std_logic;
		LUPLED_N         : out  std_logic;
		FLOWCTLLED_N     : out  std_logic;
		ACTIVITYLED_N    : out  std_logic;
		XCLK             : in   std_logic;
		TX_CLK           : in   std_logic;
		ENABLE           : out std_logic;
		TXD              : out   std_logic_vector(15 downto 0);
		TX_EN            : out  std_logic;
		TX_ER            : out  std_logic;
		RXD              : in   std_logic_vector(15 downto 0);
		RX_CLK           : in   std_logic;
		RX_ER            : in   std_logic;
		RX_DV            : in   std_logic;
		LCLK_IN          : in   std_logic;
		DLL_RESET        : in std_logic
	);
	end component;

	function choose_powerup_length(IS_SIMULATION : in integer) return integer is
		constant powerup_length_sim : integer := 2;
		constant powerup_length_nosim : integer:= 255;
	begin
		if IS_SIMULATION = 0 then
			return powerup_length_nosim;
		else
			return powerup_length_sim;
		end if;
	end function choose_powerup_length;
	
begin

	dll_reset_gnd <= '0';
	CALIBCLK <= SLOWCLK;
	
	-- AK 12/16/2011: synchronize power-up reset and also ensure it's stable for many clocks
	-- powerup_n <= POWER_UP_RST_N;
	powerup_reg : process (CALIBCLK)
	begin
		if CALIBCLK'event and CALIBCLK = '1' then  -- rising clock edge
			powerup_n_d1 <= POWER_UP_RST_N;
			powerup_n_d2 <= powerup_n_d1;
		end if;
	end process powerup_reg;
	
	powerup_stable : process (CALIBCLK,powerup_n_d2)
		constant powerup_length : integer := choose_powerup_length(SIMULATION);
		variable cnt : integer range 0 to powerup_length := 0;
	begin
		if powerup_n_d2 = '0' then
			cnt := 0;
			powerup_n <= '0';
		elsif CALIBCLK'event and CALIBCLK = '1' then  -- rising clock edge
			if cnt = powerup_length then
				powerup_n <= '1';
			else
				cnt := cnt + 1;
				powerup_n <= '0';
			end if;
		end if;
	end process powerup_stable;

	-- LOS_stable signal
	los_stable_pr : process (CALIBCLK,powerup_n)
	begin
	if powerup_n = '0' then
		LOS_d <= "00000";
		LOS_stable <= '0';
	elsif CALIBCLK'event and CALIBCLK = '1' then  -- rising clock edge
		LOS_d(0) <= '0';     -- TODO: add LOS port to ldc_top
		LOS_d(1) <= LOS_d(0);
		LOS_d(2) <= LOS_d(1);
		LOS_d(3) <= LOS_d(2);
		LOS_d(4) <= LOS_d(3);
		LOS_stable <= LOS_d(4) and LOS_d(3) and LOS_d(2) and LOS_d(1);
	end if;
	end process los_stable_pr;
	
	altgx_wrap : altgx_tlk
	generic map(
		SIMULATION      => SIMULATION,
		SIMPLE_TLK_WRAP => 0,
		DATAGEN_MODE    => 0,
		BONDING_MODE    => 0
	)
	port map(
		POWER_UP_RST_N => powerup_n,
		-- transmitter (to both DAQ and FTK)
		TXD            => txd_sig,
		TX_EN          => tx_en_sig,
		TX_ER          => tx_er_sig,
		TX_CLK         => tx_clk_sig,
		ICLK           => open,
		-- receiver (from DAQ):
		RXD0           => rxd0_sig,
		RX_CLK0        => rx_clk0_sig,
		RX_LOCKED0     => open,
		RX_ER0         => rx_er0_sig,
		RX_DV0         => rx_dv0_sig,
		-- receiver (from FTK):
		RXD1           => open,
		RX_CLK1        => open,
		RX_LOCKED1     => open,
		RX_ER1         => open,
		RX_DV1         => open,
		-- Cyclone transceiver ports (connect to pins)
		SLOWCLK        => SLOWCLK,
		CALIBCLK       => SLOWCLK,
		REFCLK0        => REFCLK,
		REFCLK1        => REFCLK,
		-- serial links
		GXB_TX0        => GXB_TX0,
		GXB_RX0        => GXB_RX0,
		GXB_TX1        => open,
		GXB_RX1        => '0',
		debug_bits     => open,
		-- LOS signals
		LOS0_stable	   => LOS_stable,
		LOS1_stable    => '1',
		-- send special acknowledgement to FTK_XOFF_ENA
		FTK_SEND_ACK   => '0'
	);
	   
	ldc_inst: holaldc_core
	generic map(
		ALTERA_XILINX => ALTERA_XILINX,
		SIMULATION => SIMULATION,
		LCLK_SELECT => LCLK_SELECT,
		XCLK_FREQ => XCLK_FREQ,
		ACTIVITY_LENGTH => ACTIVITY_LENGTH,
		FIFODEPTH => FIFODEPTH,
		LOG2DEPTH => LOG2DEPTH,
		FULLMARGIN => FULLMARGIN
    )
	port map (
		POWER_UP_RST_N => POWER_UP_RST_N,
		LD            => LD,
		UXOFF_N       => UXOFF_N,
		URESET_N      => URESET_N,
		UTDO_N        => UTDO_N,
		LCTRL_N       => LCTRL_N,
		LWEN_N        => LWEN_N,
		LCLK          => LCLK,
		LDERR_N       => LDERR_N,
		URL           => URL,
		LDOWN_N       => LDOWN_N,
		TESTLED_N     => TESTLED_N,
		LDERRLED_N    => LDERRLED_N,
		LUPLED_N      => LUPLED_N,
		FLOWCTLLED_N  => FLOWCTLLED_N,
		ACTIVITYLED_N => ACTIVITYLED_N,
		XCLK          => XCLK, -- main 100 MHz internal clock, also feed 100->40 MHz PLL
		TX_CLK        => tx_clk_sig,  -- to feed tx phase comp fifo
		ENABLE        => open,
		TXD           => txd_sig,
		TX_EN         => tx_en_sig,
		TX_ER         => tx_er_sig,
		RXD          => rxd0_sig,
		RX_CLK       => rx_clk0_sig,
		RX_ER        => rx_er0_sig,
		RX_DV        => rx_dv0_sig,
		LCLK_IN       => LCLK_IN,
		DLL_RESET     => dll_reset_gnd
	);

end structure;
