-- SPECIAL MODULE THAT SIMULATES AN LSC BOARD STUCK IN A COUNTER STATE
-- THIS MODULE ALSO ALLOWS TO PERFORM VALIDATION OF RECEIVED COUNTER WORDS
-- NOTE: it injects sequences of IDLE to assist with syncrhonization

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;

entity lsc_count_simulator is
generic (
	NDATA : integer := 65535
);
port (
	POWER_UP_RST_N  : in  std_logic;
	-- outputs to send
	TX_CLK          : in  std_logic;  -- TX_CLK from altgx
	TXD             : out std_logic_vector(15 downto 0);
	TX_EN           : out std_logic;
	TX_ER           : out std_logic;
	-- inputs to receive
	RX_CLK          : in  std_logic;  -- RX_CLK from altgx
	RXD             : in std_logic_vector(15 downto 0);
	RX_DV           : in std_logic;
	RX_ER           : in std_logic;
	valid           : out std_logic
);
end lsc_count_simulator;

architecture structure of lsc_count_simulator is
	type state_type is (IDLE,DATA);
	signal pres_state, next_state : state_type;
	attribute syn_encoding : string;
	attribute syn_encoding of state_type : type is "safe";
	signal do_idle_cnt : std_logic;
	signal idle_cnt_done : std_logic;
	signal do_data_cnt : std_logic;
	signal data_cnt_done : std_logic;
	signal data_cnt_val : std_logic_vector(15 downto 0);
	signal RXD_LAST : std_logic_vector(15 downto 0);
	-- register outputs
	signal txd_sig : std_logic_vector(15 downto 0);
	signal tx_en_sig : std_logic;
	signal tx_er_sig : std_logic;
	signal valid_sig: std_logic;
begin

-- FSM sequential logic
count_fsm : process (TX_CLK, POWER_UP_RST_N)
begin
	if POWER_UP_RST_N = '0' then  -- asynchronous reset (active low)
		pres_state <= IDLE;
	elsif TX_CLK'event and TX_CLK = '1' then  -- rising clock edge
		pres_state <= next_state;
	end if;
end process count_fsm;

-- FSM combinational logic
count_comb : process (pres_state, idle_cnt_done, data_cnt_done)
begin
	next_state <= IDLE;
	case pres_state is
		when IDLE =>
			if(idle_cnt_done='1') then
				next_state <= DATA;
			else
				next_state <= IDLE;
			end if;
		when DATA =>
			if(data_cnt_done='1') then
				next_state <= IDLE;
			else
				next_state <= DATA;
			end if;
	end case;
end process count_comb;

-- FSM output multiplexer [with an extra register]
out_mux : process (pres_state, TX_CLK, POWER_UP_RST_N)
begin
	if POWER_UP_RST_N = '0' then  -- asynchronous reset (active low)
		txd_sig   <= (others => '0');
		tx_en_sig <= '0';
		tx_er_sig <= '0';
		do_idle_cnt <= '0';
		do_data_cnt <= '0';
	elsif TX_CLK'event and TX_CLK = '1' then  -- rising clock edge
		txd_sig   <= (others => '0');
		tx_en_sig <= '0';
		tx_er_sig <= '0';
		do_idle_cnt <= '0';
		do_data_cnt <= '0';
		case pres_state is	
			when IDLE =>
				txd_sig   <= (others => '0');
				tx_en_sig <= '0';
				tx_er_sig <= '0';
				do_idle_cnt <= '1';
				do_data_cnt <= '0';
			when DATA =>
				txd_sig   <= data_cnt_val;
				tx_en_sig <= '1';
				tx_er_sig <= '0';
				do_idle_cnt <= '0';
				do_data_cnt <= '1';
		end case;
	end if;
end process out_mux;

TXD <= txd_sig;
TX_EN <= tx_en_sig;
TX_ER <= tx_er_sig;
valid <= valid_sig;

-- counts 16 IDLE's
idle_cnt : process (TX_CLK, POWER_UP_RST_N)
	variable cnt: integer range 0 to 15 :=0;
begin
	if POWER_UP_RST_N = '0' then  -- asynchronous reset (active low)
		cnt := 0;
		idle_cnt_done <= '0';
	elsif TX_CLK'event and TX_CLK = '1' then  -- rising clock edge
		idle_cnt_done <= '0';
		if do_idle_cnt = '1' then
			if cnt=15 then
				cnt := 0;
				idle_cnt_done <= '1';
			else
				cnt := cnt + 1;
			end if;
		end if;
	end if;
end process idle_cnt;

-- counts 2^16 datas
data_cnt : process (TX_CLK, POWER_UP_RST_N)
	variable cnt: integer range 0 to NDATA :=0;
begin
	if POWER_UP_RST_N = '0' then  -- asynchronous reset (active low)
		cnt := 0;
		data_cnt_done <= '0';
		data_cnt_val <= (others => '0');
	elsif TX_CLK'event and TX_CLK = '1' then  -- rising clock edge
		data_cnt_done <= '0';
		data_cnt_val <= (others => '0');
		if do_data_cnt = '1' then
			if cnt=NDATA then
				cnt := 0;
				data_cnt_done <= '1';
			else
				cnt := cnt + 1;
			end if;
			data_cnt_val <= std_logic_vector(to_unsigned(cnt,data_cnt_val'length));
		end if;
	end if;
end process data_cnt;

-- verify received counter data
verify_proc : process(RX_CLK,POWER_UP_RST_N)
begin
	if POWER_UP_RST_N = '0' then  -- asynchronous reset (active low)
		valid_sig <= '0';
		RXD_LAST <= (others => '0');
	elsif RX_CLK'event and RX_CLK = '1' then  -- rising clock edge
		valid_sig <= '1';
		RXD_LAST <= (others => '0');
		if RX_DV='1' and RX_ER='0' then
			RXD_LAST <= RXD;
			if RXD_LAST = std_logic_vector(unsigned(RXD)-1) or unsigned(RXD_LAST) = NDATA or unsigned(RXD_LAST) = 0 then
				valid_sig <= '1';
			else
				valid_sig <= '0';
			end if;
		end if;
	end if;
end process verify_proc;

end structure;
