-- SPECIAL MODULE THAT SIMULATES AN LSC BOARD STUCK IN SEND_RESET STATE
-- (THAT'S THE INITIAL STATE OF AN LSC AFTER POWER UP)
-- 16 IDLE's, followed by CARRIER, followed by RRES ("0000000000000001")

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity lsc_rres_simulator is
port (
	POWER_UP_RST_N  : in  std_logic;
	TX_CLK          : in  std_logic;  -- TX_CLK from altgx
	TXD             : out std_logic_vector(15 downto 0);
	TX_EN           : out std_logic;
	TX_ER           : out std_logic
	-- TODO: add RX ports and perform validation; output validation status
);
end lsc_rres_simulator;

architecture structure of lsc_rres_simulator is
	CONSTANT RRES : STD_LOGIC_VECTOR := "0000000000000001";
	type state_type is (IDLE,CMD,DATA);
	signal pres_state, next_state : state_type;
	attribute syn_encoding : string;
	attribute syn_encoding of state_type : type is "safe";
	signal do_cnt : std_logic;
	signal cnt_done : std_logic;
	-- register outputs
	signal txd_sig : std_logic_vector(15 downto 0);
	signal tx_en_sig : std_logic;
	signal tx_er_sig : std_logic;
begin

-- FSM sequential logic
rres_fsm : process (TX_CLK, POWER_UP_RST_N)
begin
	if POWER_UP_RST_N = '0' then  -- asynchronous reset (active low)
		pres_state <= IDLE;
	elsif TX_CLK'event and TX_CLK = '1' then  -- rising clock edge
		pres_state <= next_state;
	end if;
end process rres_fsm;

-- FSM combinational logic
rres_comb : process (pres_state, cnt_done)
begin
	next_state <= IDLE;
	case pres_state is
		when IDLE =>
			if(cnt_done='1') then
				next_state <= CMD;
			else
				next_state <= IDLE;
			end if;
		when CMD =>
			next_state <= DATA;
		when DATA =>
			next_state <= IDLE;
		when others =>
			next_state <= IDLE;
	end case;
end process rres_comb;

-- FSM output multiplexer [with an extra register]
out_mux : process (pres_state, TX_CLK, POWER_UP_RST_N)
begin
	if POWER_UP_RST_N = '0' then  -- asynchronous reset (active low)
		txd_sig   <= (others => '0');
		tx_en_sig <= '0';
		tx_er_sig <= '0';
		do_cnt <= '0';
	elsif TX_CLK'event and TX_CLK = '1' then  -- rising clock edge
		txd_sig   <= (others => '0');
		tx_en_sig <= '0';
		tx_er_sig <= '0';
		do_cnt <= '0';
		case pres_state is	
			when IDLE =>
				txd_sig   <= (others => '0');
				tx_en_sig <= '0';
				tx_er_sig <= '0';
				do_cnt <= '1';
			when CMD =>
				txd_sig   <= (others => '0');
				tx_en_sig <= '0';
				tx_er_sig <= '1';
				do_cnt <= '0';
			when DATA =>
				txd_sig   <= RRES;
				tx_en_sig <= '1';
				tx_er_sig <= '0';
				do_cnt <= '0';
			when others => 
				txd_sig   <= (others => '0');
				tx_en_sig <= '0';
				tx_er_sig <= '0';
				do_cnt <= '0';
		end case;
	end if;
end process out_mux;

TXD <= txd_sig;
TX_EN <= tx_en_sig;
TX_ER <= tx_er_sig;

-- counts 16 IDLE's
idle_cnt : process (TX_CLK, POWER_UP_RST_N, do_cnt)
	variable cnt: integer range 0 to 15 :=0;
begin
	if POWER_UP_RST_N = '0' then  -- asynchronous reset (active low)
		cnt := 0;
		cnt_done <= '0';
	elsif TX_CLK'event and TX_CLK = '1' then  -- rising clock edge
		cnt_done <= '0';
		if do_cnt = '1' then
			if cnt=15 then
				cnt := 0;
				cnt_done <= '1';
			else
				cnt := cnt + 1;
			end if;
		end if;
	end if;
end process idle_cnt;

end structure;
