library IEEE;
use     IEEE.Std_Logic_1164.all;

-- !!! Remember to change SIMULATION=0 before FPGA synthesis !!!

entity lsc_top is
generic(
	SIMULATION       : integer :=    0;   -- simulation mode
	ALTERA_XILINX    : integer :=    1;   -- 1=Altera 0=Xilinx
	XCLK_FREQ        : integer :=  100;   -- Reference clock oscillator frequency
										  -- 100 -> 2.0 Gb/s link speed (default)
										  -- 125 -> 2.5 Gb/s link speed
    USE_PLL          : integer :=    1;   -- ICLK_2 generation:
	  									  -- 0 = via vhdl simulation
										  -- 1 = via PLL inside core_holalsc that's fed by PLL_CLK (default)
										  -- 2 = directly mapped to PLL_CLK (generated inside altgx_tlk via BONDING_MODE=3)
	ACTIVITY_LENGTH  : integer :=   15;   -- ACTLED duration
	FIFODEPTH        : integer :=   64;   -- only powers of 2
	LOG2DEPTH        : integer :=    6;   -- 2log of depth
	FULLMARGIN       : integer :=   16    -- words left when LFF_N
);

port (
	POWER_UP_RST_N : in  std_logic;

	-- LSC core signals from FEMB
	UD            : in  std_logic_vector(31 downto 0);
	URESET_N      : in  std_logic;
	UTEST_N       : in  std_logic;
	UCTRL_N       : in  std_logic;
	UWEN_N        : in  std_logic;
	UCLK          : in  std_logic;  -- 40 MHz
	UDW           : in  std_logic_vector(1 downto 0); --unused

	-- From optical transceivers
	TXFAULT0      : in std_logic;
	TXFAULT1      : in std_logic;
	LOS0     	  : in std_logic;
	LOS1   	      : in std_logic;
	
	-- Four related 100 MHZ clocks
	REFCLK0       : in std_logic;   -- LVDS, for altgx (always) and ICLK_2 (if USE_PLL=2)
	REFCLK1       : in std_logic;   -- LVDS, for ICLK_2 (if USE_PLL=1)
	PLLCLK0       : in std_logic;   -- TTL, for altgx calibration
	PLLCLK1       : in std_logic;   -- TTL, for return line sampling
	-- Additional clocks
	SLOWCLK         : in std_logic;   -- 50 MHz (for altgx reconfiguration)
	
	-- Cyclone transceiver ports
	GXB_TX0       : out std_logic;
	GXB_RX0       : in std_logic;
	GXB_TX1       : out std_logic;
	GXB_RX1       : in std_logic;
	
	-- LSC core outputs
	LFF_N         : out std_logic;
	LRL           : out std_logic_vector(3 downto 0);
	LDOWN_N       : out std_logic;
	
	-- LED outputs
	TESTLED_N     : out std_logic;       -- Red
	LDERRLED_N    : out std_logic;       -- Red. Set to ground in LSC.
	LUPLED0_N     : out std_logic;       -- Green, wired to control block (=NOT LDOWN)
	LUPLED1_N     : out std_logic;       -- Green, tell whether FTK CDR locked to return channel data
	FLOWCTLLED0_N : out std_logic;       -- Red, wired to LFF flag
	FLOWCTLLED1_N : out std_logic;       -- Red, wired to XOFF from FTK return channel
	ACTIVITYLED_N : out std_logic        -- Green. Activity LED (data being sent)
   );

end lsc_top;

architecture structure of lsc_top is
	signal powerup_n  : STD_LOGIC;
	signal powerup_n_d1  : STD_LOGIC;
	signal powerup_n_d2  : STD_LOGIC;
	signal CALIBCLK  : STD_LOGIC;
	signal PLL_CLK   : STD_LOGIC;  -- for 50 MHz (ICLK_2) in LSC core
	-- Signals for LSC core
	signal dll_reset_gnd : STD_LOGIC;
	signal iclk_sig    : STD_LOGIC;
	-- transmitter signals
	signal tx_pll_areset_sig : STD_LOGIC;
	signal tx_clk_sig  : STD_LOGIC;
	signal tx_en_sig   : STD_LOGIC;
	signal tx_er_sig   : STD_LOGIC;
	signal txd_sig     : STD_LOGIC_VECTOR(15 DOWNTO 0);
	-- receiver signals (from DAQ)
	signal rx_clk0_sig : STD_LOGIC;
	signal rx_dv0_sig  : STD_LOGIC;
	signal rx_er0_sig  : STD_LOGIC;
	signal rx_locked0_sig  : STD_LOGIC;
	signal rxd0_sig    : STD_LOGIC_VECTOR(15 DOWNTO 0);
	-- receiver signals (from FTK)
	signal rx_clk1_sig : STD_LOGIC;
	signal rx_dv1_sig  : STD_LOGIC;
	signal rx_er1_sig  : STD_LOGIC;
	signal rx_locked1_sig  : STD_LOGIC;
	signal rxd1_sig    : STD_LOGIC_VECTOR(15 DOWNTO 0);
	-- HOLA signals
	signal xoff_ena_sig : STD_LOGIC;
	signal FTK_SEND_ACK : STD_LOGIC;
	
	-- LOS filtering
	signal LOS0_stable  : STD_LOGIC;
	signal LOS1_stable  : STD_LOGIC;
	signal LOS0_d : std_logic_vector(4 downto 0);
	signal LOS1_d : std_logic_vector(4 downto 0);
	
	component altgx_tlk is
		generic (
			SIMULATION       : integer;
			SIMPLE_TLK_WRAP  : integer;  -- enables "Simple" version of TLK2501 TX emulation (no disparity recalculation)
			DATAGEN_MODE     : integer;  -- 0=regular operation; 1=inject SEND_RESET sequences; 2=inject counter data
			BONDING_MODE     : integer   -- 0=single-channel; 1=dual channel via tx_coreclk; 2=dual-channel via bonding
		);
		port (
		POWER_UP_RST_N    : in  std_logic;
		-- transmitter (to both DAQ and FTK)
		TXD            : in std_logic_vector(15 downto 0);
		TX_EN          : in std_logic;
		TX_ER          : in std_logic;
		TX_CLK         : out std_logic;  -- generated internally, passed to LSC/LDC core
		ICLK           : out std_logic;  -- optionally generates and outputs 50 MHz ICLK
		-- receiver (from DAQ):
		RXD0           : out std_logic_vector(15 downto 0); -- Received parallel data
		RX_CLK0        : out std_logic;         -- Recovered clock.
		RX_LOCKED0     : out std_logic;
		RX_ER0         : out std_logic;
		RX_DV0         : out std_logic;
		-- receiver (from FTK):
		RXD1           : out std_logic_vector(15 downto 0); -- Received parallel data
		RX_CLK1        : out std_logic;         -- Recovered clock.
		RX_LOCKED1     : out std_logic;
		RX_ER1         : out std_logic;
		RX_DV1         : out std_logic;
		-- Cyclone transceiver ports (connect to pins)
		SLOWCLK       : in std_logic;   -- 50 MHz, altgx reconfiguation
		CALIBCLK      : in std_logic;   -- 100 MHz altgx calibration
		REFCLK0       : in std_logic;   -- 100 MHz, LVDS - altgx clock for DAQ channel MPLL
		REFCLK1       : in std_logic;   -- 100 MHz, LVDS - altgx clock for FTK channel MPLL
									    -- (should be the same as REFCLK0 to bond the channels)
		GXB_TX0       : out std_logic;
		GXB_RX0       : in std_logic;
		GXB_TX1       : out std_logic;
		GXB_RX1       : in std_logic;
		debug_bits    : out std_logic_vector( 3 downto 0);
		-- LOS signals
		LOS0_stable   : in STD_LOGIC;
	    LOS1_stable   : in STD_LOGIC;
		FTK_SEND_ACK  : in STD_LOGIC
	);
	end component;

	-- holalsc_core component
	component holalsc_core is
		GENERIC(
			SIMULATION       : integer :=    0;
			ALTERA_XILINX    : integer :=    1;
			XCLK_FREQ        : integer :=  100;
			USE_PLL          : integer :=    1;
			ACTIVITY_LENGTH  : integer :=   15;
			FIFODEPTH        : integer :=   64;
			LOG2DEPTH        : integer :=    6;
			FULLMARGIN       : integer :=   16
		);
		PORT (
			POWER_UP_RST_N : in  std_logic;
			UD            : in  std_logic_vector(31 downto 0);
			URESET_N      : in  std_logic;
			UTEST_N       : in  std_logic;
			UCTRL_N       : in  std_logic;
			UWEN_N        : in  std_logic;
			UCLK          : in  std_logic;
			LFF_N         : out std_logic;
			XOFF_ENA      : out std_logic;
			FTK_SEND_ACK  : out std_logic;
			LRL           : out std_logic_vector( 3 downto 0);
			LDOWN_N       : out std_logic;
			TESTLED_N     : out std_logic;
			LDERRLED_N    : out std_logic;
			LUPLED_N      : out std_logic;
			FLOWCTLLED0_N  : out std_logic;
			FLOWCTLLED1_N  : out std_logic;
			ACTIVITYLED_N : out std_logic;
			XCLK          : in std_logic;
			PLL_CLK       : in std_logic;
			TX_CLK        : in std_logic;
			ENABLE        : out std_logic;
			TXD           : out std_logic_vector(15 downto 0);
			TX_EN         : out std_logic;
			TX_ER         : out std_logic;
			RXD0           : in std_logic_vector(15 downto 0);
			RX_CLK0        : in std_logic;
			RX_ER0         : in std_logic;
			RX_DV0         : in std_logic;
			LOS0_stable    : in std_logic;
			RXD1           : in std_logic_vector(15 downto 0);
			RX_CLK1        : in std_logic;
			RX_ER1         : in std_logic;
			RX_DV1         : in std_logic;
			LOS1_stable    : in std_logic;
			DLL_RESET     : in std_logic
		);
	end component;

	function choose_powerup_length(IS_SIMULATION : in integer) return integer is
		constant powerup_length_sim : integer := 2;
		constant powerup_length_nosim : integer:= 255;
	begin
		if IS_SIMULATION = 0 then
			return powerup_length_nosim;
		else
			return powerup_length_sim;
		end if;
	end function choose_powerup_length;

begin
	dll_reset_gnd <= '0';
	-- powerup_n <= POWER_UP_RST_N;
	CALIBCLK <= PLLCLK0;
	-- if USE_PLL=2, the 50 MHz clock is generated inside altgx
	-- if USE_PLL=1, the 50 MHz clock is generated inside core_holalsc
	PLL_CLK <= iclk_sig when USE_PLL = 2 ELSE REFCLK1; 

	-- AK 09/27/2011: changed meaning of LUPLED1 to show the status of FTK_XOFF_ENA register
	--                this is used to assist in production testing of the HOLAs
	--LUPLED1_N <= not rx_locked1_sig;
	LUPLED1_N <= not xoff_ena_sig;
	
	-- AK 12/16/2011: synchronize power-up reset and also ensure it's stable for many clocks
	-- Use 100 MHz TTL clock CALIBCLK (which is related to transceiver master clock)
	powerup_reg : process (CALIBCLK)
	begin
		if CALIBCLK'event and CALIBCLK = '1' then  -- rising clock edge
			powerup_n_d1 <= POWER_UP_RST_N;
			powerup_n_d2 <= powerup_n_d1;
		end if;
	end process powerup_reg;
	
	powerup_stable : process (CALIBCLK,powerup_n_d2)
		constant powerup_length : integer := choose_powerup_length(SIMULATION);
		variable cnt : integer range 0 to powerup_length := 0;
	begin
		if powerup_n_d2 = '0' then
			cnt := 0;
			powerup_n <= '0';
		elsif CALIBCLK'event and CALIBCLK = '1' then  -- rising clock edge
			if cnt = powerup_length then
				powerup_n <= '1';
			else
				cnt := cnt + 1;
				powerup_n <= '0';
			end if;
		end if;
	end process powerup_stable;

	-- LOS_stable signal
	los_stable : process (CALIBCLK,powerup_n)
	begin
	if powerup_n = '0' then
		LOS0_d <= "00000";
		LOS0_stable <= '0';
		LOS1_d <= "00000";
		LOS1_stable <= '0';
	elsif CALIBCLK'event and CALIBCLK = '1' then  -- rising clock edge
		LOS0_d(0) <= LOS0;
		LOS0_d(1) <= LOS0_d(0);
		LOS0_d(2) <= LOS0_d(1);
		LOS0_d(3) <= LOS0_d(2);
		LOS0_d(4) <= LOS0_d(3);
		LOS0_stable <= LOS0_d(4) and LOS0_d(3) and LOS0_d(2) and LOS0_d(1);
		LOS1_d(0) <= LOS1;
		LOS1_d(1) <= LOS1_d(0);
		LOS1_d(2) <= LOS1_d(1);
		LOS1_d(3) <= LOS1_d(2);
		LOS1_d(4) <= LOS1_d(3);
		LOS1_stable <= LOS1_d(4) and LOS1_d(3) and LOS1_d(2) and LOS1_d(1);
	end if;
	end process los_stable;
	
	altgx_wrap : altgx_tlk
	generic map(
		SIMULATION      => SIMULATION,
		SIMPLE_TLK_WRAP => 1,
		DATAGEN_MODE    => 0,
		BONDING_MODE    => 2
	)
	port map(
		POWER_UP_RST_N => powerup_n,
		-- transmitter (to both DAQ and FTK)
		TXD            => txd_sig,
		TX_EN          => tx_en_sig,
		TX_ER          => tx_er_sig,
		TX_CLK         => tx_clk_sig,
		ICLK           => iclk_sig,
		-- receiver (from DAQ):
		RXD0           => rxd0_sig,
		RX_CLK0        => rx_clk0_sig,
		RX_LOCKED0     => rx_locked0_sig,
		RX_ER0         => rx_er0_sig,
		RX_DV0         => rx_dv0_sig,
		-- receiver (from FTK):
		RXD1           => rxd1_sig,
		RX_CLK1        => rx_clk1_sig,
		RX_LOCKED1     => rx_locked1_sig,
		RX_ER1         => rx_er1_sig,
		RX_DV1         => rx_dv1_sig,
		-- Cyclone transceiver ports (connect to pins)
		SLOWCLK        => SLOWCLK,
		CALIBCLK       => PLLCLK0,
		REFCLK0        => REFCLK0,
		REFCLK1        => REFCLK0,
		-- serial links
		GXB_TX0        => GXB_TX0,
		GXB_RX0        => GXB_RX0,
		GXB_TX1        => GXB_TX1,
		GXB_RX1        => GXB_RX1,
		debug_bits     => open,
		-- LOS signals
		LOS0_stable	   => LOS0_stable,
		LOS1_stable    => LOS1_stable,
		-- send special acknowledgement to FTK_XOFF_ENA
		FTK_SEND_ACK   => FTK_SEND_ACK
	);

	lsc_inst: holalsc_core
	generic map(
		SIMULATION => SIMULATION,
		ALTERA_XILINX => ALTERA_XILINX,
		XCLK_FREQ => XCLK_FREQ,
		USE_PLL => USE_PLL,
		ACTIVITY_LENGTH => ACTIVITY_LENGTH,
		FIFODEPTH => FIFODEPTH,
		LOG2DEPTH => LOG2DEPTH,
		FULLMARGIN => FULLMARGIN
    )
	port map (
		POWER_UP_RST_N => powerup_n,
		UD            => UD,
		URESET_N      => URESET_N,
		UTEST_N       => UTEST_N,
		UCTRL_N       => UCTRL_N,
		UWEN_N        => UWEN_N,
		UCLK          => UCLK,
		LFF_N         => LFF_N,
		XOFF_ENA      => xoff_ena_sig,
		FTK_SEND_ACK  => FTK_SEND_ACK,
		LRL           => LRL,
		LDOWN_N       => LDOWN_N,
		TESTLED_N     => TESTLED_N,
		LDERRLED_N    => LDERRLED_N,
		LUPLED_N      => LUPLED0_N,
		FLOWCTLLED0_N  => FLOWCTLLED0_N,
		FLOWCTLLED1_N  => FLOWCTLLED1_N,
		ACTIVITYLED_N => ACTIVITYLED_N,
		XCLK          => PLLCLK1,    -- to sample return line status
		PLL_CLK       => PLL_CLK,  
		TX_CLK        => tx_clk_sig, -- to feed tx phase comp fifo
		ENABLE        => open,
		TXD           => txd_sig,
		TX_EN         => tx_en_sig,
		TX_ER         => tx_er_sig,
		RXD0          => rxd0_sig,
		RX_CLK0       => rx_clk0_sig,
		RX_ER0        => rx_er0_sig,
		RX_DV0        => rx_dv0_sig,
		LOS0_stable	  => LOS0_stable,
		RXD1          => rxd1_sig,
		RX_CLK1       => rx_clk1_sig,
		RX_ER1        => rx_er1_sig,
		RX_DV1        => rx_dv1_sig,
		LOS1_stable	  => LOS1_stable,
		DLL_RESET     => dll_reset_gnd
	);

end structure;
