-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--                                                                           --
--  File name : tb_dlink.vhd                                                  --
--  Title     : Testbench for HOLA S-LINK Simulation                         --
--  Author(s) : Aurelio Ruiz Garcia                                          --
--              Jesper Moller Haee                                           --
--              Stefan Haas                                                  --
--  Notes     : Box containig the whole structure of an HOLA S-Link          --
--              implementation (dual-channel).                               --
--                                                                           --
--                                                                           --
--      +------+   +------+   +------+   +------+   +------+   +------+      --
--      |      |-->|      |-->|      |-->|      |-->|      |-->|      |      --
--      | FEMB |   | HOLA |   | TLK  |   | TLK  |   | HOLA |   | ROMB |      --
--      |      |   | LSC  |   | WRAP |   | WRAP |   | LDC  |   |      |      --
--      |      |<--|      |<--|      |<--|      |<--|      |<--|      |      --
--      +------+   +------+   +------+   +------+   +------+   +------+      --
--                                       +------+   +------+   +------+      --
--                                    -->|      |-->|      |-->|      |      --
--                                       | TLK  |   | HOLA |   | ROMB |      --
--                                       | WRAP |   | LDC  |   |      |      --
--                                    <--|      |<--|      |<--|      |      --
--                                       +------+   +------+   +------+      --
--                                                                           --
--             Inputs and outputs of the link are written into the following --
--             text files:                                                   --
--                                                                           --
--               - fc_in.txt  -> Forward channel input data  : UCTRL_N, UD   --
--               - fc_out.txt -> Forward channel output data : LCTRL_N, LD   --
--               - rc_in.txt  -> Return channel input data   : URL           --
--               - rc_out.txt -> Return channel output data  : LRL           --
--                                                                           --
-------------------------------------------------------------------------------
--     Version | Author |  Mod.Date   |          Changes made                --
-------------------------------------------------------------------------------
--     1.0     |  ARG   | 11-Mar-2002 | Original coding                      --
--     1.1     |  ARG   | 17-Apr-2002 | TX_ERROR added                       --
--     1.1     |  ARG   | 21-May-2002 | FIBER_DISC added                     --
--     1.2     |  JMH   |  9-dec-2002 | DLL_RESET added to both LDC and LSC  --
--             |        |             | the signal only affects xilinx ver.  --
--     1.3     |  JMH   | 24-jan-2003 | XCLK seperated into LSC_xclk and     --
--             |        |             | LDC_xclk these are now slightly      --
--             |        |             | misaligned to be more realistic      --
--             |        |             | also the RX_CLK signals is seperated --
--     1.4     |  JMH   | 12-feb-2003 | generics added for XCLKs             --
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_textio.all ;
use std.textio.all;


entity tb_dlink is
  generic (
    POWER_UP_RST_DELAY_TIME     : time      := 1 ns;
    POWER_UP_RST_LSC_TIME       : time      := 1050 ns;
	POWER_UP_RST_LDC0_TIME      : time      := 1050 ns;
	POWER_UP_RST_LDC1_TIME      : time      := 1050 ns;
	POWER_UP_RST_LEN_TIME       : time      := 100000 ns;
    LSC_XCLK_HIGH_TIME          : time      := 5 ns;
    LSC_XCLK_LOW_TIME           : time      := 5 ns;
	LSC_XCLK_START_OFFSET       : time      := 7.6 ns;
    LDC0_XCLK_HIGH_TIME          : time      := 5 ns;
    LDC0_XCLK_LOW_TIME           : time      := 5 ns;
    LDC0_XCLK_START_OFFSET       : time      := 2.7 ns;
	LDC1_XCLK_HIGH_TIME          : time      := 5 ns;
    LDC1_XCLK_LOW_TIME           : time      := 5 ns;
    LDC1_XCLK_START_OFFSET       : time      := 11.2 ns;
	LSC_SLOWCLK_LOW_TIME        : time      := 10 ns;
	LSC_SLOWCLK_HIGH_TIME       : time      := 10 ns;
	LDC0_SLOWCLK_LOW_TIME        : time      := 10 ns;
	LDC0_SLOWCLK_HIGH_TIME       : time      := 10 ns;
	LDC0_SLOWCLK_START_OFFSET    : time      := 11.1 ns;
	LDC1_SLOWCLK_LOW_TIME        : time      := 10 ns;
	LDC1_SLOWCLK_HIGH_TIME       : time      := 10 ns;
	LDC1_SLOWCLK_START_OFFSET    : time      := 13.4 ns;
	OUTFILE1NAME                : string    := "out_dlink_fc_in.txt";
	OUTFILE2DAQNAME             : string    := "out_dlink_fc_out_daq.txt";
	OUTFILE2FTKNAME             : string    := "out_dlink_fc_out_ftk.txt";
	OUTFILE3NAME                : string    := "out_dlink_rc_in.txt";
	OUTFILE4NAME                : string    := "out_dlink_rc_out.txt"
  );
end tb_dlink;

library hola_sim_lib;
use hola_sim_lib.hola_sim_components.all;

library work;
library work;

architecture structure of tb_dlink is

	signal POWER_UP_RST_LSC_N : std_logic := '0';
	signal POWER_UP_RST_LDC0_N : std_logic := '0';
	signal POWER_UP_RST_LDC1_N : std_logic := '0';
	-------------------------------------------------------------------------------
	-- signals FEMB <-> LSC
	-------------------------------------------------------------------------------

	signal UD       : std_logic_vector(1 to 32);
	signal LFF_n    : std_logic;
	signal URESET_n : std_logic := '0';
	signal UTEST_n  : std_logic;
	signal UCTRL_n  : std_logic;
	signal UWEN_n   : std_logic;
	signal UCLK     : std_logic := '0';
	signal LRL      : std_logic_vector(1 to 4);
	signal LDOWN_n  : std_logic;

	-------------------------------------------------------------------------------
	-- signals LDC <-> ROMB (DAQ)
	-------------------------------------------------------------------------------
	signal LD0        :   std_logic_vector(1 to 32);
	signal UXOFF0_n   :   std_logic;
	signal RURESET0_n :   std_logic := '0';
	signal UTDO0_n    :   std_logic;
	signal LCTRL0_n   :   std_logic;
	signal LWEN0_n    :   std_logic;
	signal LCLK0      :   std_logic;
	signal LDERR0_n   :   std_logic;
	signal URL0       :   std_logic_vector(1 to 4);
	signal RLDOWN0_n  :   std_logic;
	-------------------------------------------------------------------------------
	-- signals LDC <-> ROMB (FTK)
	-------------------------------------------------------------------------------
	signal LD1        :   std_logic_vector(1 to 32);
	signal UXOFF1_n   :   std_logic;
	signal RURESET1_n :   std_logic := '0';
	signal UTDO1_n    :   std_logic;
	signal LCTRL1_n   :   std_logic;
	signal LWEN1_n    :   std_logic;
	signal LCLK1      :   std_logic;
	signal LDERR1_n   :   std_logic;
	signal URL1       :   std_logic_vector(1 to 4);
	signal RLDOWN1_n  :   std_logic;
	
	-----------------------------------------------------------------------------
	-- LSC <-> LDC link
	-----------------------------------------------------------------------------
	signal LSC_XCLK : std_logic;
	signal LDC0_XCLK : std_logic;
	signal LDC1_XCLK : std_logic;
	signal LSC_SLOWCLK : std_logic;
	signal LDC0_SLOWCLK : std_logic;
	signal LDC1_SLOWCLK : std_logic;
	signal GXB_TX0 : std_logic;
	signal GXB_RX0 : std_logic;
	signal GXB_TX1 : std_logic;
	signal GXB_RX1 : std_logic;
	signal LSC_TXFAULT0 : std_logic := '0';
	signal LSC_TXFAULT1 : std_logic := '0';
	signal LSC_LOS0 : std_logic := '0';
	signal LSC_LOS1 : std_logic := '0';
	signal LDC0_TXFAULT0 : std_logic := '0';
	signal LDC0_LOS0 : std_logic := '0';
	signal LDC1_TXFAULT0 : std_logic := '0';
	signal LDC1_LOS0 : std_logic := '0';
	
	-----------------------------------------------------------------------------
	-- LSC and LDC LED signals
	-----------------------------------------------------------------------------
	signal LSC_TESTLEDN     : std_logic;
	signal LSC_LUPLED0N     : std_logic;
	signal LSC_LUPLED1N     : std_logic;
	signal LSC_LDERRLEDN    : std_logic;
	signal LSC_ACTIVITYLEDN : std_logic;
	signal LSC_FLOWCTLLED0N : std_logic;
	signal LSC_FLOWCTLLED1N : std_logic;
	
	signal LDC0_TESTLEDN     : std_logic;
	signal LDC0_LUPLEDN      : std_logic;
	signal LDC0_LDERRLEDN    : std_logic;
	signal LDC0_ACTIVITYLEDN : std_logic;
	signal LDC0_FLOWCTLLEDN  : std_logic;

	signal LDC1_TESTLEDN     : std_logic;
	signal LDC1_LUPLEDN      : std_logic;
	signal LDC1_LDERRLEDN    : std_logic;
	signal LDC1_ACTIVITYLEDN : std_logic;
	signal LDC1_FLOWCTLLEDN  : std_logic;
	
	-----------------------------------------------------------------------------
	-- Other signals
	-----------------------------------------------------------------------------
	signal LCLK_PLL0  : std_logic;   -- external clock connection on LDC
	signal LCLK_PLL1  : std_logic;   -- external clock connection on LDC
	signal DLL_RESET  : std_logic;   -- DLL reset for Xilinx version
	signal GND : std_logic := '0';
	signal LSC_REFCLK0 : std_logic;
	signal LSC_REFCLK1 : std_logic;
	signal LSC_PLLCLK0 : std_logic;
	signal LSC_PLLCLK1 : std_logic;
	
	component lsc_top
	port (
		POWER_UP_RST_N : in  std_logic;
		-- LSC core signals from FEMB
		UD            : in  std_logic_vector(31 downto 0);
		URESET_N      : in  std_logic;
		UTEST_N       : in  std_logic;
		UCTRL_N       : in  std_logic;
		UWEN_N        : in  std_logic;
		UCLK          : in  std_logic;  -- 40 MHz
		UDW           : in  std_logic_vector(1 downto 0); --unused
		-- From optical transceivers
		TXFAULT0      : in std_logic;
		TXFAULT1      : in std_logic;
		LOS0     	  : in std_logic;
		LOS1   	      : in std_logic;
		-- Four related 100 MHZ clocks
		REFCLK0       : in std_logic;   -- LVDS, for MPLL_2 (transceiver operation)
		REFCLK1       : in std_logic;   -- LVDS, for MPLL_1 (generates 50 MHz ICLK_2)
		PLLCLK0       : in std_logic;   -- TTL, for transceiver calibration
		PLLCLK1       : in std_logic;   -- TTL, for return line sampling
		-- Additional clocks
		SLOWCLK         : in std_logic;   -- 50 MHz (for reconfiguration & calibration)
		-- Cyclone transceiver ports
		GXB_TX0       : out std_logic;
		GXB_RX0       : in std_logic;
		GXB_TX1       : out std_logic;
		GXB_RX1       : in std_logic;
		-- LSC core outputs
		LFF_N         : out std_logic;
		LRL           : out std_logic_vector(3 downto 0);
		LDOWN_N       : out std_logic;
		-- LED outputs
		TESTLED_N     : out std_logic;       -- Red
		LDERRLED_N    : out std_logic;       -- Red. Set to ground in LSC.
		LUPLED0_N     : out std_logic;       -- Green, wired to control block (=NOT LDOWN)
		LUPLED1_N     : out std_logic;       -- Green, tell whether FTK CDR locked to return channel data
		FLOWCTLLED0_N : out std_logic;       -- Red, wired to LFF flag
		FLOWCTLLED1_N : out std_logic;       -- Red, wired to XOFF from FTK return channel
		ACTIVITYLED_N : out std_logic        -- Green. Activity LED (data being sent)
	);
	end component;
	
	component ldc_top
	port (
		POWER_UP_RST_N : in  std_logic;
		-- ldc core signals from FEMB
		LD            : out std_logic_vector(31 downto 0);
		UXOFF_N       : in  std_logic;
		URESET_N      : in  std_logic;
		UTDO_N        : in  std_logic;
		LCTRL_N       : out std_logic;
		LWEN_N        : out std_logic;
		LCLK          : out std_logic;  -- 40 MHz
		LDERR_N       : out std_logic;
		URL           : in  std_logic_vector( 3 downto 0);
		LDOWN_N       : out std_logic;
		-- LED outputs
		TESTLED_N     : out std_logic;       -- Red
		LDERRLED_N    : out std_logic;       -- Red.
		LUPLED_N      : out std_logic;        -- Green, wired to control block (=NOT LDOWN)
		FLOWCTLLED_N  : out std_logic;        -- Red
		ACTIVITYLED_N : out std_logic;        -- Green
		-- Free-running clocks from oscillators
		XCLK          : in std_logic;   -- 100 MHz (to sample receiver status)
		SLOWCLK       : in std_logic;   -- 50 MHz (for reconfiguration & calibration)
		LCLK_IN       : in   std_logic;
		-- Cyclone transceiver ports
		REFCLK        : in std_logic;   -- 100 MHz, LVDS
		GXB_TX0       : out std_logic;
		GXB_RX0       : in std_logic
		);
	end component;

begin  -- structure

	-------------------------------------------------------------------------------
	-->       FEMB instantiation                                                <--
	-------------------------------------------------------------------------------
	FEMB1: FEMB
	port map (
		UD       => UD,
		LFF_n    => LFF_n,
		URESET_n => URESET_n,
		UTEST_n  => UTEST_n,
		UCTRL_n  => UCTRL_n,
		UWEN_n   => UWEN_n,
		UCLK     => UCLK,
		LRL      => LRL,
		LDOWN_n  => LDOWN_n
	);

	-----------------------------------------------------------------------------
	-- HOLA LSC instantiation
	-----------------------------------------------------------------------------
	-- LVDS clocks:
	LSC_REFCLK0 <= LSC_XCLK;
	LSC_REFCLK1 <= LSC_XCLK'delayed(50 ps);
	-- TTL clocks (from LVDS->TTL converter):
	LSC_PLLCLK0 <= LSC_XCLK'delayed(2.6 ns);
	LSC_PLLCLK1 <= LSC_XCLK'delayed(2.2 ns);
	-- HOLA LSC
	HOLALSC1: lsc_top
	port map (
		POWER_UP_RST_N => POWER_UP_RST_LSC_N,
		-- LSC core signals from FEMB
		UD            => UD,
		URESET_N      => URESET_n,
		UTEST_N       => UTEST_n,
		UCTRL_N       => UCTRL_n,
		UWEN_N        => UWEN_n,
		UCLK          => UCLK,
		UDW           => "00",
		-- From optical transceivers
		TXFAULT0      => LSC_TXFAULT0,
		TXFAULT1      => LSC_TXFAULT1,
		LOS0          => LSC_LOS0,
		LOS1          => LSC_LOS1,
		-- Four related 100 MHZ clocks
		REFCLK0       => LSC_REFCLK0,
		REFCLK1       => LSC_REFCLK1,
		PLLCLK0       => LSC_PLLCLK0,
		PLLCLK1       => LSC_PLLCLK1,
		-- Additional clocks
		SLOWCLK       => LSC_SLOWCLK,
		-- Cyclone transceiver ports
		GXB_TX0       => GXB_TX0,
		GXB_RX0       => GXB_RX0,
		GXB_TX1       => GXB_TX1,
		GXB_RX1       => GXB_RX1,
		-- LSC core outputs
		LFF_N         => LFF_n,
		LRL           => LRL,
		LDOWN_N       => LDOWN_n,
		-- LED outputs
		TESTLED_N     => LSC_TESTLEDN,
		LDERRLED_N    => LSC_LDERRLEDN,
		LUPLED0_N     => LSC_LUPLED0N,
		LUPLED1_N     => LSC_LUPLED1N,
		FLOWCTLLED0_N => LSC_FLOWCTLLED0N,
		FLOWCTLLED1_N => LSC_FLOWCTLLED1N,
		ACTIVITYLED_N => LSC_ACTIVITYLEDN
	);

	-----------------------------------------------------------------------------
	-- HOLA LDC (DAQ) instantiation
	-----------------------------------------------------------------------------
	HOLALDC0: ldc_top
	port map (
		POWER_UP_RST_N => POWER_UP_RST_LDC0_N,
		-- ldc core signals from FEMB
		LD             => LD0,
		UXOFF_N        => UXOFF0_N,
		URESET_N       => RURESET0_N,
		UTDO_N         => UTDO0_N,
		LCTRL_N        => LCTRL0_N,
		LWEN_N         => LWEN0_N,
		LCLK           => LCLK0,
		LDERR_N        => LDERR0_N,
		URL            => URL0,
		LDOWN_N        => RLDOWN0_N,
		-- LED outputs
		TESTLED_N      => LDC0_TESTLEDN,
		LDERRLED_N     => LDC0_LDERRLEDN,
		LUPLED_N       => LDC0_LUPLEDN,
		FLOWCTLLED_N   => LDC0_FLOWCTLLEDN,
		ACTIVITYLED_N  => LDC0_ACTIVITYLEDN,
		-- Free-running clocks from oscillators
		XCLK           => LDC0_XCLK,
		SLOWCLK        => LDC0_SLOWCLK,
		LCLK_IN        => LCLK_PLL0,
		-- Cyclone transceiver ports
		REFCLK         => LDC0_XCLK,
		GXB_TX0        => GXB_RX0,
		GXB_RX0        => GXB_TX0
	);
	
	-------------------------------------------------------------------------------
	-->       ROMB (DAQ) instantiation                                               <--
	-------------------------------------------------------------------------------
	ROMB0: ROMB
	port map (
		LD       => LD0,
		UXOFF_n  => UXOFF0_n,
		URESET_n => RURESET0_n,
		UTDO_n   => UTDO0_n,
		LCTRL_n  => LCTRL0_n,
		LWEN_n   => LWEN0_n,
		LCLK     => LCLK0,
		LDERR_n  => LDERR0_n,
		URL      => URL0,
		LDOWN_n  => RLDOWN0_n
	);

		-----------------------------------------------------------------------------
	-- HOLA LDC (FTK) instantiation
	-----------------------------------------------------------------------------
	HOLALDC1: ldc_top
	port map (
		POWER_UP_RST_N => POWER_UP_RST_LDC1_N,
		-- ldc core signals from FEMB
		LD             => LD1,
		UXOFF_N        => UXOFF1_N,
		URESET_N       => RURESET1_N,
		UTDO_N         => UTDO1_N,
		LCTRL_N        => LCTRL1_N,
		LWEN_N         => LWEN1_N,
		LCLK           => LCLK1,
		LDERR_N        => LDERR1_N,
		URL            => URL1,
		LDOWN_N        => RLDOWN1_N,
		-- LED outputs
		TESTLED_N      => LDC1_TESTLEDN,
		LDERRLED_N     => LDC1_LDERRLEDN,
		LUPLED_N       => LDC1_LUPLEDN,
		FLOWCTLLED_N   => LDC1_FLOWCTLLEDN,
		ACTIVITYLED_N  => LDC1_ACTIVITYLEDN,
		-- Free-running clocks from oscillators
		XCLK           => LDC1_XCLK,
		SLOWCLK        => LDC1_SLOWCLK,
		LCLK_IN        => LCLK_PLL1,
		-- Cyclone transceiver ports
		REFCLK         => LDC1_XCLK,
		GXB_TX0        => GXB_RX1,
		GXB_RX0        => GXB_TX1
	);
	
	-------------------------------------------------------------------------------
	-->       ROMB (DAQ) instantiation                                               <--
	-------------------------------------------------------------------------------
	ROMB1: ROMB
	port map (
		LD       => LD1,
		UXOFF_n  => UXOFF1_n,
		URESET_n => RURESET1_n,
		UTDO_n   => UTDO1_n,
		LCTRL_n  => LCTRL1_n,
		LWEN_n   => LWEN1_n,
		LCLK     => LCLK1,
		LDERR_n  => LDERR1_n,
		URL      => URL1,
		LDOWN_n  => RLDOWN1_n
	);
	
	-----------------------------------------------------------------------------
	-- Other signals
	-----------------------------------------------------------------------------
	LCLK_PLL0  <= LCLK0;                    -- connect LCLK output to LCLK_PLL input
	LCLK_PLL1  <= LCLK1;                    -- connect LCLK output to LCLK_PLL input
	DLL_RESET <= '0';

	-----------------------------------------------------------------------------
	-- POWER-UP RESET
	-----------------------------------------------------------------------------
	power_up_prc : process
	begin
		loop
			POWER_UP_RST_LSC_N <= '1';
			wait for POWER_UP_RST_DELAY_TIME;
			POWER_UP_RST_LSC_N <= '0';
			wait for POWER_UP_RST_LSC_TIME;
			POWER_UP_RST_LSC_N <= '1';
			wait for POWER_UP_RST_LEN_TIME;
		end loop;
	end process;
 
 	power_up_ldc0_prc : process
	begin
		loop
			POWER_UP_RST_LDC0_N <= '1';
			wait for POWER_UP_RST_DELAY_TIME;
			POWER_UP_RST_LDC0_N <= '0';
			wait for POWER_UP_RST_LDC0_TIME;
			POWER_UP_RST_LDC0_N <= '1';
			wait for POWER_UP_RST_LEN_TIME;
		end loop;
	end process;

	power_up_ldc1_prc : process
	begin
		loop
			POWER_UP_RST_LDC1_N <= '1';
			wait for POWER_UP_RST_DELAY_TIME;
			POWER_UP_RST_LDC1_N <= '0';
			wait for POWER_UP_RST_LDC1_TIME;
			POWER_UP_RST_LDC1_N <= '1';
			wait for POWER_UP_RST_LEN_TIME;
		end loop;
	end process;
	-----------------------------------------------------------------------------
	-- External 100 MHz clock generation                                       --
	-----------------------------------------------------------------------------
	LSC_xclk_gen : process
	begin
		LSC_xclk   <= '0';
		wait for LSC_XCLK_START_OFFSET;
		loop
			LSC_xclk <= '0';
			wait for LSC_XCLK_LOW_TIME;       -- 100 MHz default
			LSC_xclk <= '1';
			wait for LSC_XCLK_HIGH_TIME;
		end loop;
	end process;

	LDC0_xclk_gen : process
	begin
		LDC0_xclk   <= '0';
		wait for LDC0_XCLK_START_OFFSET;     -- 1 ns default
		loop
			LDC0_xclk <= '0';
			wait for LDC0_XCLK_LOW_TIME;       -- 100 MHz default
			LDC0_xclk <= '1';
			wait for LDC0_XCLK_HIGH_TIME;
		end loop;
	end process;

	LDC1_xclk_gen : process
	begin
		LDC1_xclk   <= '0';
		wait for LDC1_XCLK_START_OFFSET;     -- 1 ns default
		loop
			LDC1_xclk <= '0';
			wait for LDC1_XCLK_LOW_TIME;       -- 100 MHz default
			LDC1_xclk <= '1';
			wait for LDC1_XCLK_HIGH_TIME;
		end loop;
	end process;
	
	-----------------------------------------------------------------------------
	-- External 50 MHz clock generation                                       --
	-----------------------------------------------------------------------------
	LSC_slowclk_gen : process
	begin
		loop
			LSC_slowclk <= '0';
			wait for LSC_SLOWCLK_LOW_TIME;       -- 50 MHz default
			LSC_slowclk <= '1';
			wait for LSC_SLOWCLK_HIGH_TIME;
		end loop;
	end process;

	LDC0_slowclk_gen : process
	begin
		LDC0_slowclk   <= '0';
		wait for LDC0_SLOWCLK_START_OFFSET;     -- 1 ns default
		loop
		LDC0_slowclk <= '0';
		wait for LDC0_SLOWCLK_LOW_TIME;       -- 50 MHz default
		LDC0_slowclk <= '1';
		wait for LDC0_SLOWCLK_HIGH_TIME;
		end loop;
	end process;

	LDC1_slowclk_gen : process
	begin
		LDC1_slowclk   <= '0';
		wait for LDC1_SLOWCLK_START_OFFSET;     -- 1 ns default
		loop
		LDC1_slowclk <= '0';
		wait for LDC1_SLOWCLK_LOW_TIME;       -- 50 MHz default
		LDC1_slowclk <= '1';
		wait for LDC1_SLOWCLK_HIGH_TIME;
		end loop;
	end process;
	
	-----------------------------------------------------------------------------
	-- Log data between FEMB/LSC and ROMB/LDC
	-----------------------------------------------------------------------------
	DATLOG_B : block
		file OUTFILE1 : text open write_mode is OUTFILE1NAME;
		file OUTFILE2DAQ : text open write_mode is OUTFILE2DAQNAME;
		file OUTFILE2FTK : text open write_mode is OUTFILE2FTKNAME;
		file OUTFILE3 : text open write_mode is OUTFILE3NAME;
		file OUTFILE4 : text open write_mode is OUTFILE4NAME;
		begin  -- block DATLOG_B	
		-------------------------------------------------------------------------------
		-- INLSC_WRITE process                                                       --
		-- User input words (UWEN_N low) written in file "fc_in.txt"                 --
		-------------------------------------------------------------------------------
		INLSC_WRITE : process
			variable ll       : line;
		begin
			wait until UCLK'event and UCLK = '1';
			if (UWEN_N = '0') then
				if (UCTRL_N = '0') then
					write(ll, UCTRL_N & UD);
					writeline(OUTFILE1, ll);
				else
					write(ll, UCTRL_N & UD);
					writeline(OUTFILE1, ll);
				end if;
			end if;
		end process;
		---------------------------------------------------------------------------
		-- OUTLDC_WRITE process (DAQ channel)                                    --
		-- Ouput words for ROMB (LWEN_N low) written in file "fc_out.txt"        --
		---------------------------------------------------------------------------
		OUTLDC0_WRITE : process
			variable ll, ll_error : line;
		begin
			ll_error := new string'("ERROR LED OR LDERR_N LINE ON");
			wait until LCLK0'event and LCLK0 = '1';
			if (LWEN0_N = '0') then
				write(ll, LCTRL0_N & LD0);
				writeline(OUTFILE2DAQ, ll);
			end if;
			if (((LDERR0_N = '0') or (LDC0_LDERRLEDN = '0')) and RLDOWN0_N = '1') then
				writeline(OUTFILE2DAQ, ll_error);
			end if;
		end process;
		---------------------------------------------------------------------------
		-- OUTLDC_WRITE process (FTK channel)                                    --
		-- Ouput words for ROMB (LWEN_N low) written in file "fc_out.txt"        --
		---------------------------------------------------------------------------
		OUTLDC1_WRITE : process
			variable ll, ll_error : line;
		begin
			ll_error := new string'("ERROR LED OR LDERR_N LINE ON");
			wait until LCLK1'event and LCLK1 = '1';
			if (LWEN1_N = '0') then
				write(ll, LCTRL1_N & LD1);
				writeline(OUTFILE2FTK, ll);
			end if;
			-- AK 07/26/2011: DERRLED latches upon receipt of an error
			-- DERRLED is only cleared with a reset OR by going in/out of test mode.
			-- Whoever writes FTK LDC firmware will need to ensure that DERRLED does not
			-- latch on the first frame received after synchronization!
			-- (because FTK LDC synchronizes in a "spy" mode, which can happen mid-frame)
--			if (((LDERR1_N = '0') or (LDC1_LDERRLEDN = '0')) and RLDOWN1_N = '1') then
			if (((LDERR1_N = '0')) and RLDOWN1_N = '1') then
				writeline(OUTFILE2FTK, ll_error);
			end if;
		end process;
		-----------------------------------------------------------------------------
		-- INLDC_WRITE                                                             --
		-- Input return lines from ROMB are written in file "rc_in.txt"            --
		-----------------------------------------------------------------------------
		INLDC_WRITE : process(URL0)
			variable ll : line;
		begin
			write(ll, URL0);
			writeline(OUTFILE3, ll);
		end process;
		-----------------------------------------------------------------------------
		-- OUTLSC_WRITE process                                                    --
		-- Output return lines for FEMB are written to file "rc_out.txt"           --
		-----------------------------------------------------------------------------
		OUTLSC_WRITE_ALL : process(LRL)
			variable ll       : line;
		begin
			write(ll, LRL);
			writeline(OUTFILE4,ll);
		end process;
		
	end block DATLOG_B;

end structure;
