-- test-bench for communicating between two instances of altgx_single
-- Note that this test bench does NOT include tlk wrappers

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity tb_gxss is
port (
	POWER_UP_RST_N    : in  std_logic;
	-- transmitter
	TXD            : in std_logic_vector(15 downto 0);
	TX_CTRL        : in std_logic_vector(1 downto 0);
	-- Cyclone transceiver ports (connect to pins)
	SLOWCLK       : in std_logic;   -- 50 MHz, reconfig & calib clock
	REFCLK        : in std_logic   -- 100 MHz, LVDS
);
end tb_gxss;

architecture structure of tb_gxss is	

-- altgx_reconfig component (required for receiver channels)
component altgx_reco_single
	PORT
	(
		reconfig_clk		: IN STD_LOGIC ;
		reconfig_fromgxb		: IN STD_LOGIC_VECTOR (4 DOWNTO 0);
		busy		: OUT STD_LOGIC ;
		error		: OUT STD_LOGIC ;
		reconfig_togxb		: OUT STD_LOGIC_VECTOR (3 DOWNTO 0)
	);
end component;

-- reset FSM components
component altgx_reset_tx
	port (
		clk             : in  std_logic;
		POWER_UP_RST_N  : in  std_logic;
		pll_locked      : in  std_logic_vector (0 downto 0);
		pll_areset      : out std_logic_vector (0 downto 0);
		tx_digitalreset : out std_logic_vector (0 downto 0);
		tx_ready        : out std_logic; -- for rx reset machine
		debug_st        : out std_logic_vector (1 downto 0) -- will be synthesized away if not used
	);
end component;
component altgx_reset_rx is
	generic (
		TLTD : integer := 200
	);
	port (
		clk             : in  std_logic;
		POWER_UP_RST_N  : in  std_logic;
		tx_ready        : in  std_logic; -- rx is only initialized after tx is up
		busy            : in  std_logic;
		rx_analogreset  : out std_logic_vector (0 downto 0);
		rx_freqlocked   : in  std_logic_vector (0 downto 0);
		rx_digitalreset : out std_logic_vector (0 downto 0);
		rx_ready        : out std_logic; -- for byte ordering block
		debug_st        : out std_logic_vector (1 downto 0) -- will be synthesized away if not used
	);
end component;

-- altgx component
component altgx_single
	PORT
	(
		cal_blk_clk		: IN STD_LOGIC ;
		pll_areset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		pll_inclk		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		reconfig_clk		: IN STD_LOGIC ;
		reconfig_togxb		: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		rx_analogreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_datain		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_ctrlenable		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_datain		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		tx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dispval		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_forcedisp		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		pll_locked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		reconfig_fromgxb		: OUT STD_LOGIC_VECTOR (4 DOWNTO 0);
		rx_clkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_ctrldetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_dataout		: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		rx_disperr		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_errdetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_freqlocked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_patterndetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_runningdisp		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_syncstatus		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_clkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dataout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0)
	);
end component;

	-- serial link signals (directly connect two transceivers)
	signal GXB_TX       : std_logic_vector(0 downto 0);
	signal GXB_RX       : std_logic_vector(0 downto 0);
	
	-- Signals for altgx_reconfig
	signal AAbusy_sig		: STD_LOGIC ;
	signal BBbusy_sig		: STD_LOGIC ;
	
	-- Signals for ALTGX AA
	signal AApll_areset_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal AAreconfig_togxb_sig		: STD_LOGIC_VECTOR (3 DOWNTO 0);
	signal AArx_analogreset_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal AArx_digitalreset_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal AAtx_digitalreset_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal AAtx_dispval_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal AAtx_forcedisp_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal AApll_locked_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal AAreconfig_fromgxb_sig		: STD_LOGIC_VECTOR (4 DOWNTO 0);
	signal AArx_clkout_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal AArx_ctrldetect_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal AArx_dataout_sig		: STD_LOGIC_VECTOR (15 DOWNTO 0);
	signal AArx_disperr_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal AArx_errdetect_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal AArx_freqlocked_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal AArx_patterndetect_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal AArx_runningdisp_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal AArx_syncstatus_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal AAtx_clkout_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal AAtx_ready_sig      : STD_LOGIC;
	signal AArx_ready_sig      : STD_LOGIC;

	-- Signals for ALTGX BB
	signal BBpll_areset_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal BBreconfig_togxb_sig		: STD_LOGIC_VECTOR (3 DOWNTO 0);
	signal BBrx_analogreset_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal BBrx_digitalreset_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal BBtx_digitalreset_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal BBtx_dispval_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal BBtx_forcedisp_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal BBpll_locked_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal BBreconfig_fromgxb_sig		: STD_LOGIC_VECTOR (4 DOWNTO 0);
	signal BBrx_clkout_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal BBrx_ctrldetect_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal BBrx_dataout_sig		: STD_LOGIC_VECTOR (15 DOWNTO 0);
	signal BBrx_disperr_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal BBrx_errdetect_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal BBrx_freqlocked_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal BBrx_patterndetect_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal BBrx_runningdisp_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal BBrx_syncstatus_sig		: STD_LOGIC_VECTOR (1 DOWNTO 0);
	signal BBtx_clkout_sig		: STD_LOGIC_VECTOR (0 DOWNTO 0);
	signal BBtx_ready_sig      : STD_LOGIC;
	signal BBrx_ready_sig      : STD_LOGIC;

begin

	-------------------------------------------------------
	--- FIRST TRANSMITTER (LSC)
	-------------------------------------------------------
	-- altgx_reconfig instance
	reco_instAA : altgx_reco_single PORT MAP (
		reconfig_clk	 => SLOWCLK,
		reconfig_fromgxb	 => AAreconfig_fromgxb_sig,
		busy	 => AAbusy_sig,
		error	 => open,
		reconfig_togxb	 => AAreconfig_togxb_sig
	);

	-- transmitter reset fsm instance
	tx_rstAA : altgx_reset_tx PORT MAP (
		clk              => SLOWCLK,
		POWER_UP_RST_N   => POWER_UP_RST_N,
		pll_locked       => AApll_locked_sig,
		pll_areset       => AApll_areset_sig,
		tx_digitalreset  => AAtx_digitalreset_sig,
		tx_ready         => AAtx_ready_sig,
		debug_st         => open
	);
	
	-- gx instance
	gx_instAA : altgx_single PORT MAP (
		cal_blk_clk	 => SLOWCLK,
		pll_areset	 => AApll_areset_sig,
		pll_inclk(0)	 => REFCLK,
		reconfig_clk	 => SLOWCLK,
		reconfig_togxb	 => AAreconfig_togxb_sig,
		rx_analogreset	 => AArx_analogreset_sig,
		rx_datain	 => GXB_RX,
		rx_digitalreset	 => AArx_digitalreset_sig,
		tx_ctrlenable	 => TX_CTRL,  -- what we send
		tx_datain 		 => TXD,
		tx_digitalreset	 => AAtx_digitalreset_sig,
		tx_dispval => "00",
		tx_forcedisp	 => "00",
		pll_locked	 => AApll_locked_sig,
		reconfig_fromgxb	 => AAreconfig_fromgxb_sig,
		rx_clkout	 => AArx_clkout_sig,
		rx_ctrldetect	 => AArx_ctrldetect_sig,
		rx_dataout	 => AArx_dataout_sig,
		rx_disperr   => AArx_disperr_sig,
		rx_errdetect	 => AArx_errdetect_sig,
		rx_freqlocked	 => AArx_freqlocked_sig,
		rx_patterndetect => AArx_patterndetect_sig,
		rx_runningdisp	 => AArx_runningdisp_sig,
		rx_syncstatus	 => AArx_syncstatus_sig,
		tx_clkout	 => AAtx_clkout_sig,
		tx_dataout	 => GXB_TX
	);
	
	-- receiver reset fsm instance
	rx_rstAA : altgx_reset_rx 
		GENERIC MAP (
			TLTD => 4
		)
		PORT MAP (
			clk              => SLOWCLK,
			POWER_UP_RST_N   => POWER_UP_RST_N,
			tx_ready         => AAtx_ready_sig,
			busy             => AAbusy_sig,
			rx_analogreset   => AArx_analogreset_sig,
			rx_freqlocked    => AArx_freqlocked_sig,
			rx_digitalreset  => AArx_digitalreset_sig,
			rx_ready         => AArx_ready_sig,
			debug_st         => open
		);

		
	-------------------------------------------------------
	--- SECOND TRANSMITTER (LDC)
	-------------------------------------------------------
	-- altgx_reconfig instance
	reco_instBB : altgx_reco_single PORT MAP (
		reconfig_clk	 => SLOWCLK,
		reconfig_fromgxb	 => BBreconfig_fromgxb_sig,
		busy	 => BBbusy_sig,
		error	 => open,
		reconfig_togxb	 => BBreconfig_togxb_sig
	);

	-- transmitter reset fsm instance
	tx_rstBB : altgx_reset_tx PORT MAP (
		clk              => SLOWCLK,
		POWER_UP_RST_N   => POWER_UP_RST_N,
		pll_locked       => BBpll_locked_sig,
		pll_areset       => BBpll_areset_sig,
		tx_digitalreset  => BBtx_digitalreset_sig,
		tx_ready         => BBtx_ready_sig,
		debug_st         => open
	);
	
	-- gx instance
	gx_instBB : altgx_single PORT MAP (
		cal_blk_clk	 => SLOWCLK,
		pll_areset	 => BBpll_areset_sig,
		pll_inclk(0)	 => REFCLK,
		reconfig_clk	 => SLOWCLK,
		reconfig_togxb	 => BBreconfig_togxb_sig,
		rx_analogreset	 => BBrx_analogreset_sig,
		rx_datain	 => GXB_TX,
		rx_digitalreset	 => BBrx_digitalreset_sig,
		tx_ctrlenable	 => TX_CTRL,  -- what we send
		tx_datain 		 => TXD,
		tx_digitalreset	 => BBtx_digitalreset_sig,
		tx_dispval => "00",
		tx_forcedisp	 => "00",
		pll_locked	 => BBpll_locked_sig,
		reconfig_fromgxb	 => BBreconfig_fromgxb_sig,
		rx_clkout	 => BBrx_clkout_sig,
		rx_ctrldetect	 => BBrx_ctrldetect_sig,
		rx_dataout	 => BBrx_dataout_sig,
		rx_disperr   => BBrx_disperr_sig,
		rx_errdetect	 => BBrx_errdetect_sig,
		rx_freqlocked	 => BBrx_freqlocked_sig,
		rx_patterndetect => BBrx_patterndetect_sig,
		rx_runningdisp	 => BBrx_runningdisp_sig,
		rx_syncstatus	 => BBrx_syncstatus_sig,
		tx_clkout	 => BBtx_clkout_sig,
		tx_dataout	 => GXB_RX
	);
	
	-- receiver reset fsm instance
	rx_rstBB : altgx_reset_rx 
		GENERIC MAP (
			TLTD => 4
		)
		PORT MAP (
			clk              => SLOWCLK,
			POWER_UP_RST_N   => POWER_UP_RST_N,
			tx_ready         => BBtx_ready_sig,
			busy             => BBbusy_sig,
			rx_analogreset   => BBrx_analogreset_sig,
			rx_freqlocked    => BBrx_freqlocked_sig,
			rx_digitalreset  => BBrx_digitalreset_sig,
			rx_ready         => BBrx_ready_sig,
			debug_st         => open

		);

	
end structure;
