-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--                                                                           --
--  File name : tb_oldlink.vhd                                             --
--  Title     : Testbench for HOLA S-LINK Simulation                         --
--  Author(s) : Aurelio Ruiz Garcia                                          --
--              Jesper Moller Haee                                           --
--              Stefan Haas                                                  --
--  Notes     : Box containig the whole structure of an HOLA S-Link          --
--              implementation.                                              --
--                                                                           --
--                                                                           --
--      +------+   +------+   +------+   +------+   +------+   +------+      --
--      |      |-->|      |-->|      |-->|      |-->|      |-->|      |      --
--      | FEMB |   | HOLA |   | TLK  |   | TLK  |   | HOLA |   | ROMB |      --
--      |      |   | LSC  |   | 2501 |   | 2501 |   | LDC  |   |      |      --
--      |      |<--|      |<--|      |<--|      |<--|      |<--|      |      --
--      +------+   +------+   +------+   +------+   +------+   +------+      --
--                                                                           --
--             Inputs and outputs of the link are written into the following --
--             text files:                                                   --
--                                                                           --
--               - fc_in.txt  -> Forward channel input data  : UCTRL_N, UD   --
--               - fc_out.txt -> Forward channel output data : LCTRL_N, LD   --
--               - rc_in.txt  -> Return channel input data   : URL           --
--               - rc_out.txt -> Return channel output data  : LRL           --
--                                                                           --
-------------------------------------------------------------------------------
--     Version | Author |  Mod.Date   |          Changes made                --
-------------------------------------------------------------------------------
--     1.0     |  ARG   | 11-Mar-2002 | Original coding                      --
--     1.1     |  ARG   | 17-Apr-2002 | TX_ERROR added                       --
--     1.1     |  ARG   | 21-May-2002 | FIBER_DISC added                     --
--     1.2     |  JMH   |  9-dec-2002 | DLL_RESET added to both LDC and LSC  --
--             |        |             | the signal only affects xilinx ver.  --
--     1.3     |  JMH   | 24-jan-2003 | XCLK seperated into LSC_xclk and     --
--             |        |             | LDC_xclk these are now slightly      --
--             |        |             | misaligned to be more realistic      --
--             |        |             | also the RX_CLK signals is seperated --
--     1.4     |  JMH   | 12-feb-2003 | generics added for XCLKs             --
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_textio.all ;
use std.textio.all;


entity tb_oldlink is
  generic (
    POWER_UP_RST_DELAY_TIME     : time      := 1 ns;
    POWER_UP_RST_TIME           : time      := 10 ns;
	POWER_UP_RST_LEN_TIME       : time      := 100000 ns;
    LSC_XCLK_HIGH_TIME          : time      := 5 ns; -- AKK changed all times to 100 MHz
    LSC_XCLK_LOW_TIME           : time      := 5 ns;
    LDC_XCLK_HIGH_TIME          : time      := 5 ns;
    LDC_XCLK_LOW_TIME           : time      := 5 ns;
    LDC_XCLK_START_OFFSET       : time      := 1 ns
  );
end tb_oldlink;

library hola_sim_lib;
use hola_sim_lib.hola_sim_components.all;

library work;
library work;

architecture structure of tb_oldlink is

  signal POWER_UP_RST_N : std_logic := '0';
  -------------------------------------------------------------------------------
  -- signals FEMB <-> LSC
  -------------------------------------------------------------------------------

  signal UD       : std_logic_vector(1 to 32);
  signal LFF_n    : std_logic;
  signal URESET_n : std_logic := '0';
  signal UTEST_n  : std_logic;
  signal UCTRL_n  : std_logic;
  signal UWEN_n   : std_logic;
  signal UCLK     : std_logic := '0';
  signal LRL      : std_logic_vector(1 to 4);
  signal LDOWN_n  : std_logic;

  -------------------------------------------------------------------------------
  -- signals LDC <-> ROMB
  -------------------------------------------------------------------------------
  signal LD        :   std_logic_vector(1 to 32);
  signal UXOFF_n   :   std_logic;
  signal RURESET_n :   std_logic := '0';
  signal UTDO_n    :   std_logic;
  signal LCTRL_n   :   std_logic;
  signal LWEN_n    :   std_logic;
  signal LCLK      :   std_logic;
  signal LDERR_n   :   std_logic;
  signal URL       :   std_logic_vector(1 to 4);
  signal RLDOWN_n  :   std_logic;

  -----------------------------------------------------------------------------
  -- LSC <-> TLK
  -----------------------------------------------------------------------------
  signal LSC_XCLK : std_logic;
  signal LSC_TXEN : std_logic;
  signal LSC_TXER : std_logic;
  signal LSC_TXD  : std_logic_vector(15 downto 0);
  signal LSC_ENABLE : std_logic;

  signal LSC_RXCLK : std_logic;
  signal LSC_RXER : std_logic;
  signal LSC_RXDV : std_logic;
  signal LSC_RXD  : std_logic_vector(15 downto 0);

  signal LSC_RXCLK1 : std_logic := '0';
  signal LSC_RXER1 : std_logic := '1';
  signal LSC_RXDV1 : std_logic := '1';
  signal LSC_RXD1  : std_logic_vector(15 downto 0);

  -----------------------------------------------------------------------------
  -- LDC <-> TLK
  -----------------------------------------------------------------------------
  signal LDC_XCLK : std_logic;
  signal LDC_TXEN : std_logic;
  signal LDC_TXER : std_logic;
  signal LDC_TXD  : std_logic_vector(15 downto 0);
  signal LDC_ENABLE : std_logic;

  signal LDC_RXCLK : std_logic;
  signal LDC_RXER : std_logic;
  signal LDC_RXDV : std_logic;
  signal LDC_RXD  : std_logic_vector(15 downto 0);

  -----------------------------------------------------------------------------
  -- LSC and LDC LED signals
  -----------------------------------------------------------------------------
  signal LSC_TESTLEDN     : std_logic;
  signal LSC_LUPLEDN      : std_logic;
  signal LSC_LDERRLEDN    : std_logic;
  signal LSC_ACTIVITYLEDN : std_logic;
  signal LSC_FLOWCTLLEDN  : std_logic;

  signal LDC_TESTLEDN     : std_logic;
  signal LDC_LUPLEDN      : std_logic;
  signal LDC_LDERRLEDN    : std_logic;
  signal LDC_ACTIVITYLEDN : std_logic;
  signal LDC_FLOWCTLLEDN  : std_logic;

  -----------------------------------------------------------------------------
  -- Other signals
  -----------------------------------------------------------------------------

  signal LCLK_PLL        : std_logic;   -- external clock connection on LDC
  signal DLL_RESET  : std_logic; -- DLL reset for Xilinx version
  signal GND : std_logic := '0';
  
  component holalsc_core
    port (
	  POWER_UP_RST_N : in  std_logic;
      UD            : in  std_logic_vector(31 downto 0);
      URESET_N      : in  std_logic;
      UTEST_N       : in  std_logic;
      UCTRL_N       : in  std_logic;
      UWEN_N        : in  std_logic;
      UCLK          : in  std_logic;
      LFF_N         : out std_logic;
	  XOFF_ENA      : out std_logic;
	  FTK_SEND_ACK  : out std_logic;
      LRL           : out std_logic_vector( 3 downto 0);
      LDOWN_N       : out std_logic;
      TESTLED_N     : out std_logic;
      LDERRLED_N    : out std_logic;
      LUPLED_N      : out std_logic;
      FLOWCTLLED0_N : out std_logic;
	  FLOWCTLLED1_N : out std_logic;
      ACTIVITYLED_N : out std_logic;
      XCLK          : in  std_logic;
	  PLL_CLK       : in  std_logic;
	  TX_CLK        : in  std_logic;
      ENABLE        : out std_logic;
      TXD           : out std_logic_vector(15 downto 0);
      TX_EN         : out std_logic;
      TX_ER         : out std_logic;
      RXD0          : in  std_logic_vector(15 downto 0);
      RX_CLK0       : in  std_logic;
      RX_ER0        : in  std_logic;
      RX_DV0        : in  std_logic;
	  LOS0_stable   : in  std_logic;
	  RXD1          : in  std_logic_vector(15 downto 0);
      RX_CLK1       : in  std_logic;
      RX_ER1        : in  std_logic;
      RX_DV1        : in  std_logic;
	  LOS1_stable   : in  std_logic;
      DLL_RESET     : in  std_logic);
  end component;

  component holaldc_core
    port (
	  POWER_UP_RST_N : in  std_logic;
      LD            : out std_logic_vector(31 downto 0);
      UXOFF_N       : in  std_logic;
      URESET_N      : in  std_logic;
      UTDO_N        : in  std_logic;
      LCTRL_N       : out std_logic;
      LWEN_N        : out std_logic;
	  LCLK          : out std_logic;
      LDERR_N       : out std_logic;
      URL           : in  std_logic_vector( 3 downto 0);
      LDOWN_N       : out std_logic;
      TESTLED_N     : out std_logic;
      LDERRLED_N    : out std_logic;
      LUPLED_N      : out std_logic;
      FLOWCTLLED_N  : out std_logic;
      ACTIVITYLED_N : out std_logic;
      XCLK          : in  std_logic;
	  TX_CLK        : in  std_logic;
      ENABLE        : out std_logic;
      TXD           : out std_logic_vector(15 downto 0);
      TX_EN         : out std_logic;
      TX_ER         : out std_logic;
      RXD           : in  std_logic_vector(15 downto 0);
      RX_CLK        : in  std_logic;
      RX_ER         : in  std_logic;
      RX_DV         : in  std_logic;
      LCLK_IN       : in  std_logic;
      DLL_RESET     : in  std_logic);
  end component;

begin  -- structure

-------------------------------------------------------------------------------
-->       FEMB instantiation                                                <--
-------------------------------------------------------------------------------

  FEMB1: FEMB
    port map (
      UD       => UD,
      LFF_n    => LFF_n,
      URESET_n => URESET_n,
      UTEST_n  => UTEST_n,
      UCTRL_n  => UCTRL_n,
      UWEN_n   => UWEN_n,
      UCLK     => UCLK,
      LRL      => LRL,
      LDOWN_n  => LDOWN_n);

  HOLALSC1: holalsc_core
    port map (
	  POWER_UP_RST_N => POWER_UP_RST_N,
      UD            => UD,
      URESET_N      => URESET_n,
      UTEST_N       => UTEST_n,
      UCTRL_N       => UCTRL_n,
      UWEN_N        => UWEN_n,
      UCLK          => UCLK,
      LFF_N         => LFF_n,
	  XOFF_ENA      => open,
	  FTK_SEND_ACK  => open,
      LRL           => LRL,
      LDOWN_N       => LDOWN_n,
      TESTLED_N     => LSC_TESTLEDN,
      LDERRLED_N    => LSC_LDERRLEDN,
      LUPLED_N      => LSC_LUPLEDN,
      FLOWCTLLED0_N  => LSC_FLOWCTLLEDN,
	  FLOWCTLLED1_N  => open,
      ACTIVITYLED_N => LSC_ACTIVITYLEDN,
      XCLK          => LSC_XCLK,
	  PLL_CLK       => LSC_XCLK,
	  TX_CLK        => LSC_XCLK,
      ENABLE        => LSC_ENABLE,
      TXD           => LSC_TXD,
      TX_EN         => LSC_TXEN,
      TX_ER         => LSC_TXER,
      RXD0          => LSC_RXD,
      RX_CLK0       => LSC_RXCLK,
      RX_ER0        => LSC_RXER,
      RX_DV0        => LSC_RXDV,
	  LOS0_stable   => GND,
	  RXD1          => LSC_RXD1,
      RX_CLK1       => LSC_RXCLK1,
      RX_ER1        => LSC_RXER1,
      RX_DV1        => LSC_RXDV1,
	  LOS1_stable   => GND,
      DLL_RESET     => DLL_RESET);

  -----------------------------------------------------------------------------
  -- TLK2501 instantiation
  -----------------------------------------------------------------------------

  TLK2501_1: tlk2501
    port map (
      LOOPEN     => GND,
      PRBSEN     => GND,
      TX_EN      => LSC_TXEN,
      TX_ER      => LSC_TXER,
      TD         => LSC_TXD,
      GTX_CLK    => LSC_XCLK,
      TESTEN     => GND,
      ENABLE     => LSC_ENABLE,
      RX_ER      => LDC_RXER,
      RX_CLK     => LDC_RXCLK,
      RX_DV      => LDC_RXDV,
      RD         => LDC_RXD);

  -----------------------------------------------------------------------------
  -- TLK2501 instantiation
  -----------------------------------------------------------------------------

  TLK2501_2: tlk2501
    port map (
      LOOPEN     => GND,
      PRBSEN     => GND,
      TX_EN      => LDC_TXEN,
      TX_ER      => LDC_TXER,
      TD         => LDC_TXD,
      GTX_CLK    => LDC_XCLK,
      TESTEN     => GND,
      ENABLE     => LDC_ENABLE,
      RX_ER      => LSC_RXER,
      RX_CLK     => LSC_RXCLK,
      RX_DV      => LSC_RXDV,
      RD         => LSC_RXD);

  -----------------------------------------------------------------------------
  -- HOLA LDC instantiation
  -----------------------------------------------------------------------------

  HOLALDC1: holaldc_core
    port map (
	  POWER_UP_RST_N => POWER_UP_RST_N,
      LD             => LD,
      UXOFF_N        => UXOFF_N,
      URESET_N       => RURESET_N,
      UTDO_N         => UTDO_N,
      LCTRL_N        => LCTRL_N,
      LWEN_N         => LWEN_N,
      LCLK           => LCLK,
      LDERR_N        => LDERR_N,
      URL            => URL,
      LDOWN_N        => RLDOWN_N,
      TESTLED_N      => LDC_TESTLEDN,
      LDERRLED_N     => LDC_LDERRLEDN,
      LUPLED_N       => LDC_LUPLEDN,
      FLOWCTLLED_N   => LDC_FLOWCTLLEDN,
      ACTIVITYLED_N  => LDC_ACTIVITYLEDN,
      XCLK           => LDC_XCLK,
	  TX_CLK         => LDC_XCLK,
      ENABLE         => LDC_ENABLE,
      TXD            => LDC_TXD,
      TX_EN          => LDC_TXEN,
      TX_ER          => LDC_TXER,
      RXD            => LDC_RXD,
      RX_CLK         => LDC_RXCLK,
      RX_ER          => LDC_RXER,
      RX_DV          => LDC_RXDV,
	  LCLK_IN        => LCLK_PLL,
      DLL_RESET      => DLL_RESET);

-------------------------------------------------------------------------------
-->       ROMB  instantiation                                               <--
-------------------------------------------------------------------------------

  ROMB1: ROMB
    port map (
      LD       => LD,
      UXOFF_n  => UXOFF_n,
      URESET_n => RURESET_n,
      UTDO_n   => UTDO_n,
      LCTRL_n  => LCTRL_n,
      LWEN_n   => LWEN_n,
      LCLK     => LCLK,
      LDERR_n  => LDERR_n,
      URL      => URL,
      LDOWN_n  => RLDOWN_n);

  -----------------------------------------------------------------------------
  -- Other signals
  -----------------------------------------------------------------------------

  LCLK_PLL  <= LCLK;                    -- connect LCLK output to LCLK_PLL input
  DLL_RESET <= '0';

  -----------------------------------------------------------------------------
  -- POWER-UP RESET
  -----------------------------------------------------------------------------
  power_up_prc : process
  begin
	loop
		POWER_UP_RST_N <= '1';
		wait for POWER_UP_RST_DELAY_TIME;
		POWER_UP_RST_N <= '0';
		wait for POWER_UP_RST_TIME;
		POWER_UP_RST_N <= '1';
		wait for POWER_UP_RST_LEN_TIME;
	end loop;
  end process;
  
  -----------------------------------------------------------------------------
  -- External 125 MHz clock generation                                       --
  -----------------------------------------------------------------------------

  LSC_xclk_gen : process
  begin
    loop
      LSC_xclk <= '0';
      wait for LSC_XCLK_LOW_TIME;       -- 125 MHz default
      LSC_xclk <= '1';
      wait for LSC_XCLK_HIGH_TIME;
    end loop;

  end process;

  LDC_xclk_gen : process
  begin
    LDC_xclk   <= '0';
    wait for LDC_XCLK_START_OFFSET;     -- 1 ns default
    loop
      LDC_xclk <= '0';
      wait for LDC_XCLK_LOW_TIME;       -- 125 MHz default
      LDC_xclk <= '1';
      wait for LDC_XCLK_HIGH_TIME;
    end loop;
  end process;

  -----------------------------------------------------------------------------
  -- Log data between FEMB/LSC and ROMB/LDC
  -----------------------------------------------------------------------------

  DATLOG_B : block
    file OUTFILE1 : text open write_mode is "out_oldlink_fc_in.txt";
    file OUTFILE2 : text open write_mode is "out_oldlink_fc_out.txt";
    file OUTFILE3 : text open write_mode is "out_oldlink_rc_in.txt";
    file OUTFILE4 : text open write_mode is "out_oldlink_rc_out.txt";
	begin  -- block DATLOG_B

    -------------------------------------------------------------------------------
    -- INLSC_WRITE process                                                       --
    -- User input words (UWEN_N low) written in file "fc_in.txt"                 --
    -------------------------------------------------------------------------------
    INLSC_WRITE : process
      variable ll       : line;
    begin
      wait until UCLK'event and UCLK = '1';
      if (UWEN_N = '0') then
        if (UCTRL_N = '0') then
          write(ll, UCTRL_N & UD);
          writeline(OUTFILE1, ll);
        else
          write(ll, UCTRL_N & UD);
          writeline(OUTFILE1, ll);
        end if;
      end if;
    end process;

    ---------------------------------------------------------------------------
    -- OUTLDC_WRITE process                                                  --
    -- Ouput words for ROMB (LWEN_N low) written in file "fc_out.txt"        --
    ---------------------------------------------------------------------------
    OUTLDC_WRITE : process
      variable ll, ll_error : line;
    begin
      ll_error := new string'("ERROR LED OR LDERR_N LINE ON");
      wait until LCLK'event and LCLK = '1';
      if (LWEN_N = '0') then
        write(ll, LCTRL_N & LD);
        writeline(OUTFILE2, ll);
      end if;
      if (((LDERR_N = '0') or (LDC_LDERRLEDN = '0')) and RLDOWN_N = '1') then
        writeline(OUTFILE2, ll_error);
      end if;
    end process;

    -----------------------------------------------------------------------------
    -- INLDC_WRITE                                                             --
    -- Input return lines from ROMB are written in file "rc_in.txt"            --
    -----------------------------------------------------------------------------
    INLDC_WRITE : process(URL)
      variable ll : line;
    begin
      write(ll, URL);
      writeline(OUTFILE3, ll);
    end process;

    -----------------------------------------------------------------------------
    -- OUTLSC_WRITE process                                                    --
    -- Output return lines for FEMB are written to file "rc_out.txt"           --
    -----------------------------------------------------------------------------
    OUTLSC_WRITE : process(LRL)
      variable ll       : line;
    begin
      write(ll, LRL);
      writeline(OUTFILE4,ll);
    end process;
    
  end block DATLOG_B;
  
end structure;

-------------------------------------------------------------------------------
-- Configuration
-------------------------------------------------------------------------------

library work;
library work;
library hola_sim_lib;

configuration cfg_tb_oldlink_functional_altera of tb_oldlink is

  for structure

    for all: FEMB use entity hola_sim_lib.FEMB(ARCH)
      generic map (
        UCLK_HIGH      =>  12500 ps,  -- UCLK high time
        UCLK_LOW       =>  12500 ps,  -- UCLK low time
        DATA_SETUP     =>     10 ns,  -- Data setup time
        DATA_HOLD      =>      1 ns,  -- Data hold time
        ENABLE_SETUP   =>     10 ns,  -- Enable setup time
        ENABLE_HOLD    =>      1 ns,  -- Enable hold time
        MAX_CLK_TO_LFF =>     12 ns,  -- Maximum clock to Full flag
		FEMB_IN_FILENAME  => "femb_in.txt",
		FEMB_OUT_FILENAME => "out_oldlink_femb_out.txt"
      );
    end for;
    
    for all : tlk2501 use entity hola_sim_lib.tlk2501(beh_tlk2501)
      generic map(
        in_setup          => 1.5 ns,
        in_hold           => 0.4 ns,  -- no timing in func sim --AKK was:0
        out_setup         => 3.0 ns,
        out_hold          => 3.0 ns   -- no timing in func sim --AKK was:0
      );
    end for;
    
    -- to simulate the LSC, using Altera or Xilinx,
    -- simply change the generic "ALTERA_XILINX"
    for all : HOLALSC_CORE use entity work.holalsc_core(structure)
      generic map(
        SIMULATION        => 1,   -- simulation mode
        ALTERA_XILINX     => 1,   -- 1=Altera 0=Xilinx
		XCLK_FREQ         => 100, -- Reference clock oscillator frequency
		USE_PLL           => 1,
        ACTIVITY_LENGTH   => 15,  -- ACTLED duration
        FIFODEPTH         => 64,  -- only powers of 2
        LOG2DEPTH         => 6,   -- 2log of depth
        FULLMARGIN        => 16   -- words left when LFF_N  set
      );
      for structure
        use work.all;
      end for;
    end for;
    
    -- to simulate the LDC, using Altera or Xilinx,
    -- simply change the generic "ALTERA_XILINX"
    for all : HOLALDC_CORE use entity work.holaldc_core(structure)
      generic map(
        ALTERA_XILINX    => 1,    -- 1=Altera 0=Xilinx
        SIMULATION       => 1,    -- simulation mode
        LCLK_SELECT      => 1,    -- LCLK generator selection
                                  -- 0 -> no clk gen. 1 -> 40MHz clk gen.
                                  -- 2 -> 62.5MHz clk gen.
		XCLK_FREQ        => 100,  -- Reference clock oscillator frequency
        ACTIVITY_LENGTH  => 15,   -- ACTLED duration
        FIFODEPTH        => 512,  -- only powers of 2
        LOG2DEPTH        => 9,    -- 2log of depth
        FULLMARGIN       => 256   -- words left when LFF_N set
      );
      for structure
        use work.all;
      end for;
    end for;

    for all : ROMB use entity hola_sim_lib.ROMB(ARCH)
      generic map(
        DATA_SETUP   =>  10 ns,  -- Data setup time
        DATA_HOLD    =>   0 ns,  -- Data hold  time
        ENABLE_SETUP =>  10 ns,  -- Enable setup time
        ENABLE_HOLD  =>   0 ns,  -- Enable hold  time
        CLOCK_PERIOD =>  25 ns,   -- Clock period
		ROMB_IN_FILENAME  => "romb_in.txt",
		ROMB_OUT_FILENAME => "out_oldlink_romb_out.txt"
      );
    end for;

  end for;                              -- structure

end cfg_tb_oldlink_functional_altera;
