-- test-bench for communicating between:
-- altgx_tlk_dual and two altgs_tlk_single's

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity tb_tgxds is
generic(
	SIMULATION  : integer :=  0
);
port (
	POWER_UP_RST_N0    : in  std_logic;
	POWER_UP_RST_N1    : in  std_logic;
	-- transmitter
	TXD            : in std_logic_vector(15 downto 0);
	TX_EN          : in std_logic;
	TX_ER          : in std_logic;
	TX_CLK         : out std_logic;
	-- receiver daq
	RXD0           : out std_logic_vector(15 downto 0);
	RX_ER0         : out std_logic;
	RX_DV0         : out std_logic;
	RX_CLK0        : out std_logic;	
	-- reveiver ftk
	RXD1           : out std_logic_vector(15 downto 0);
	RX_ER1         : out std_logic;
	RX_DV1         : out std_logic;
	RX_CLK1        : out std_logic;
	-- Cyclone transceiver ports (connect to pins)
	SLOWCLK       : in std_logic;   -- 50 MHz, reconfig & calib clock
	REFCLK        : in std_logic;   -- 100 MHz, LVDS
	debug_bits    : out std_logic_vector(3 downto 0)
);
end tb_tgxds;

architecture structure of tb_tgxds is	

component altgx_tlk_single is
	port (
		POWER_UP_RST_N    : in  std_logic;
		-- transmitter
		TXD            : in std_logic_vector(15 downto 0);
		TX_EN          : in std_logic;
		TX_ER          : in std_logic;
		TX_CLK         : out std_logic;  -- generated internally, passed to LSC/LDC core
		-- receiver (from DAQ):
		RXD0           : out std_logic_vector(15 downto 0); -- Received parallel data
		RX_CLK0        : out std_logic;         -- Recovered clock.
		RX_LOCKED0     : out std_logic;
		RX_ER0         : out std_logic;
		RX_DV0         : out std_logic;
		-- Cyclone transceiver ports (connect to pins)
		SLOWCLK       : in std_logic;   -- 50 MHz, reconfig & calib clock
		REFCLK        : in std_logic;   -- 100 MHz, LVDS
		GXB_TX0       : out std_logic;
		GXB_RX0       : in std_logic
	   );
end component;

component altgx_tlk_dual is
	port (
		POWER_UP_RST_N    : in  std_logic;
		-- transmitter (to both DAQ and FTK)
		TXD            : in std_logic_vector(15 downto 0);
		TX_EN          : in std_logic;
		TX_ER          : in std_logic;
		TX_CLK         : out std_logic;  -- generated internally, passed to LSC/LDC core
		-- receiver (from DAQ):
		RXD0           : out std_logic_vector(15 downto 0); -- Received parallel data
		RX_CLK0        : out std_logic;         -- Recovered clock.
		RX_LOCKED0     : out std_logic;
		RX_ER0         : out std_logic;
		RX_DV0         : out std_logic;
		-- receiver (from FTK):
		RXD1           : out std_logic_vector(15 downto 0); -- Received parallel data
		RX_CLK1        : out std_logic;         -- Recovered clock.
		RX_LOCKED1     : out std_logic;
		RX_ER1         : out std_logic;
		RX_DV1         : out std_logic;
		-- Cyclone transceiver ports (connect to pins)
		SLOWCLK       : in std_logic;   -- 50 MHz, reconfig & calib clock
		REFCLK        : in std_logic;   -- 100 MHz, LVDS
		GXB_TX0       : out std_logic;
		GXB_RX0       : in std_logic;
		GXB_TX1       : out std_logic;
		GXB_RX1       : in std_logic;
		debug_bits    : out std_logic_vector(3 downto 0)
	);
end component;
   
	-- serial link signals (directly connect two transceivers)
	signal GXB_TX0       : std_logic;
	signal GXB_TX1       : std_logic;
	signal GXB_RX0       : std_logic;
	signal GXB_RX1       : std_logic;

begin

	-------------------------------------------------------
	--- FIRST TRANSMITTER (LSC)
	-------------------------------------------------------
	dual : altgx_tlk_dual
	port map(
		POWER_UP_RST_N   => POWER_UP_RST_N0,
		-- transmitter
		TXD              => TXD,
		TX_EN            => TX_EN,
		TX_ER            => TX_ER,
		TX_CLK           => TX_CLK,
		-- receiver (from DAQ):
		RXD0             => RXD0,
		RX_CLK0          => RX_CLK0,
		RX_LOCKED0       => open,
		RX_ER0           => RX_ER0,
		RX_DV0           => RX_DV0,
		-- receiver (from FTK):
		RXD1             => RXD1,
		RX_CLK1          => RX_CLK1,
		RX_LOCKED1       => open,
		RX_ER1           => RX_ER1,
		RX_DV1           => RX_DV1,
		-- clocks
		SLOWCLK          => SLOWCLK,
		REFCLK           => REFCLK,
		-- serial links
		GXB_TX0          => GXB_TX0,
		GXB_TX1          => GXB_TX1,
		GXB_RX0          => GXB_RX0,
		GXB_RX1          => GXB_RX1,
		debug_bits       => debug_bits
	   );
	   
	-------------------------------------------------------
	--- SECOND TRANSMITTER (LDC DAQ)
	-------------------------------------------------------
	single0 : altgx_tlk_single
	port map(
		POWER_UP_RST_N   => POWER_UP_RST_N0,
		-- transmitter
		TXD              => TXD,
		TX_EN            => TX_EN,
		TX_ER            => TX_ER,
		TX_CLK           => open,
		-- receiver (from DAQ):
		RXD0             => open,
		RX_CLK0          => open,
		RX_LOCKED0       => open,
		RX_ER0           => open,
		RX_DV0           => open,
		-- clocks
		SLOWCLK          => SLOWCLK,
		REFCLK           => REFCLK,
		-- serial links
		GXB_TX0          => GXB_RX0,
		GXB_RX0          => GXB_TX0
	   );
	
	-------------------------------------------------------
	--- THIRD TRANSMITTER (LDC FTK)
	-------------------------------------------------------
	single1 : altgx_tlk_single
	port map(
		POWER_UP_RST_N   => POWER_UP_RST_N1,
		-- transmitter
		TXD              => TXD,
		TX_EN            => TX_EN,
		TX_ER            => TX_ER,
		TX_CLK           => open,
		-- receiver (from DAQ):
		RXD0             => open,
		RX_CLK0          => open,
		RX_LOCKED0       => open,
		RX_ER0           => open,
		RX_DV0           => open,
		-- clocks
		SLOWCLK          => SLOWCLK,
		REFCLK           => REFCLK,
		-- serial links
		GXB_TX0          => GXB_RX1,
		GXB_RX0          => GXB_TX1
	   );
		
end structure;


library work;
configuration cfg_tb_tgxds_timing_altera of tb_tgxds is

  for structure

    for all: altgx_tlk_single use entity work.altgx_tlk_single(structure)
      generic map (
		SIMULATION => 0
      );
    end for;
    
    -- Because we are using a gate-level altgx_tlk_dual entity, we 
	-- cannot use any generics/configurations for lsc entity
    
  end for;                              -- structure

end cfg_tb_tgxds_timing_altera;
