-- wrapper around TLK2501 [ DO NOT USE, THIS IS BROKEN ]
-- [ USE tlk_* files instead for a working TLK2501 wrapper! ]

-- Clocks:
-- tx and rx clocks (100 MHz) are generated in ALTGX via PLL
-- inside LSC core, tx clock will reduced through another PLL to 50 MHz

LIBRARY work;
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL; -- for comparison operator

-- Receiver FSM that implements tlk2501 synchronization for ALTGX
ENTITY tlk2501_receive IS
	PORT (
		RESET : IN STD_LOGIC;
		-- signals to LSC core, DAQ and FTK
		RX_ER : OUT STD_LOGIC;
		RX_DV : OUT STD_LOGIC;
		RD    : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		-- signals from/to ALTGX
		RX_CLK : IN STD_LOGIC;      				   -- recovered clock
		GX_RD : IN STD_LOGIC_VECTOR(15 DOWNTO 0);      -- datain, two bytes
		GX_RDCTRL : IN STD_LOGIC_VECTOR(1 DOWNTO 0);   -- rx_ctrldetect, two bytes
		GX_RDERR : IN STD_LOGIC_VECTOR(1 DOWNTO 0);    -- rx_errdetect, two bytes
		GX_DETECT : IN STD_LOGIC;   				   -- signal-detect from optical receiver, asynchronous
		-- comma aligner for ALTGX
		GX_ALIGN : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);   -- enable pattern align (edge-sensitive)
		GX_SYNCST : IN STD_LOGIC_VECTOR(3 DOWNTO 0);   -- rx_syncstatus, successful alignment status
		GX_COMMA : IN STD_LOGIC_VECTOR(3 DOWNTO 0)     -- rx_patterndetect, found comma in current boundary
	);
END tlk2501_receive;

ARCHITECTURE structure OF tlk2501_wrap IS
    -- note that this emulates both comma-alignment AND synchronization states for TLK2501
	TYPE state_values IS (algn, acq0, acq1, acq2, sync, check, cs0, cs1, cs2);
	-- Signals
	SIGNAL pres_state, next_state : state_values;
	SIGNAL rx_bits_in  : STD_LOGIC_VECTOR(1 DOWNTO 0);
	SIGNAL found_comma_trans : STD_LOGIC;
	SIGNAL lost_link_trans: STD_LOGIC;
	-- Signals that summarize the state of ALTGX receiver terminal
	SIGNAL data_invalid : STD_LOGIC;
	SIGNAL data_carrier : STD_LOGIC;
	SIGNAL data_error : STD_LOGIC;
	SIGNAL data_idle : STD_LOGIC;
	SIGNAL data_normal : STD_LOGIC;
	SIGNAL idle_or_carrier : STD_LOGIC;
	-- Signals to cache output to LSC/LDC
	SIGNAL RX_RD_SIG : STD_LOGIC_VECTOR(15 DOWNTO 0);
	SIGNAL RX_DV_SIG : STD_LOGIC;
	SIGNAL RX_ER_SIG : STD_LOGIC;
	
	-- Special character code values (rightmost bit is LSB)
	CONSTANT ONES8 : STD_LOGIC_VECTOR := "11111111";
	CONSTANT K23p7 : STD_LOGIC_VECTOR := "11110111";
	CONSTANT K28p5 : STD_LOGIC_VECTOR := "10111100";
	CONSTANT K30p7 : STD_LOGIC_VECTOR := "11111110";
	CONSTANT D5p6  : STD_LOGIC_VECTOR := "11000101";
	CONSTANT D16p2 : STD_LOGIC_VECTOR := "01010000";
	-- TLK-2501 keywords (rightmost byte is sent first)
	CONSTANT IDLE1 : STD_LOGIC_VECTOR   := D5p6 & K28p5;    --K25.8, D5.6
	CONSTANT IDLE2 : STD_LOGIC_VECTOR   := D16p2 & K28p5;   --K28.5, D16.2
	CONSTANT CARRIER : STD_LOGIC_VECTOR := K23p7 & K23p7;   --K23.7, K23.7
	CONSTANT GXERR : STD_LOGIC_VECTOR   := K30p7 & K30p7;   --K30.7, K30.7
	CONSTANT ONES16 : STD_LOGIC_VECTOR  := ONES8 & ONES8;   -- dummy all-ones pattern
BEGIN

-- parse the data on the receive terminal: data[16], control, and error status from ALTGX
data_invalid <= GX_RDERR(0) OR GX_RDERR(1);
data_carrier <= '1' WHEN (GX_RD = CARRIER AND GX_RDCTRL = "11") ELSE '0';
data_error <= '1' WHEN (GX_RD = GXERR AND GX_RDCTRL = "11") ELSE '0';
data_idle <= '1' WHEN ( (GX_RD = IDLE1 OR GX_RD = IDLE2) AND GX_RDCTRL = "10") ELSE '0';
data_normal <= NOT (data_carrier OR data_error OR data_idle);
idle_or_carrier <= data_idle OR data_carrier;
-- RX_DV & RX_ER
rx_bits <= "11" WHEN data_invalid ELSE (data_normal OR data_error) & (data_carrier OR data_error);

-- FSM register
STATE_REG : PROCESS (RX_CLK, RESET, next_state)
BEGIN
	IF (RESET = '1') THEN  -- async reset
		pres_state <= algn;
	ELSIF (RX_CLK = '1') and (RX_CLK'EVENT) THEN
		pres_state <= next_state;
	END IF;
END PROCESS STATE_REG;


-- FSM combinational block
--FIXME - this is incorrect: using parallel assignments inside PROCESS is not allowed!
FSM : PROCESS (pres_state, data_invalid, idle_or_carrier, found_comma_trans, lost_link_trans)
BEGIN
	CASE pres_state IS
		WHEN algn =>
			CASE found_comma_trans IS
				WHEN '1' => next_state <= acq0;
				WHEN '0' => next_state <= algn;
				WHEN OTHERS => NULL;
			END CASE;
		WHEN acq0 =>
			next_state <= acq0 WHEN data_invalid ELSE acq1 WHEN idle_or_carrier ELSE sync;
		WHEN acq1 =>
			next_state <= acq0 WHEN data_invalid ELSE acq2 WHEN idle_or_carrier ELSE sync;
		WHEN acq2 =>
			next_state <= acq0 WHEN data_invalid ELSE sync;
		WHEN sync =>
			next_state <= check WHEN data_invalid ELSE sync;
		WHEN check =>
			CASE lost_link IS
				WHEN '1' => next_state <= algn;
				WHEN '0' => next_state <= check WHEN data_invalid ELSE cs0;
				WHEN OTHERS => NULL;
			END CASE;
		WHEN cs0 =>
			next_state <= check WHEN data_invalid ELSE cs1;
		WHEN cs1 =>
			next_state <= check WHEN data_invalid ELSE cs2;
		WHEN cs2 =>
			next_state <= check WHEN data_invalid ELSE sync;
	END CASE;
END PROCESS FSM;

-- generate loss of link (received 3 invalid data patterns)
-- RX_CLK will be unavailable in case of severe signal loss
-- But this is OK because ALTGX 
LOSS_OF_LINK : process (RX_CLK, incr, pres_state)
  variable temp : integer;    
BEGIN
    if (ENABLE = '0' or FIBER_DISC = '1') then
      temp := 0;
    elsif (((GTX_CLK'event) and (GTX_CLK = '1')) and (incr = '1') and ((pres_state = check)or
                                                                         (pres_state = cs0) or (pres_state = cs1) or (pres_state = cs2))) then
      temp := temp + 1;
	elsif ((GTX_CLK'event) and (GTX_CLK = '1')) then -- AK fix - falling clock edge was erasing temp variable
	  temp := temp;
    else
      temp := 0;
    end if;
    if (temp >= 3) then
      lost_link_trans <= '1';
    else
      lost_link_trans <= '0';
    end if;
END PROCESS LOSS_OF_LINK;

-- FSM Moore output to LSC/LDC: RD, RX_DV, RX_ER
-- NOTE: we do not update RD[16] in accordance with TLK specs UNLESS RX_DV=1 and RX_ER=0
-- (TLK can set RD[16] to special patterns depending on RX_DV/LOS and RX_ER)
OUTPUTS_HOLA : PROCESS (pres_state, rx_bits, GX_RD)
BEGIN
	CASE pres_state IS
		WHEN sync =>
			RX_DV_SIG <= rx_bits(1);
			RX_ER_SIG <= rx_bits(0);
			RX_RD_SIG <= GX_RD;
		WHEN check =>
			RX_DV_SIG <= rx_bits(1);
			RX_ER_SIG <= rx_bits(0);
			RX_RD_SIG <= GX_RD;
		WHEN cs0 =>
			RX_DV_SIG <= rx_bits(1);
			RX_ER_SIG <= rx_bits(0);
			RX_RD_SIG <= GX_RD;
		WHEN cs1 =>
			RX_DV_SIG <= rx_bits(1);
			RX_ER_SIG <= rx_bits(0);
			RX_RD_SIG <= GX_RD;
		WHEN cs2 =>
			RX_DV_SIG <= rx_bits(1);
			RX_ER_SIG <= rx_bits(0);
			RX_RD_SIG <= GX_RD;
		WHEN OTHERS =>
			-- RX_ER and RX_DV are pulled high unless we are in an operating state (SYNC/CHECK)
			RX_RD_SIG <= ONES16;
			RX_DV_SIG <= '1';
			RX_ER_SIG <= '1';
	END CASE;
END PROCESS OUTPUTS_HOLA;

-- Control of ALTGX (comma alignment circuit)
OUTPUTS_ALTGX : PROCESS(pres_state)
BEGIN

END PROCESS OUTPUTS_ALTGX;

END structure;


LIBRARY work;
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY tlk2501_wrap IS
	PORT (
		--------------------------------------------------
		-- transmitter part
		-- signals from LSC core
		RESET : IN STD_LOGIC;
		TX_EN : IN STD_LOGIC;
		TX_ER : IN STD_LOGIC;
		TD    : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		-- signals to/from ALTGX, to be duplicated for DAQ and FTK channels
		TX_CLK : IN STD_LOGIC;                         -- coreclkout, combined bonded channel clock
		GX_TD : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);     -- datain, two bytes x DAQ+FTK
		GX_TDCTRL : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);  -- tx_ctrlenable, two bytes x DAQ+FTK
		--------------------------------------------------
		-- receiver part (for two channels: DAQ and FTK)
		-- signals to LSC core, DAQ and FTK
		RX_EN : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
		RX_DV : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
		RD   : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
		-- signals from/to ALTGX
		RX_CLK : IN STD_LOGIC_VECTOR(1 DOWNTO 0);      -- recovered clocks
		RX_LOCKED : IN STD_LOGIC_VECTOR(1 DOWNTO 0);   -- rx_freqlocked, per channel
		GX_RD : IN STD_LOGIC_VECTOR(31 DOWNTO 0);      -- datain, two bytes x two channels
		GX_RDCTRL : IN STD_LOGIC_VECTOR(3 DOWNTO 0);   -- rx_ctrldetect, two bytes x two channels
		GX_RDERR : IN STD_LOGIC_VECTOR(3 DOWNTO 0);    -- rx_errdetect, two bytes x two channels
		GX_DETECT : IN STD_LOGIC_VECTOR(1 DOWNTO 0);   -- signal-detect from optical receivers, per channel, async
		-- comma aligner for ALTGX
		GX_ALIGN : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);   -- enable pattern align (edge-sensitive), per channel
		GX_SYNCST : IN STD_LOGIC_VECTOR(3 DOWNTO 0);   -- rx_syncstatus, successful alignment status
		GX_COMMA : IN STD_LOGIC_VECTOR(3 DOWNTO 0)     -- rx_patterndetect, found comma in current boundary
	);
END tlk2501_wrap;

ARCHITECTURE structure OF tlk2501_wrap IS
	SIGNAL tx_bits : std_logic_vector(1 DOWNTO 0);   --TX_EN & TX_ER
BEGIN

-- Loss of sync FSM (ACQ,SYNC,CHECK,ACQ) - clocked at recovered RX_CLK
-- - we will be stuck in ACQ state until recovered clock is working, which is just what we want!

-- Register off FSM output from ALTGX with RXCLK. Before feeding DFF, pull DV/ER high when receiver has problems (as reported by ALTGX)
-- worry: freqlocked has jitter and cross-talk from other channel. Is protection in RETCH sufficient? Is it even doing what we think???
-- Note that signaldetect is unavailable in non-PCI modes

-------------------------------------------------------
-- Transmitter part
-------------------------------------------------------
-- We send identical data to DAQ and FTK
tx_bits <= TX_EN & TX_ER;
tx_process: PROCESS(tx_bits)
BEGIN
	CASE tx_bits IS
		WHEN "00" =>
			GX_TD <= IDLE1 & IDLE2;
			GX_TDCTRL <= "1010";
		WHEN "01" =>
			GX_TD <= CARRIER & CARRIER;
			GX_TDCTRL <= "1111";
		WHEN "10" =>
			GX_TD <= TD & TD;
			GX_TDCTRL <= "0000";
		WHEN "11" =>
			GX_TD <= GXERR & GXERR;
			GX_TDCTRL <= "1111";
		WHEN others =>
			null;
	END CASE;
END PROCESS;

-------------------------------------------------------
-- Receiver part
-------------------------------------------------------

END structure;
